
#######  This script accompanies the manuscript published in Ecosystem Services:
######  Branoff,Benjamin L., Cicchetti, G., Jackson, SJ, Pryor, M., Sharpe, L.M., Shumchenia, E., and and Susan H. Yee. (2023) A Matrix Approach to Quantify Spatio-temporal Trends in Ecosystem Services Capacity for Massachusetts Bays National Estuary Embayments

###  there are two scripts as part of the analysis
##  This first script prepares the spatial dataesets and modifies them according to the description in the analysis
##  In summary: 
##  1) collects the seagrass files from MassDEP and integrates them into the NOAA CCAP land cover data
##  2) reclassifies NOAA CCAP saltmarsh as either high or low elevation, or unclassified if no elevation data is available
##  3) cleans up the resulting dataset to remove duplicates and bad geometries
##  4) create rasters of the new landcover data, including MassDEP seagrass and slatmarsh elevation classes
##  5) from the rasters, create a new vector file of individual patches, with patches defined as any adjoining habitat of the same class
##  6) perform a land use land cover change analysis for each patch of habitat
##  7) perform a connectivity analysis

##  The results are stored in intermediate and output folders within the parent directory
##  the necessary input files are as follows:
##  1) MassBays extended embayment delineations from this same repository: https://osf.io/umrb3/
##  2) MassBays energy type delineations from this same repository: https://osf.io/umrb3/
##  3) seagrass delineations from MassDEP: https://www.mass.gov/info-details/massgis-data-massdep-eelgrass-mapping-project#massdep-sav-mapping-layers-
##  4) NOAA CCAP land cover data: https://coast.noaa.gov/digitalcoast/data/ccapregional.html
##  5) saltmarsh elevation classifications from the Saltmarsh Habitat and Avian Research Program:  https://www.tidalmarshbirds.org/?page_id=2168



###  Start with the original NOAA CCAP rasters
##   Remove any seagrass from these rasters and replace with open water
###  Then add in the MassDEP seagrass areas
library(raster)
library(sf)
library(stars)
## read in the seagrass files
seagrassfiles <- list.files(".\\Data\\Input\\eelgrass_YY_clipped\\",pattern=".shp",full.names=T)
seagrassfiles <- seagrassfiles[-grep(".xml|lock",seagrassfiles)]
###  get the original NOAA CCAP raster files
Rasters <- list.files("./Data/Input/NOAA CCAP",pattern=".img|.tif",recursive=T,full.names = T)
Rasters <- Rasters[grep("conus",Rasters)]
###  read in the embayment delineations
CoastComm_extended <- read_sf("./Data/Input/Coastal Communities Extended.shp")
CoastComm_extended <- st_transform(CoastComm_extended,raster(Rasters[1])@crs)
### setup the progress bar
pb <- txtProgressBar(min = 1, max = 5, style = 3)
###  create a table to compare the frequencies of the different LC classes
##  Just for troubleshooting later if needed
freqs <- matrix(nrow=0,ncol=3)
colnames(freqs) <- c("year","freq1","freq2")
for (y in c(1996,2001,2006,2010,2016)){
  ##  read in the seagrass files depending on the year
  if(y==1996){seagrass <-read_sf(seagrassfiles[grepl("_95",seagrassfiles)]) 
  }else if (y ==1951){
    seagrass <- read_sf(seagrassfiles[grepl("1950s",seagrassfiles)])
  ##  in 2010, the survey was incomplete, so supplement with 2012 data
  }else if (y ==2010){
    seagrass1 <- read_sf(seagrassfiles[grepl(substr(2012,3,4),seagrassfiles)])
    seagrass2 <- read_sf(seagrassfiles[grepl(substr(2010,3,4),seagrassfiles)])
    seagrass <- st_cast(rbind(seagrass1,seagrass2),"POLYGON")
    seagrass <- st_cast(st_sf(st_union(seagrass)),"POLYGON")
    colnames(seagrass) <- "geometry"
    st_geometry(seagrass) <- seagrass$geometry
  ## in 2016, the survey was incomplete, so supplement with 2015 and 2017
  }else if (y ==2016){
    seagrass1 <- read_sf(seagrassfiles[grepl(substr(2016,3,4),seagrassfiles)])
    seagrass2 <- read_sf(seagrassfiles[grepl(substr(2017,3,4),seagrassfiles)])
    seagrass3 <- read_sf(seagrassfiles[grepl(substr(2015,3,4),seagrassfiles)])
    seagrass <- st_cast(rbind(seagrass1,seagrass2,seagrass3),"POLYGON")
    seagrass <- st_cast(st_sf(st_union(seagrass)),"POLYGON")
    colnames(seagrass) <- "geometry"
    st_geometry(seagrass) <- seagrass$geometry
  }else{
    seagrass <- read_sf(seagrassfiles[grepl(substr(y,3,4),seagrassfiles)])
  }
  ##  read in the NOAA CCAP data
  r <- raster(Rasters[which(grepl(y,Rasters))])
  ##  crop and mask to the NEP area
  r <- crop(r,CoastComm_extended)
  r <- mask(r,CoastComm_extended)
  ##  make sure everything is in the same CRS
  seagrass <- st_transform(seagrass,r@crs)
  ## set the Class of the seagrass, same as the NOAA class for "submerged aquatic vegetation'
  seagrass$Class <- 23
  ##  calcualate the area
  seagrass$AREA_M <- st_area(seagrass)
  ###  set aside small patches. These are too small for the 30m resolution of the NOAA data
  ##  So they would not be registered upon conversion. 
  ##  We will add them back in later
  seagrass_small <- seagrass[as.numeric(seagrass$AREA_M)<=1000,]
  seagrass <- seagrass[as.numeric(seagrass$AREA_M)>1000,]
  ###  create a new copy of the NOAA raster
  rnew <- r
  ##  convert all the seagrass to open water
  rnew[rnew==23] <- 21
  ##  inverse mask with the seagrass delineations
  ##  converts any covered area to seagrass pixels
  rnew <- mask(rnew,seagrass,updatevalue=23,inverse=T)# mask(rnew,seagrass_ras,updatevalue=23,inverse=T)
  ##  save the new frequency numbers to the troubleshoot data set
  f <- data.frame(freq(rnew))
  count1 = f$count[which(f$value==23)]
  ### add back the smaller areas
  if(nrow(seagrass_small)>0){
    ##  find the center point of the small seagrass patch and convert it to a point
    seagrass_small <- st_centroid(seagrass_small)
    ##  add this point as a small 30m square to the larger data set
    seagrass_small <- SpatialPointsDataFrame(coords = st_coordinates(seagrass_small), data = data.frame(seagrass_small$Class), proj4string = crs(r))
    rnew <- rasterize(seagrass_small,rnew,field="seagrass_small.Class",update=T)
  }
  ##  recount the frequencies with the small areas
  f <- data.frame(freq(rnew))
  count2 = f$count[which(f$value==23)]
  freqs <- rbind(freqs,c(y,count1,count2))
  ##  convert the rasters to shapefiles
  patches <- st_as_stars(rnew) %>%
    st_as_sf(merge=T)
  patches$year = y
  colnames(patches)[1] <- "Class"
  if (y==1996){allpatches <- patches}else{allpatches <- rbind(allpatches,patches)}
  ### save the updated seagrass modified raster
  outfile <- gsub("Input/NOAA CCAP","Intermediate",Rasters[which(grepl(y,Rasters))])
  writeRaster(rnew,gsub(".tif","_seagrass_modified.tif",outfile),overwrite=T)
  Sys.sleep(0.1); setTxtProgressBar(pb,which(c(1996,2001,2006,2010,2016)==y))
}
write_sf(allpatches, "./Data/Intermediate/allpatches_seagrass_modified.shp")

### check the 'freqs' data for agreement, more or less
##  There will be some change due to differences in resolution, but should be minimal
freqs
remove(seagrass1,seagrass2,seagrass3,seagrass,seagrass_small,
       r,rnew,pb,count1,count2,outfile,y,freqs,patches)


####  Now reclassify saltmarsh as high or low elevation
####  This is done with the SHARP data on salt marshes in New England
##    https://www.tidalmarshbirds.org/?page_id=2168
###   This data is sometimes informed by digital elevation models (DEM) and in some cases without DEM
###  We take both for now and remove the duplicates later, prioritizing the DEM data when available
###  the data also comes in two "zones", so we have to figure out where each zone overlaps with our data
###  just in case, make the allpatches valid
library(stars)
library(raster)
library(sf)
library(dplyr)
allpatches <- read_sf( "./Data/Intermediate/allpatches_seagrass_modified.shp")
allpatches <- st_make_valid(allpatches)
allpatches$id <- 1:nrow(allpatches)
allbbox <- st_as_sf(st_as_sfc(st_bbox(allpatches)))
st_crs(allbbox) <- st_crs(allpatches)
Zone1DEM <- raster(".\\Data\\Input\\Marsh Classifications\\Z_1\\Z1_DEM.tif")
###  set the NA value
NAvalue(Zone1DEM) <- 0
###   crop to the area of interest
Zone1DEM <- crop(Zone1DEM,st_transform(allbbox,Zone1DEM@crs))
##  additional trim
Zone1DEM <- crop(Zone1DEM, extent(Zone1DEM, 1, 9629, 1000, 9163),filename=paste0(tempdir(),"\\Zone1DEM.tif"),overwrite=T)
Zone1DEM_bbox <- st_as_sf(st_as_sfc(st_bbox(Zone1DEM)),crs=Zone1DEM@crs)
Zone1DEM_bbox$DEM <- "YES";Zone1DEM_bbox$Zone <- 1
###  Do the same for the other marsh data sets
Zone1NoDEM <- raster(".\\Data\\Input\\Marsh Classifications\\Z_1\\Zone1_NoDEM.tif")
NAvalue(Zone1NoDEM) <- 0
Zone1NoDEM <- crop(Zone1NoDEM, extent(Zone1NoDEM, 9000, 22500, 2500, 8500),filename=paste0(tempdir(),"\\Zone1NoDEM.tif"),overwrite=T)
Zone1NoDEM_bbox <- st_as_sf(st_as_sfc(st_bbox(Zone1NoDEM)),crs=Zone1NoDEM@crs)
Zone1NoDEM_bbox$DEM <- "NO";Zone1NoDEM_bbox$Zone <- 1
Zone2DEM <- raster(".\\Data\\Input\\Marsh Classifications\\Z_2\\Zone2_DEM.tif")
NAvalue(Zone2DEM) <- 0
Zone2DEM <- crop(Zone2DEM,st_transform(allbbox,Zone2DEM@crs),filename=paste0(tempdir(),"\\Zone2DEM.tif"),overwrite=T)
Zone2DEM_bbox <- st_as_sf(st_as_sfc(st_bbox(Zone2DEM)),crs=Zone2DEM@crs)
Zone2DEM_bbox$DEM <- "YES";Zone2DEM_bbox$Zone <- 2

Zone2NoDEM <- raster(".\\Data\\Input\\Marsh Classifications\\Z_2\\Zone2_NoDEM.tif")
NAvalue(Zone2NoDEM) <- 0
Zone2NoDEM_bbox <- st_as_sf(st_as_sfc(st_bbox(Zone2NoDEM)),crs=Zone2NoDEM@crs)
Zone2NoDEM_bbox$DEM <- "NO";Zone2NoDEM_bbox$Zone <- 2
## the SHARP data is too high resolution for the NOAA data
##aggregate it to a lower resolution
#Zone2DEM <- aggregate(Zone2DEM, fact = 10, fun = modal, na.rm = TRUE)
#Zone1DEM <- aggregate(Zone1DEM, fact = 10, fun = modal, na.rm = TRUE)
#Zone2NoDEM <- aggregate(Zone2NoDEM, fact = 10, fun = modal, na.rm = TRUE)
#Zone1NoDEM <- aggregate(Zone1NoDEM, fact = 10, fun = modal, na.rm = TRUE)
###  also need a reference raster to match the new raster too (align the cells)
refRas <- raster(".\\Data\\Input\\NOAA CCAP\\conus_1996_ccap_landcover_20200311.tif")
## add this information to the patches
allpatches2 <- rbind(st_intersection(allpatches[allpatches$Class==18,],st_transform(Zone1DEM_bbox[,c("DEM","Zone")],st_crs(allpatches))),
                     st_intersection(allpatches[allpatches$Class==18,],st_transform(Zone1NoDEM_bbox[,c("DEM","Zone")],st_crs(allpatches))),
                     st_intersection(allpatches[allpatches$Class==18,],st_transform(Zone2DEM_bbox[,c("DEM","Zone")],st_crs(allpatches))),
                     st_intersection(allpatches[allpatches$Class==18,],st_transform(Zone2NoDEM_bbox[,c("DEM","Zone")],st_crs(allpatches))))
###  find out which areas overlap with multiple SHARP data sets
allpatches2$geom <- as.character(allpatches2$geometry)
allpatches2$geom <- substr(allpatches2$geom,1,25)
allpatches2 <- allpatches2 %>%
  group_by(id) %>% 
  mutate(id2=cur_group_id()) %>%
  mutate(n=n())
##  remove the areas that overlap and have no DEM
allpatches2 <- allpatches2[-which(allpatches2$n>1&allpatches2$DEM=="NO"),]
##  the remaining overlap areas are shared between the two zones
##  one may be better (more classifications) but we wont know until we read the two data sets
allpatches2 <- allpatches2 %>%
  group_by(id) %>% 
  mutate(id2=cur_group_id()) %>%
  mutate(n=n())
### in case the is repeated, remove the previous dem_st_all data
remove(dem_st_all)
###  to keep track of areas, store them in a data frame
Areas <- data.frame(Zone=NULL, DEM=NULL,g=NULL,Source=NULL,area=NULL)
###   for each of the SHAPR data sets, overlay with the NOAA marshes and apply the elevation classification
###  This is only done where marsh exists in the NOAA data
for (r in c(Zone1DEM,Zone2DEM,Zone1NoDEM,Zone2NoDEM)){
  if (grepl("1",names(r))){Zone=1}else{Zone=2}
  if (grepl("noDEM|NoDEM",r@file@name)){dem="NO"}else{dem="YES"}
  ##  get the bounding box for the current zone
  bbx <-st_as_sfc(st_bbox(r));st_crs(bbx) <- crs(r)
  ####  divide it into grids, otherwise the area is too big for the computer
  grd <- st_make_grid(bbx,n=10) %>% # the grid, covering bounding box
    st_sf()
  ## initialize the progress bar
  pb <- txtProgressBar(min = 1, max = nrow(grd), style = 3)
   ##  transform allpatches to the grid, and thus to the Zone CRS
  allpatches <-st_transform(allpatches2[allpatches2$Zone==Zone&allpatches2$DEM==dem,],st_crs(grd))
  ##  for each of the grid boxes, overlay the marsh elevation data and apply the elevation classification
  for (g in 1:nrow(grd)){
    ##  crop the NOAA marshes to the current grid box
    patches <- st_crop(allpatches,grd[g,])
    ## if there are any marshes in this area, get the elevation data
    if (nrow(patches)>0){
      ## crop the elevation data to this area
      ##  dont mask, takes too long and unnecessary
      r2 <- crop(r,patches)
      refRas2 <- crop(refRas,st_transform(patches,refRas@crs))
      r2 <- projectRaster(r2,crs=refRas2@crs,method='ngb',na.rm=T)
      r2 <- resample(r2,refRas2,method="ngb",na.rm=T)
      ##  to avoid issues with NA, convert to a non-use number 
      r2[is.na(r2)] <- 10
      ##  if there are more than one marsh elevation classifications, split up the NOAA data into the different parts
      if (length(unique(getValues(r2)))>1){
        ##  convert the marsh classification raster to a shapefile
        dem_st <- st_as_stars(r2,merge=T) %>% st_as_sf(merge=T);colnames(dem_st)[1] <- "type"
        ##  make sure its valid
        dem_st <- st_make_valid(dem_st)
        ##  intersect it with the NOAA Marshes and apply the "type" field
        dem_st <- st_transform(dem_st,st_crs(patches))
        dem_st <- st_snap(dem_st,patches,2)
        dem_st <- st_intersection(patches,dem_st[,"type"])
        dem_st <- st_collection_extract(dem_st,"POLYGON")
        dem_st$area <- as.numeric(st_area(dem_st))
        Areas <- rbind(Areas,
                       data.frame(Zone=Zone,DEM=dem,g=g,area.orig = sum(as.numeric(st_area(patches))),
                                  area.int = sum(as.numeric(st_area(dem_st)))))
        #  remove invalid geometries
        dem_st <- st_collection_extract(dem_st,type="POLYGON")
        ##  convert the 10s back to NA
        dem_st$type[dem_st$type==10]<-NA
        ##  convert the elevation types into a NOAA class
        dem_st$Class[is.na(dem_st$type)] <- 18
        dem_st$Class[dem_st$type %in% c(1,9,8)] <- 18.1 ## high marsh
        dem_st$Class[dem_st$type %in% c(2,4,6,7)] <- 18.2 ## low marsh
        dem_st$Class[dem_st$type %in% c(5)] <- 18.3  #phragmites
        dem_st$area <- as.numeric(st_area(dem_st))
        dem_st$Class <- factor(dem_st$Class)
        ##  dissolve internal boundaries
        dem_st <- dem_st %>%
          group_by(id, year, Class,DEM,Zone) %>%
          summarize(area=sum(area)) %>%
          st_cast("MULTIPOLYGON")
        ##  bind the data to the running dataset
        if (nrow(dem_st)>0){
          if (exists("dem_st_all")){dem_st <- st_transform(dem_st,st_crs(dem_st_all));dem_st_all <- rbind(dem_st_all,dem_st)
          }else{dem_st_all <- dem_st}
        }
      ## if there is only one marsh classification, no need to split the NOAA data, just assign the one classification
      }else{
        dem_st <- patches
        dem_st$type <- unique(getValues(r2))
        dem_st$type[dem_st$type==10]<-NA
        ##  convert the elevation types into a NOAA class
        dem_st$Class[is.na(dem_st$type)] <- 18
        dem_st$Class[dem_st$type %in% c(1,9,8)] <- 18.1 ## high elevation
        dem_st$Class[dem_st$type %in% c(2,4,6,7)] <- 18.2 ##low elevation
        dem_st$Class[dem_st$type %in% c(5)] <- 18.3 #phrag
        dem_st$area <- as.numeric(st_area(dem_st))
        ##  dissolve internal boundaries
        dem_st <- dem_st %>%
          group_by(id, year, Class,DEM,Zone) %>%
          summarize(area=sum(area)) %>%
          st_cast("MULTIPOLYGON")
        if (exists("dem_st_all")){dem_st <- st_transform(dem_st,st_crs(dem_st_all));dem_st_all <- rbind(dem_st_all,dem_st)
        }else{dem_st_all <- dem_st}
      }
    }
    Sys.sleep(0.1); setTxtProgressBar(pb,g)
  }
  gc()
}
close(pb)
## check to make sure there are no missing ids
which(!unique(allpatches2$id[allpatches2$Class==18]) %in% unique(dem_st_all$id))
##  remove the areas that are too small (smaller than a pixel)
##  These will be added back in later
dem_st_all <- dem_st_all[as.numeric(st_area(dem_st_all))>=900,]
write_sf(dem_st_all,"./Data/Intermediate/dem_st_all.shp",delete_layer = T)
remove(dem_st,r,r2,grd,bbx,pb,Zone1DEM,Zone1NoDEM,Zone2DEM,Zone2NoDEM,
       patches,g,tmpfiles,allbbox,allpatches, allpatches2,
       Zone1DEM_bbox,Zone1NoDEM_bbox,Zone2DEM_bbox,Zone2NoDEM_bbox,
       area,Areas,refRas,refRas2,Zone,dem)
##  this process creates a lot temporary rasters which can quickly build up
##  delete those now that they are no longer needed
tmpfiles <- list.files(paste0(tempdir(),"\\raster\\"),pattern=".grd|.gri",full.names=T)
file.remove(tmpfiles)

####   With the marsh areas now classified according to elevation
####  check areas and remove duplicates
library(dplyr)
library(sf)
library(raster)

allpatches <- read_sf("./Data/Intermediate/allpatches_seagrass_modified.shp")
allpatches <- st_make_valid(allpatches)
allpatches <- st_transform(allpatches,st_crs(dem_st_all))
allpatches$id <- 1:nrow(allpatches)

dem_st_all <-st_cast(dem_st_all,"POLYGON")
dem_st_all <- dem_st_all[as.numeric(st_area(dem_st_all))>1,]

##  find duplicates
##  This happens because there was a DEM and an NoDEM data set
##  also because there is some overlap between zone 1 and zone 2
##  We only want one copy of each marsh patch
##  first check the areas against the original marsh delineations
allpatches$area <- st_area(allpatches)
allpatches_Agg <- aggregate(data=st_drop_geometry(allpatches[allpatches$Class==18,]),area~id,FUN=sum)
dem_st_all$area <- as.numeric(st_area(dem_st_all))
dem_st_agg <- aggregate(data=st_drop_geometry(dem_st_all),area~id+DEM+Zone,FUN=sum)
dem_st_agg <- merge(dem_st_agg,allpatches_Agg[,c("id","area")],by="id",all=T)
dem_st_agg$diff <- as.numeric(round(dem_st_agg$area.x/dem_st_agg$area.y,1))
dem_st_agg <- dem_st_agg[order(-dem_st_agg$diff),]
###  first remove duplicates from the two zones, if one has the correct area
## identify the patches that are duplicated and represent the whole original patch
IDs <- unique(dem_st_agg$id[which(duplicated(dem_st_agg$id))])
IDs <- unique(dem_st_agg$id[which(dem_st_agg$id %in% IDs&dem_st_agg$diff==1)])
##  Now identify the incorrect entries for these patches
dem_st_agg$idZone <- paste(dem_st_agg$id,dem_st_agg$Zone,sep="_")
Zones <- dem_st_agg$idZone[which(dem_st_agg$id %in% IDs&dem_st_agg$diff!=1)]
##  remove the bad zone
dem_st_all2 <- dem_st_all[-which(paste(dem_st_all$id,dem_st_all$Zone,sep="_") %in% Zones),]
##  aggregate again to get the areas
dem_st_agg <- aggregate(data=st_drop_geometry(dem_st_all2),area~id+DEM+Zone,FUN=sum)
dem_st_agg <- merge(dem_st_agg,allpatches_Agg[,c("id","area")],by="id",all=T)
dem_st_agg$diff <- as.numeric(round(dem_st_agg$area.x/dem_st_agg$area.y,1))
###  Some of the remaining duplicates should have the same area from both zones. So just choose the first one
## identify the patches that are duplicated and represent the whole original patch
IDs <- unique(dem_st_agg$id[which(duplicated(dem_st_agg$id)&dem_st_agg$diff==1)])
dem_st_all2 <- dem_st_all2[-which(dem_st_all2$id %in% IDs&dem_st_all2$Zone==2),]
###  check again the area comparisons
dem_st_agg <- aggregate(data=st_drop_geometry(dem_st_all2),area~id+DEM+Zone,FUN=sum,na.action=na.omit)
dem_st_agg <- merge(dem_st_agg,allpatches_Agg[,c("id","area")],by="id",all=T)
dem_st_agg$diff <- as.numeric(round(dem_st_agg$area.x/dem_st_agg$area.y,1))
##  Some of the remaining duplicates are split between the two zones
##  So combine them
IDs <- unique(dem_st_agg$id[which(duplicated(dem_st_agg$id))])
dem_st_agg <- aggregate(data=st_drop_geometry(dem_st_all2[dem_st_all2$id %in% IDs,]),
                        area~id+DEM,FUN=sum,na.action=na.omit)
dem_st_agg <- merge(dem_st_agg,allpatches_Agg[,c("id","area")],by="id",all.x=T)
dem_st_agg$diff <- as.numeric(round(dem_st_agg$area.x/dem_st_agg$area.y,1))
IDs <- unique(dem_st_agg$id[which(dem_st_agg$diff==1)])
dem_st_all3 <- dem_st_all[dem_st_all$id %in% IDs,] %>%
  group_by(id,year,Class,DEM) %>% summarize(area=sum(area))
dem_st_all3$Zone <- "1-2"  
dem_st_all2 <- rbind(dem_st_all2[!dem_st_all2$id %in% IDs,],
                     dem_st_all3[,names(dem_st_all2)])
##  check areas again
dem_st_agg <- aggregate(data=st_drop_geometry(dem_st_all2),
                        area~id+DEM,FUN=sum,na.action=na.omit)
dem_st_agg <- merge(dem_st_agg,allpatches_Agg[,c("id","area")],by="id",all=T)
dem_st_agg$diff <- as.numeric(round(dem_st_agg$area.x/dem_st_agg$area.y,1))
##  There are a lot of missing patches from the new data, their area shows up as NA in dem_st_agg
##   add those back in
IDs <- dem_st_agg$id[is.na(dem_st_agg$area.x)]
patches <- allpatches[allpatches$id %in% IDs,]
patches$DEM <- 3; patches$Zone <- 3
dem_st_all2 <- rbind(dem_st_all2,patches[,names(dem_st_all2)])
dem_st_all2$area <- st_area(dem_st_all2)
##  check areas again
dem_st_agg <- aggregate(data=st_drop_geometry(dem_st_all2),
                        area~id+DEM,FUN=sum,na.action=na.omit)
dem_st_agg <- merge(dem_st_agg,allpatches_Agg[,c("id","area")],by="id",all=T)
dem_st_agg$diff <- as.numeric(round(dem_st_agg$area.x/dem_st_agg$area.y,1))
summary(as.numeric(st_area(st_cast(dem_st_all2,"POLYGON"))))

### the remaining patches are either perfect,missing entirely or missing some sections
## this means they were outside fo the SHARP data
##  so add in the missing areas as NA types
IDs <- unique(dem_st_agg$id[which(dem_st_agg$diff<0.9)])
remove(patches)
pb <-  txtProgressBar(min = 1, max = length(IDs), style = 3)
for (i in IDs){
  patch1 <- allpatches[allpatches$id==i,]
  patch1 <- patch1 %>%
    summarise(geometry = st_union(geometry)) %>%
    st_buffer(0.1)
  patch2 <- dem_st_all2[dem_st_all2$id==i,] %>%
    st_buffer(0.1)
  patch2 <- st_difference(patch1,patch2)
  area1 <- sum(as.numeric(st_area(patch2)))
  ##  the difference creates slivers
  ## so remove those
  patch2 <- st_cast(patch2,"POLYGON")
  patch2 <- patch2[as.numeric(st_area(patch2))>1,]
  patch2 <- patch2 %>% 
    group_by(id,year,Class,DEM,Zone) %>%
    summarise(area=sum(area))
  area2 <- sum(as.numeric(st_area(patch2)))
  if (area2/area1>1.1){broswer()}
  if (exists("patches")){patches <- rbind(patches,patch2)}else{patches<-patch2}
  Sys.sleep(0.1); setTxtProgressBar(pb,which(IDs==i))
}
## add the missing areas back in
dem_st_all2 <- rbind(dem_st_all2,patches[,names(dem_st_all2)])
dem_st_all2$area <- st_area(dem_st_all2)

##  check areas again
dem_st_agg <- aggregate(data=st_drop_geometry(dem_st_all2),area~id,FUN=sum,na.action=na.omit)
dem_st_agg <- merge(dem_st_agg,allpatches_Agg[,c("id","area")],by="id",all=T)
dem_st_agg$area.x[is.na(dem_st_agg$area.x)] <- 0
dem_st_agg$diff <- as.numeric(round(dem_st_agg$area.x/dem_st_agg$area.y,1))
dem_st_all2 <- st_cast(dem_st_all2,"POLYGON")
summary(as.numeric(st_area(dem_st_all2)))
## there are no missing ids
which(!unique(dem_st_agg$id %in% unique(allpatches$id)))
##  all patches now match areas form the original
## remove the original marsh areas from the NOAA data
##  replace with the marsh areas now separated by elevation type
allpatches2 <- allpatches[-which(allpatches$Class==18),]

allpatches2 <- rbind(allpatches2,dem_st_all2[,names(allpatches2)])

st_write(allpatches2,"./Data/Intermediate/allpatches_seagrass_saltmarsh_modified.shp",delete_layer=T)
remove(allpatches,allpatches2,allbbox,allpatches_Agg,dem_st_agg,dem_st_all,dem_st_all2,
       dem_st_all3,diff,missing, patch,patch_buf,patch_cast,patch1,patch2,
       patches,pb,test,dem,IDs,i,int,Zone,Zones,patches2,dem_st_all3,area1,area2,IDs2)
gc()

###  now rasterize 
library(fasterize)
library(raster)
library(sf)
##  read in the original rasters
##  these are needed to provide a template for the new raster
Rasters <- list.files("./Data/Intermediate/",pattern=".img|.tif",recursive=T,full.names = T)
Rasters <- Rasters[which(grepl("seagrass",Rasters))]
###  use the embyament boundaries to crop the datasets
##   no need to do the whole state 
CoastComm_extended <- read_sf("./Data/Input/Coastal Communities Extended.shp")
CoastComm_extended <- st_transform(CoastComm_extended,raster(Rasters[2])@crs)
##  read in the newly created shapefile
##  Each feature in the shapefile will be converted to a group of raster pixels
allpatches <- read_sf("./Data/Intermediate/allpatches_seagrass_saltmarsh_modified.shp")
pb <-  txtProgressBar(min = 1996, max = 2016, style = 3)
for (y in c(1996,2001,2006,2010,2016)){   
  r <- raster(Rasters[which(grepl(y,Rasters))])
  r <- crop(r,CoastComm_extended)
  r <- mask(r,CoastComm_extended)
  patch <- allpatches[which(allpatches$year==y),]
  patch <- st_cast(patch,"POLYGON")
  patch <- st_transform(patch,r@crs)
  patch$Class <-round(as.numeric(patch$Class),1)
  r_patch <- fasterize(patch,r,field="Class")
  r_new <- overlay(r,r_patch, fun = function(x, y) {
    x[!is.na(y[])] <- y[!is.na(y)]
    return(x)
  })
  writeRaster(r_new,paste0("./Data/Output/conus_",y,"_ccap_landcover_20200311_seagrass_saltmarsh_modified.tif"),overwrite=T)
  Sys.sleep(0.1); setTxtProgressBar(pb,y)
}
remove(allpatches,patch,pb,r,r_new,r_patch)
######################################################
###  Now using the rasters from above convert back to shapefiles and 
###  join all touching patches of the same habitat, 
##   and assign new IDs to every patch
######################################################
library(sf)
library(dplyr)
library(igraph)
library(stars)
library(raster)
##  read in the rasters from above
Rasters <- list.files("./Data/Output/",pattern=".img|.tif",recursive=T,full.names = T)
remove(allpatches_dissolved)
for (y in c(1996,2001,2006,2010,2016)){
  allpatches <- st_as_sf(st_as_stars(raster(Rasters[which(grepl(y,Rasters))])))
  colnames(allpatches)[1] <- 'Class'
  allpatches$Class <- round(allpatches$Class,1)
  pb <-  txtProgressBar(min = 1, max = length(unique(allpatches$Class)), style = 3)
  for (C in unique(allpatches$Class)){
    ##  subset to the Class in question
    ##  could try to do it all at once with a smaller data set
    ##  but this would prob overload most computers
    patch <- allpatches[allpatches$Class==C,]
    patch <- st_make_valid(patch)
    patch$area <- as.numeric(st_area(patch))
    ##  separate the patches into their individual parts
    patch <- patch %>%
      summarise(area = sum(area)) %>%
      st_cast("POLYGON")
    ##  find out which patches are touching
    int <- st_touches(patch)
    ##  group together the touching patches with a unique membership ID
    int <- graph_from_adj_list(int)
    int <- components(int)$membership
    ## combine all the touching patches together
    patch <- cbind(patch,int) %>%
      group_by(int) %>%
      summarise(area = sum(area)) %>%
      st_cast()
    ## make sure none of the areas are smaller than 1
    if (any(as.numeric(st_area(st_cast(patch,'POLYGON')))<1)){browser()}
    patch$Class <- C
    patch$year <- y
    ##  add the new combined features to a new data base
    if (exists("allpatches_dissolved")){allpatches_dissolved <- rbind(allpatches_dissolved,patch)}else{allpatches_dissolved <- patch}
    Sys.sleep(0.1); setTxtProgressBar(pb,which(unique(allpatches$Class)==C))
    gc()
  }
  write_sf(allpatches_dissolved,paste0("./Data/Intermediate/allpatches",y,"_seagrass_saltmarsh_modified_dissolved.shp")
           ,delete_layer = T)
  remove(allpatches_dissolved)
}
##  join all of the years together
for (y in c(1996,2001,2006,2010,2016)){
  patch <- read_sf(paste0("./Data/Intermediate/allpatches",y,"_seagrass_saltmarsh_modified_dissolved.shp"))
  if (y==1996){allpatches <- patch}else{allpatches <- rbind(allpatches,patch)}
}
##  add a unique ID to all patches and save the final data
allpatches <- allpatches[order(allpatches$year,allpatches$Class),]
allpatches$patchID <- 1:nrow(allpatches)
allpatches$area <- as.numeric(st_area(allpatches))
ecotypes <- read_sf("./Data/Input/embayment_units_ecotypes/massbays_estuarine_embays_2021_MODIFIED.shp")
ecotypes <- st_make_valid(ecotypes)
ecotypes <- st_transform(ecotypes,st_crs(allpatches))
ecotypes$ecotype[ecotypes$ecotype2=='Yellow'] <- 'Low Energy'
ecotypes$ecotype[ecotypes$ecotype2=='Orange'] <- 'Low-Med. Energy'
ecotypes$ecotype[ecotypes$ecotype2=='Green'] <- 'Med.-Hig. Energy'
ecotypes$ecotype[ecotypes$ecotype2=='Blue'] <- 'Hig. Energy'
ecotypes$ecotype[ecotypes$ecotype2=='undefined'] <- 'Undefined'
##  do the intersection
allpatches <- st_intersection(allpatches,ecotypes[,c("NAME",'ecotype','ecotype2','mb_region2')],all=T)
##  calculate the new area, this is the area of the patch inside each embayment
allpatches$areaLocal <- as.numeric(st_area(allpatches))


write_sf(allpatches,"./Data/Output/allpatches_seagrass_saltmarsh_modified_IDs.shp",delete_layer = T)


#########################################
###   lulcc analysis ######################
###########################################
library(raster)
library(sf)
library(exactextractr)
library(dplyr)
library(tidyr)

##  read in the final patch delineations
allpatches <- read_sf("./Data/Output/allpatches_seagrass_saltmarsh_modified_IDs.shp")
## only interested in the 1996 focal habitat areas
##  as well as any other habitat in subsequent years that was convereted to a focal habitat
#patches96 <- which(allpatches$year==1996&!allpatches$Class %in% c(18,18.1,18.2,18.3,19,23))
#allpatches <- allpatches[-patches96,]
## makes sure the 1996 patches are only focal ecosystems
unique(allpatches$Class[allpatches$year==1996])
##  intersect with the embayment names 
#ecotypes <- read_sf("./Data/Input/embayment_units_ecotypes/massbays_estuarine_embays_2021_MODIFIED.shp")
#ecotypes <- st_make_valid(ecotypes)
#ecotypes <- st_transform(ecotypes,st_crs(allpatches))
#allpatches <- st_intersection(allpatches,ecotypes[,"NAME"],all=T)
#allpatches$areaLocal <- as.numeric(st_area(allpatches))
allpatches <- allpatches[allpatches$areaLocal>1,]
allpatches <- st_make_valid(allpatches)
## convert phragmites to unclassified saltmarsh
allpatches$Class[allpatches$Class==18.3] <- 18
##  get the annual rasters
Rasters <- list.files("./Data/Output/",pattern="saltmarsh_modified.tif",recursive=T,full.names = T)
detach("package:plyr", unload=TRUE)
###  for each annual raster
##  how do the frequencies of land cover classes change
##  within the original 1996 areas
years <- c(1996,2001,2006,2010,2016)
for (y in years){
  r <- raster(Rasters[which(grepl(y,Rasters))])
  r <- reclassify(r,cbind(18.28,18.32,18))
  allpatches <- st_transform(allpatches,r@crs)
  ##  function to calculate the frequency of every cell class in each patch
  if (y==1996){
    freqs <- exact_extract(r, allpatches[allpatches$year==1996,],include_cols=c("patchID","year","NAME","Class","area","areaLocal"),coverage_area=T)
  ##  because of the incomplete seagrass sampling in 2006, need to calculate change for 2001 to 2010 as well as 2006 to 2010
  #}else if (y==2010){
  #  freqs1 <- exact_extract(r, allpatches[allpatches$year==1996,],include_cols=c("patchID","year","NAME","Class","area","areaLocal"),coverage_area=T)
  #  freqs2 <- exact_extract(r, allpatches[allpatches$year%in%c(2001,2006),],include_cols=c("patchID","year","NAME","Class","area","areaLocal"),coverage_area=T)
  #  freqs <- c(freqs1, freqs2)
  }else{
    freqs <- exact_extract(r, allpatches[allpatches$year %in% c(1996,years[which(years==y)-1]),],include_cols=c("patchID","year","NAME","Class","area","areaLocal"),coverage_area=T)
  }
  ##  combine all the patches
  freqs2 <- do.call("rbind", freqs)
  freqs2$value <- round(freqs2$value,1)
  freqs2 <- freqs2[which(!is.na(freqs2$value)),] %>%
    group_by(patchID,year,NAME,Class,value,area,areaLocal) %>%
    summarise(coverage_area=sum(coverage_area,na.rm=T))
  ##  we are going to remove the non focal habitat conversions
  ##  but we still need to know the total area converted for each class in each embayment for a later step
  #conversions <- bind_rows(freqs2 %>%
  #  group_by(patchID,year,NAME,Class) %>%
  #  mutate(lost_area_T=sum(coverage_area[Class!=value],na.rm=T)),
  #  freqs2 %>%
  #    group_by(patchID,year,NAME,value) %>%
  #    mutate(added_area_T=sum(coverage_area[Class!=value],na.rm=T),
  #           Class=value)) %>%
  #  distinct(patchID,Class,value,year,NAME,.keep_all = T)%>%
  #  group_by(Class,value,year,NAME)%>%
  #  summarise(lost_area_T=sum(lost_area_T,na.rm=T),
  #            added_area_T=sum(added_area_T,na.rm=T))
  # conversions <- merge(freqs2 %>%
  #                        group_by(year,NAME,Class,value) %>%
  #                        summarise(converted_area_T=sum(coverage_area[Class!=value],na.rm=T)),
  #                      freqs2 %>%
  #                        group_by(year,NAME,value,Class) %>%
  #                        summarise(added_area_T=sum(coverage_area[Class!=value],na.rm=T)),
  #                      by.x=c("year","NAME","Class","value"),by.y=c("year","NAME","value","Class"),
  #                      all=T)
  conversions <- freqs2 %>%
    group_by(year,NAME,Class,value) %>%
    summarise(converted_area_T=sum(coverage_area[Class!=value],na.rm=T))
  #conversions$added_area_T[is.na(conversions$added_area_T)] <- 0
  #conversions$lost_area_T[is.na(conversions$lost_area_T)] <- 0
  #conversions$converted_area_T=conversions$added_area_T+conversions$lost_area_T
  ##  add the total converted area to the individual class conversions
  freqs2 <- merge(freqs2,conversions,by=c("year","NAME","Class","value"),all=T)
  ## separate the focal conversions from the non-focal conversions
  freqs2_nofocal <- freqs2[!freqs2$Class %in% c(18,18.1,18.2,18.3,19,23)&!freqs2$value %in% c(18,18.1,18.2,18.3,19,23),]
  freqs2_nofocal1 <- freqs2_nofocal %>%
    group_by(NAME,Class,year) %>%
    mutate(converted_area_T=ifelse(value==Class,0,sum(converted_area_T[Class!=value&!is.na(value)&!duplicated(value)],na.rm=T)),
           value=ifelse(value==Class,value,28)) %>%
    group_by(NAME,patchID,Class,value,year) %>%
    summarise(area=first(area),
              areaLocal=first(areaLocal),
              coverage_area=sum(coverage_area),
              converted_area_T=first(converted_area_T)) %>%
    group_by(NAME,Class,value,year) %>%
    summarise(area=sum(area),
              areaLocal=sum(areaLocal),
              coverage_area=sum(coverage_area),
              converted_area_T=first(converted_area_T)) %>% ungroup()
  freqs2_nofocal2 <- freqs2_nofocal %>%
    filter(Class!=value) %>%
    group_by(NAME,value,year) %>%
    mutate(converted_area_T=sum(converted_area_T[!is.na(Class)&!duplicated(Class)],na.rm=T),
           Class=28) %>%
    group_by(NAME,Class,value,year) %>%
    summarise(area=sum(area),
              areaLocal=sum(areaLocal),
              coverage_area=sum(coverage_area),
              converted_area_T=first(converted_area_T))%>% ungroup()
  freqs2_nofocal <- bind_rows(freqs2_nofocal1,freqs2_nofocal2)
  ##  we're only interested when the focal ecosystems are involved
  ## remove the others, but keep the value containing the total converted area
  ##  this will be used later to estimate connectivity influences
  freqs3 <- bind_rows(freqs2[freqs2$Class %in% c(18,18.1,18.2,18.3,19,23)|freqs2$value %in% c(18,18.1,18.2,18.3,19,23),],
                  freqs2_nofocal)
  freqs3 <- freqs3 %>%
    group_by(patchID,NAME,Class,year) %>%
    mutate(n=n()) %>%
    filter((n==1)|(n>1&value!=28)|(n>1&value==28&coverage_area!=0))
  ##  aggregate the patches that have no interaction with focal systems
  freqs3 <- bind_rows(freqs3 %>% filter(n>1|value!=28),
                      freqs3 %>% filter(n==1&value==28)%>%
                        group_by(NAME,Class,year,value) %>%
                        summarise(area=sum(area,na.rm=T),
                                  areaLocal=sum(areaLocal,na.rm=T),
                                  #added_area_T=sum(added_area_T,na.rm=T),
                                  #lost_area_T=sum(lost_area_T,na.rm=T),
                                  converted_area_T=max(converted_area_T,na.rm=T),
                                  coverage_area =sum(coverage_area,na.rm=T)))
  ## add the year and save
  freqs3$Rasteryear=y
  write.csv(freqs3,paste0("./Data/Output/lulcc_patches",y,".csv"),row.names=F)
}
remove(freqs,freqs2,allpatches,r,Rasters,y)
gc()
###  now rearrange in wide format and combine all years
library(plyr)
### read in the list of files created above and combine
lulcc <- list.files("./Data/Output/",pattern="lulcc_patches",full.names=T)
lulcc <- lulcc[-grep("wide|_b",lulcc)]
lulcc <-  do.call(rbind.fill , lapply(lulcc, read.csv))
#lulcc$areaLocal <- round(lulcc$areaLocal,1)
detach("package:plyr", unload=TRUE)
##  separate by embayment first, then convert to wide format
##  cannot do all at once, memory overload
#years <- c(1996,2001,2006,2010,2016)
pb <- txtProgressBar(min = 1, max = length(unique(lulcc$NAME)),style = 3)
areas <- data.frame(NAME=NULL,year=NULL,area=NULL)
for (C in unique(lulcc$NAME)){
  ##  add up the cells
  ##  the result from the above gives the percent of each cell that
  ##  falls within the patch area
  lulcc_l <- lulcc[which(lulcc$NAME==C),-which(colnames(lulcc)=='area')]
  #lulcc_l <- lulcc_l %>%
  #  distinct(NAME,Class,value,year,Rasteryear,patchID,.keep_all = T)
  #lulcc_l <- lulcc_l[!is.na(lulcc_l$value),]
  #lulcc_l[which(!lulcc_l$Class %in% c(18,18.1,18.2,18.3,19,23)&!lulcc_l$value %in% c(18,18.1,18.2,18.3,19,23)),c('value','coverage_area')] <- NA
  #lulcc_l <- lulcc_l[which(lulcc_l$coverage_area>1&!is.na(lulcc_l$coverage_area)),]
  #lulcc_l <- lulcc_l[lulcc_l$Rasteryear==years[match(lulcc_l$year,years)+1],]
  #lulcc_l_focal <- lulcc_l[lulcc_l$Class%in% c(18,18.1,18.2,18.3,19,21)|lulcc_l$value %in% c(18,18.1,18.2,18.3,19,21),]
  #lulcc_l_nofocal <- lulcc_l[!lulcc_l$Class%in% c(18,18.1,18.2,18.3,19,21)&!lulcc_l$value %in% c(18,18.1,18.2,18.3,19,21),]
  
  #lulcc_l$area_new <- (900)*lulcc_l$coverage_fraction
  #lulcc_l <- aggregate(data=lulcc_l,coverage_area~patchID+NAME+Class+value+year+areaLocal+Rasteryear,
  #                     FUN=sum,na.rm=T)
  #lulcc_l <- lulcc_l %>%
   # group_by(patchID,NAME,Class,value, year, Rasteryear,areaLocal,converted_area_T) %>%
   # summarise(area=sum(coverage_area,na.rm=T))%>%
    #ungroup()
  colnames(lulcc_l)[colnames(lulcc_l)%in% c("areaLocal")] <- c("area_o")
  colnames(lulcc_l)[colnames(lulcc_l)%in% c("coverage_area")] <- c("area")
  #lulcc_l_nofocal <- lulcc_l_nofocal %>%
   # group_by(patchID,NAME,Class,value, year, areaLocal, Rasteryear) %>%
   # summarise(area=sum(coverage_area))
  #colnames(lulcc_l_nofocal)[colnames(lulcc_l_nofocal)%in% c("areaLocal")] <- c("area_o")
  ##  reshape to wide
  ##  sometimes the original areas are slightly different for the same patch
  ##  this creates duplicates where there shouldnt be
  #  so round these to the nearest meter
  lulcc_l$area_o <- round(lulcc_l$area_o)
  #lulcc_l_nofocal$area_o <- round(lulcc_l_nofocal$area_o)
  #lulcc_w <- reshape(data.frame(lulcc_l),idvar=c("patchID","Class","NAME","value",'area_o','year'),
  #                   direction="wide",timevar="Rasteryear",v.names=c('converted_area_T','area'))
  lulcc_w <- tidyr::pivot_wider(lulcc_l[,-which(colnames(lulcc_l)=='n')],names_from=Rasteryear,
                                values_from=c(converted_area_T,area))
  #lulcc_w_nofocal <- reshape(lulcc_l_nofocal,idvar=c("patchID","Class","NAME","value",'area_o','year'),direction="wide",timevar="Rasteryear")
  #lulcc_w <- bind_rows(lulcc_w_focal,lulcc_w_nofocal)
  ##  rename the original Class as Class_o
  colnames(lulcc_w)[grepl("Class",colnames(lulcc_w))] <- "Class_o"
  colnames(lulcc_w)[grepl("value",colnames(lulcc_w))] <- "Class"
  ## combine the embayments
  areas <- rbind(areas,data.frame(NAME=C,year=1996,area=sum(lulcc_w$area_o[lulcc_w$Class_o==23&lulcc_w$year==1996&!duplicated(lulcc_w$patchID)])),
                 data.frame(NAME=C,year=2001,area=sum(lulcc_w$area_o[lulcc_w$Class_o==23&lulcc_w$year==2001&!duplicated(lulcc_w$patchID)])),
                 data.frame(NAME=C,year=2006,area=sum(lulcc_w$area_o[lulcc_w$Class_o==23&lulcc_w$year==2006&!duplicated(lulcc_w$patchID)])),
                 data.frame(NAME=C,year=2010,area=sum(lulcc_w$area_o[lulcc_w$Class_o==23&lulcc_w$year==2010&!duplicated(lulcc_w$patchID)])),
                 data.frame(NAME=C,year=2016,area=sum(lulcc_w$area_o[lulcc_w$Class_o==23&lulcc_w$year==2016&!duplicated(lulcc_w$patchID)])))
  if (C ==unique(lulcc$NAME)[1]){lulcc_all <- lulcc_w}else{lulcc_all <- rbind(lulcc_all,lulcc_w)}
  Sys.sleep(0.1); setTxtProgressBar(pb,which(unique(lulcc$NAME)==C))
}
###  save the combined data
write.csv(lulcc_all,"./Data/Output/lulcc_patches_wide_b.csv",row.names=F)


####################################################
#####        Calculate Connectivity
####################################################
##  This calculates a Connectivity scores for each patch
##  It uses two primary connectivity metrics, of which one can be  chosen
##  The Integral Index of Connectivity (IIC) and the Landscape Coincidence Probability are both described in https://link.springer.com/article/10.1007/s10980-006-0013-z
##  Each has its pros and cons
##  Both of these metrics were originally calculated in the lconnect package
##  But to improve speed and memory usage, i rewrote the functions to utilize C++ via the rcpp package
##  The modified functions are stored in a separate R script that is sourced here
library(sf)
library(Rcpp)
library(progress)
library(dplyr)
library(bigdist)
source("./Data/Input/rcpp.R", echo=TRUE)


## read in the dissolved and IDed patches
## also the delineations
allpatches <- read_sf("./Data/Output/allpatches_seagrass_saltmarsh_modified_IDs.shp")
allpatches$Class <- round(as.numeric(allpatches$Class),1)
allpatches <- allpatches[which(allpatches$Class !=0),]
ecotypes <- read_sf("./Data/Input/embayment_units_ecotypes/massbays_estuarine_embays_2021_MODIFIED.shp")
ecotypes <- st_transform(ecotypes,st_crs(allpatches))
Connections <<- data.frame(patchID=NULL,cluster=NULL,n=NULL,Class=NULL,year=NULL,scale=NULL,Variable=NULL,
                           Value=NULL,met=NULL)
for (y in c(1996,2001,2006,2010,2016)){
  s <- allpatches[allpatches$year==y,]
  ##  to be able to calculate inter-habitat connectivity
  ##  we need to be able to combine all the habitats together
   if (sum(c(23,18,18.1,18.2,18.3,19) %in% unique(s$Class))==6){
     s2 <- s[which(s$Class %in% c(23,18,18.1,18.2,18.3,19)),]
     s2$Class=24
     s <- rbind(s,s2[,colnames(s)])
     remove(s2)
   }
   classes=unique(s$Class)
   classes <- classes[order(-classes)]
  s <- s[order(-as.numeric(st_area(s))),]
  for (L in classes){#c(24,23,19,18,18.1,18.2,18.3)){
    if (L==24){clean <- TRUE}else{clean <- FALSE}
    ###  what is the best connectivity distance threshold?
    ## 400-800m https://iopscience.iop.org/article/10.1088/1757-899X/392/6/062094/pdf
    ## 500 m for multiple different coastal habitats  https://besjournals.onlinelibrary.wiley.com/doi/pdfdirect/10.1111/1365-2664.12824
    
    if (nrow(s[s$Class==L,])>1){
      Connections <- IIC_Calc1(s[s$Class == L,],max_dist=500,clean=clean,scale="All",met="IIC")
    }else{
      Connections <- rbind(Connections,data.frame(patchID =s$patchID[s$Class==L],cluster=NA,n=1,Class=L,year=y,
                                                  scale='All',Variable="Conn.",Value=100,met=NA)) 
    }
    pb <- txtProgressBar(min = 1, max = nrow(ecotypes), style = 3)
    for (C in 1:nrow(ecotypes)){
      landC <- s[s$Class %in% L&s$NAME==ecotypes$NAME[C],] 
      #int <- st_intersects(ecotypes[C,],land)
      int <- nrow(landC)
      #landC <- land[unlist(int),]
      if (int >1){#length(unlist(int))>1){
        Connections <- IIC_Calc1(landC,max_dist=500,clean=clean,scale=ecotypes$NAME[C],met="PC")
        Connections <- IIC_Calc1(landC,max_dist=500,clean=clean,scale=ecotypes$NAME[C],met="IIC")
      } else if (int==1){#(length(unlist(int))==1){
        Connections <- IIC_Calc1(landC,max_dist=500,clean=clean,scale=ecotypes$NAME[C],met="PC")
        Connections <- rbind(Connections,data.frame(patchID =landC$patchID,#patchID =landC$patchID[unlist(int)],
                                                    cluster=NA,n=1,Class=L,year=y,
                                                    scale=ecotypes$NAME[C],Variable="Conn.",Value=100,met=NA),
                             data.frame(patchID ="all",cluster=NA,n=1,Class=L,year=y,
                                        scale=ecotypes$NAME[C],Variable="Index",Value=NA,met="IIC")) 
      }
      Sys.sleep(0.1); setTxtProgressBar(pb,C)
    }
    write.csv(Connections,"./Data/Output/Connections_c.csv",row.names=F)
    gc()
  }
}
write.csv(Connections,"./Data/Output/Connections_c.csv",row.names=F)


