#######  This script accompanies the manuscript published in Ecosystem Services:
######  Branoff,Benjamin L., Cicchetti, G., Jackson, SJ, Pryor, M., Sharpe, L.M., Shumchenia, E., and and Susan H. Yee. (2023) A Matrix Approach to Quantify Spatio-temporal Trends in Ecosystem Services Capacity for Massachusetts Bays National Estuary Embayments

###  this script takes matrices from the literature that are all saved in a common directory
##  the matrices are stored with the ecosystems in the first column, the services in the first row
###  and the "scores" or capacity values for each combination of habitat and service

library(openxlsx)
library(stringdist)
library(dplyr)

files <- list.files("./Raw Matrices/",pattern=".xlsx|.csv",full.names = T)
files <- files[!grepl("Master|LitServices|pemsea|~|BlueValue|Costanza|DeGroot",files)]
LC <- matrix(ncol=2,nrow=0)
colnames(LC) <- c("LC","Source")
ES <- matrix(ncol=2,nrow=0)
colnames(ES) <- c("ES","Source")
for (f in files){
  if (grepl(".csv",f)){
    file <- read.csv(f,stringsAsFactors = F)
  }else{
    file <- read.xlsx(f,1) 
  }
  LC <- rbind(LC,cbind(unique(file[,1]),basename(f)))
  ES <- rbind(ES,cbind(colnames(file[,-1]),basename(f)))
}
###  store all the land cover classes and services from the matrices
LC <- data.frame(LC)
ES <- data.frame(ES)
Services <- unique(ES$ES)
Classes <- unique(LC$LC)
Services <- Services[!is.na(Services)&Services!=""]
Classes <- Classes[!is.na(Classes)&Classes!=""]
Services <- gsub("\\."," ",Services)
###  get the unique values
cl <- hclust(stringdistmatrix(Services, method = "lcs", useNames = T))
Services <- cl$labels
write.csv(Services,"Unique Services.csv",row.names=F)
cl <- hclust(stringdistmatrix(Classes[!is.na(Classes)], method = "cosine", useNames = "strings"))
Classes <- cl$labels
write.csv(Classes,"Unique LC Classes.csv",row.names=F)

##  read in the "key" relating each land cover class from the literature to a NOAA CCAP class
##  this is in the supplementary S1 
LCKey <- read.csv("Unique LC Classes_key.csv")
LCKey$NOAA.CCAP <- tools::toTitleCase(LCKey$NOAA.CCAP)
LC <- merge(LC,LCKey[,c("Original","NOAA.CCAP")],by.x="LC",by.y="Original",all.x=T)

###  same for the services, relate them all to a term from NESCS Plus
ESKey <- read.csv("Unique Services_key.csv")
ES$ES <- gsub("\\."," ",ES$ES)
ESKey$FEGS <- tools::toTitleCase(ESKey$FEGS)
ES <- merge(ES,ESKey[,c("Original","FEGS")],by.x="ES",by.y="Original",all.x=T)

###  are any ecosystems unrepresented
unique(LCKey$Options)[which(!unique(LCKey$Options) %in% unique(LC$NOAA.CCAP))]
table(LC$NOAA.CCAP)

###  are any service unrepresented
unique(ESKey$Options)[which(!unique(ESKey$Options) %in% unique(ES$FEGS))]
table(ESKey$FEGS)


###############################################################
####  table of unique ES and LC classes for each study
#########################################################

Table1 = matrix(ncol=4,nrow=0)
colnames(Table1) <- c("Study","nNESCS","nCCAP","max V")
for (s in unique(LC$Source)){
  Table1 <- rbind(Table1,c(s,length(unique(ES$FEGS[ES$Source==s])),length(unique(LC$NOAA.CCAP[LC$Source==s])),max()))
}
Table1 <- Table1[order(Table1[,"Study"]),]


#################################################################################
###########    normalize individual matrices     ##############################
###############################################################################
###  normalize the values in each matrix to a range of 0 to 10

Master <- matrix(ncol=5,nrow=0)
colnames(Master) <- c("Ecosystem","Service","OriginalValue","StandardizedValue","Source")

for (f in files){
  if (grepl(".csv",f)){
    file <- read.csv(f,stringsAsFactors = F)
  }else{
    file <- read.xlsx(f,1) 
  }
  if (grepl("Costanza",f)){file <- file[,c(1,which(file[1,]=="mean"))];file <- file[-1,];file[2:ncol(file)] <- sapply(file[2:ncol(file)] ,as.numeric)}
  #if (grepl("pemsea",f)){browser()}
  numC <- as.numeric(which(sapply(file,is.numeric)))
  if (grepl("Stoll",f)){numC <- numC[-1]}
  Max <- max(file[,2:ncol(file)],na.rm=T)
  Min <- min(file[,2:ncol(file)],na.rm =T)
  vals <- file[,2:ncol(file)]
  norm <- ((file[,2:ncol(file)]-Min)/(Max-Min))*(10)
  Table1[Table1[,1]==basename(f),"max V"] <-Max
  colnames(file)<- gsub("\\."," ", colnames(file))
  file_s <- file
  file_s[,2:ncol(file_s)] <- norm
  file <- reshape(file,direction="long",times=names(file)[2:ncol(file)],varying = list(names(file)[2:ncol(file)]))
  file_s <- reshape(file_s,direction="long",times=names(file_s)[2:ncol(file_s)],varying = list(names(file_s)[2:ncol(file_s)]))
  file <- cbind(file,file_s[,3])
  file <- file[,c(1,2,3,5,4)]
  colnames(file) <- c("Ecosystem","Service","OriginalValue","StandardizedValue","Source")
  file$Source <-basename(f)
  Master <- rbind(Master,file)
  #for (C in numC){
  #  Max <- max(file[,C],na.rm=T)
  #  Min <- min(file[,C],na.rm =T)
  #  norm <- ((file[,C]-Min)/(Max-Min))*(10)
  #  file[,C] <- norm
  #  Master <- rbind(Master,data.frame(Source=substr(basename(f),1,10),Ecosystem=file[,1],Service=colnames(file)[C],Value=norm))
  #}
 # write.csv(file,paste0("C:\\Users\\bbranoff\\OneDrive - Environmental Protection Agency (EPA)\\Profile\\Documents\\Ecosystem Services\\Matrices\\Standardized\\",
    #                    gsub(".csv|.xlsx","",substr(basename(f),1,10)),"_stand.csv"),row.names=F)
}


###############################################
####  create formated matrices and save them
#################################################
Master2 <- merge(Master,ES[!duplicated(ES[,c("ES","FEGS")]),c("ES","FEGS")],by.x="Service",by.y="ES",all=T)
Master2 <- merge(Master2, LC[!duplicated(LC$LC),c("LC","NOAA.CCAP")], by.x="Ecosystem",by.y="LC",all=T)

###  include regional coordinator definitions
Master2$ServiceRC[Master2$FEGS %in% c("Commercially Important Fauna","Edible Fauna")] <- "Edible.or.Commercially.Important.Fauna"
Master2$ServiceRC[Master2$FEGS %in% c("Extreme Weather Events","Flooding")] <- "Protection.From.Extreme.Weather.Events.and.Flooding"
Master2$ServiceRC[Master2$FEGS %in% c("Fauna Community","Rare Fauna","Charasmatic Fauna","Keystone Fauna")] <- "Fauna.Community"
Master2$ServiceRC[Master2$FEGS %in% c("Water Quality")] <- "Water.Quality"
Master2$ServiceRC[Master2$FEGS %in% c("Water Quantity")] <- "Water.Quantity"
Master2$ServiceRC[Master2$FEGS %in% c("Unclassified - Climate Change Impacts Mitigation")] <- 'Climate.Change.Impacts.Mitigation'

Master_agg_FEGS <- do.call(data.frame,aggregate(data=Master2,StandardizedValue~FEGS+NOAA.CCAP,FUN= function(x) c(mean=mean(x,na.rm=T),median=median(x,na.rm=T),
                                                                                                                      sd=sd(x,na.rm=T),max=max(x,na.rm=T),
                                                                                                                      min=min(x,na.rm=T),n=length(x[!is.na(x)]))))
Master_agg_FEGS$FEGS <- factor(Master_agg_FEGS$FEGS,levels = tools::toTitleCase(ESKey$FEGS.Options[ESKey$FEGS.Options!=""]))
Master_agg_FEGS  <- Master_agg_FEGS[order(as.numeric(Master_agg_FEGS$FEGS),-Master_agg_FEGS$StandardizedValue.mean),]

write.csv(Master_agg_FEGS,"Master_agg_FEGS.csv",row.names=F)

Master_agg_RC <- do.call(data.frame,aggregate(data=Master2,StandardizedValue~ServiceRC+NOAA.CCAP,FUN= function(x) c(mean=mean(x,na.rm=T),median=median(x,na.rm=T),
                                                                                                                         sd=sd(x,na.rm=T),max=max(x,na.rm=T),
                                                                                                                         min=min(x,na.rm=T),n=length(x[!is.na(x)]))))
write.csv(Master_agg_RC,"Master_agg_RC.csv",row.names=F)

write.csv(Master2[!is.na(Master2$OriginalValue),],"LitReviewValues.csv",row.names = F)

######################################################################################
############    Create another formatted table  ###################################
####################################################################################


Master_agg_FEGS <-read.csv("Master_agg_FEGS.csv")
Master_agg_RC <- read.csv("Master_agg_RC.csv")
CCAP <- read.csv("\\\\ad.uni-hamburg.de\\redir\\redir0010\\bba4705\\Documents\\MassBays\\Data\\Input\\CCAP_colors.csv")
ESKey <- read.csv("Unique Services_key.csv")
ESKey$FEGS <- tools::toTitleCase(ESKey$FEGS)

Master_agg_FEGS <- merge(Master_agg_FEGS,CCAP[,c("Class","Numeric")],by.x="NOAA.CCAP",by.y="Class",all.x=T)
Master_agg_FEGS$NOAA.CCAP <- factor(Master_agg_FEGS$NOAA.CCAP,levels=CCAP$Class)
Master_agg_FEGS$FEGS <- factor(Master_agg_FEGS$FEGS,levels=tools::toTitleCase(ESKey$FEGS.Options[ESKey$FEGS.Options!=""]))

Master_agg_FEGS <- Master_agg_FEGS[order(Master_agg_FEGS$FEGS,Master_agg_FEGS$Numeric),]
Master_agg_FEGS$formatted <- paste0(round(Master_agg_FEGS$StandardizedValue.mean,1),"[",round(Master_agg_FEGS$StandardizedValue.median,1),"]","?",round(Master_agg_FEGS$StandardizedValue.sd,1),"(",Master_agg_FEGS$StandardizedValue.n,")")

Master_Wide_formatted_FEGS <- reshape(Master_agg_FEGS[,c("FEGS","formatted","NOAA.CCAP")],direction="wide",idvar="NOAA.CCAP",timevar="FEGS")
colnames(Master_Wide_formatted_FEGS) <- gsub("formatted.","",colnames(Master_Wide_formatted_FEGS))
Master_Wide_formatted_FEGS <- Master_Wide_formatted_FEGS[,c("NOAA.CCAP",levels(Master_agg_FEGS$FEGS)[levels(Master_agg_FEGS$FEGS) %in% colnames(Master_Wide_formatted_FEGS)])]
write.csv(Master_Wide_formatted_FEGS,"Master_formatted_FEGS.csv",row.names = F)

Master_Wide_formatted_FEGS_others <- cbind(reshape(Master_agg_FEGS[,c("FEGS","StandardizedValue.mean","NOAA.CCAP")],direction="wide",idvar="NOAA.CCAP",timevar="FEGS") %>%
                           rename_with(~paste0("mean",.x)),
                         reshape(Master_agg_FEGS[,c("FEGS","StandardizedValue.median","NOAA.CCAP")],direction="wide",idvar="NOAA.CCAP",timevar="FEGS") %>%
                           rename_with(~paste0("median",.x)),
                         reshape(Master_agg_FEGS[,c("FEGS","StandardizedValue.sd","NOAA.CCAP")],direction="wide",idvar="NOAA.CCAP",timevar="FEGS") %>%
                           rename_with(~paste0("sd",.x)),
                         reshape(Master_agg_FEGS[,c("FEGS","StandardizedValue.n","NOAA.CCAP")],direction="wide",idvar="NOAA.CCAP",timevar="FEGS") %>%
                           rename_with(~paste0("n",.x)))
write.csv(Master_Wide_formatted_FEGS_others,"Master_formatted_FEGS_others.csv",row.names = F)



Master_agg <- read.csv(paste0(dirname(dirname(dirname(getwd()))),'/Output/Master_agg.csv'))
Master_agg <- Master_agg[Master_agg$ES2!="Area Only",]
Master_agg <- Master_agg[Master_agg$EcoSystem!="All Estuarine Emergent Wetland",]
Master_agg$EcoSystem[Master_agg$EcoSystem=="Estuarine Aquatic Bed"] <- "Seagrass"
Master_agg$EcoSystem[Master_agg$EcoSystem=="Unconsolidated Shore"] <- "Tidal Flats"
Master_agg$EcoSystem <- gsub("Estuarine Emergent Wetland","Saltmarsh",Master_agg$EcoSystem)
Master_agg$EcoSystem <- gsub("High","High Elevation",Master_agg$EcoSystem)
Master_agg$EcoSystem <- gsub("Low","Low Elevation",Master_agg$EcoSystem)


Master_agg <- bind_rows(Master_agg,
                        Master_agg %>%
                          filter(Numeric==23&Region=="All") %>%
                          mutate(Numeric=22,
                                 source='Lit',
                                 EcoSystem='Palustrine Aquatic Beds'),
                        Master_agg %>%
                          filter(Numeric==16&ES2=="Climate change impacts mitigation") %>%
                          mutate(Numeric=13,
                                 source='Lit',
                                 EcoSystem='Palustrine Forested Wetland'),
                        Master_agg %>%
                          filter(!Numeric %in% c(18,18.1,18.2,18.3,18.4,19,23)) %>%
                          mutate(Numeric=28) %>%
                          group_by(ES2,Numeric, source) %>%
                          summarise(Value=NA)) %>%#mean(Value, na.rm=T))) 
  group_by(Region, ES2) %>%
  arrange(-Value) %>%
  mutate(rank=1:n()) %>% ungroup()

Master_aggRC <- Master_agg[Master_agg$source=="RC"&Master_agg$Region!="All",]
Master_aggLit <- Master_agg[Master_agg$source=="Lit",]

Master_aggRC$Region <- factor(Master_aggRC$Region,levels=c("Upper North Shore","Lower North Shore","Boston","South Shore","Cape Cod"))
Master_aggRC$EcoSystem <- factor(Master_aggRC$EcoSystem,levels=c("High Elevation Saltmarsh","Low Elevation Saltmarsh",
                                                                 "Unclass. Elevation Saltmarsh","Seagrass" ,
                                                                 "Tidal Flats"))
Master_aggRC <- Master_aggRC[order(Master_aggRC$ES2,Master_aggRC$Region,Master_aggRC$EcoSystem),]
Master_aggRC_wide <- tidyr::pivot_wider(Master_aggRC,id_cols=c('EcoSystem',"ES2"),names_from = c("Region"),values_from='Value')
Master_aggRC_wide$Average = round(rowMeans(Master_aggRC_wide[,c(3:7)],na.rm=T),1)

customGreen0 = "#DeF7E9"
customGreen = "#71CA97"
library(formattable)
formattable(Master_aggRC_wide[,c(2,1,3:8)], align =c("l","r","c","c","c", "c","c","r"), list(
  `ES2` = formatter("span", style = ~ style(color = "grey",font.weight = "bold")), 
  `EcoSystem`= formatter("span", style = ~ style(color = "grey",font.weight = "bold")),
  `Upper North Shore`= color_tile(customGreen0, customGreen),
  `Lower North Shore`= color_tile(customGreen0, customGreen),
  `Boston`= color_tile(customGreen, customGreen0),
  `South Shore`= color_tile(customGreen0, customGreen),
  `Cape Cod`= color_tile(customGreen0, customGreen),
  `Average`= color_tile('lightblue', 'dodgerblue')
))

write.csv(Master_aggRC_wide,paste0(dirname(dirname(dirname(getwd()))),"/Output/Master_aggWide.csv"),row.names=F)

Master_aggLit$Value <- round(Master_aggLit$Value,1)
Master_aggLit <- Master_aggLit[order(Master_aggLit$Numeric,Master_aggLit$ES2),]
Master_aggLit <- Master_aggLit[!is.na(Master_aggLit$EcoSystem),]
Master_aggLit <- Master_aggLit[-which(Master_aggLit$Region=="All"),]
Master_aggLit$formatted <- paste0(Master_aggLit$Value,"?",round(Master_aggLit$Value.sd,1))
Master_aggLit_wide <- tidyr::pivot_wider(Master_aggLit,id_cols=c('EcoSystem'),names_from = c("ES2"),values_from='formatted')
Master_aggLit_wide2 <- tidyr::pivot_wider(Master_aggLit,id_cols=c('ES2'),names_from = c("EcoSystem"),values_from='formatted')

write.csv(Master_aggLit_wide2,paste0(dirname(dirname(dirname(dirname(getwd())))),"/Tables and Figures/Master_aggLitWide.csv"),row.names=F)
