#######  This script accompanies the manuscript published in Ecosystem Services:
######  Branoff,Benjamin L., Cicchetti, G., Jackson, SJ, Pryor, M., Sharpe, L.M., Shumchenia, E., and and Susan H. Yee. (2023) A Matrix Approach to Quantify Spatio-temporal Trends in Ecosystem Services Capacity for Massachusetts Bays National Estuary Embayments

###  there are two scripts as part of the analysis, this is the second
##  This first script prepares the spatial datasets and modifies them according to the description in the analysis
##  the second performs aggregations and analyses on the modified spatial data and results from the first script

##  In summary: 
##  1) generates a summary of the land cover and changes by region for each of th years 1996, 2001, 2006, 2010 and 2016
##  2) read in and prepare the connectivity and ecosystem services matrices
##  3) process the land use land cover change analysis, aggregate to embayment and then regional scale
##  4) visualize land cover change and compare with the results from step 1
##  5) add in the connectivity and ES matrices, compute ecosystem service and changes
##  6) visualize the ecosystem services dynamics 
##  7) create embayment level maps and analyses.


################################################################
########   regional and ecotype level analysis #################
################################################################
###  using the vector data of individual patch delineations from each year, aggregate to the regional levels
library(sf)
library(dplyr)
library(officer)
library(rvg)
library(ggplot2)
###  read in the allpatches shapefile and intersect with the embayment names and ecotypes
##  this is the cleaned dataset assembled by combining the NOAA_CCAP, MassBays seagrass, and SHARP saltmarsh data
##  This process is included in a the "dataset preparation" script
allpatches <- read_sf("./Data/Output/allpatches_seagrass_saltmarsh_modified_IDs.shp")
##  for now, only interested in the coastal focal ecosystems
allpatches <- allpatches[which(allpatches$Class %in% c(18,18.1,18.2,18.3,19,23)),]
##  manage the columns
allpatches$year <- as.numeric(allpatches$year)
##  add descriptive habitats
allpatches$Habitat[allpatches$Class %in% c(18,18.3)] <- "Unclassified Salt Marsh"
allpatches$Class[allpatches$Class==18.3] <- 18
allpatches$Habitat[allpatches$Class==18.1] <- "High Elevation Salt Marsh"
allpatches$Habitat[allpatches$Class==18.2] <- "Low Elevation Salt Marsh"
allpatches$Habitat[allpatches$Class==19] <- "Tidal Flats"
allpatches$Habitat[allpatches$Class==23] <- "Seagrass"
allpatches$mb_region2 <- factor(allpatches$mb_region2,levels=c('Upper North Shore','Lower North Shore',
                                                               'Boston','South Shore','Cape Cod'))
allpatches$Habitat <- factor(allpatches$Habitat,levels=c('High Elevation Salt Marsh','Low Elevation Salt Marsh',
                                                               'Unclassified Salt Marsh','Seagrass','Tidal Flats'))
allpatches$ecotype <- factor(allpatches$ecotype,levels=c('Low Energy','Low-Med. Energy','Med.-Hig. Energy',
                                                         'Hig. Energy','Undefined'))
allpatches$area[allpatches$mb_region2 %in% c('Lower North Shore','Cape Cod')&allpatches$year==2006&allpatches$Class==23] <- NA

##  aggregate to region and ecotype 
allpatches_reg <- st_drop_geometry(allpatches) %>%
  group_by(mb_region2,ecotype,Habitat,Class, year) %>%
  summarize(area=sum(areaLocal,na.rm=T))
###  fill in missing years with 0 
allpatches_reg <- allpatches_reg %>%
  tidyr::pivot_wider(names_from=year,values_from=area,values_fill=0) %>%
  tidyr::gather(year,area,-c(mb_region2,ecotype,Habitat,Class))
##  add entries for all ecotypes and all regions
allpatches_reg <- bind_rows(allpatches_reg,
                            allpatches_reg %>%
                              group_by(mb_region2,Habitat,Class,year) %>%
                              summarize(area=sum(area,na.rm=T)) %>%
                            mutate(ecotype = 'All'))
allpatches_reg <- bind_rows(allpatches_reg,
                            allpatches_reg %>%
                              group_by(ecotype,Habitat,Class,year) %>%
                              summarize(area=sum(area,na.rm=T)) %>%
                            mutate(mb_region2 = 'All'))
##  now add an entry for all focal ecosystems
allpatches_reg <- bind_rows(allpatches_reg,
                            allpatches_reg %>%
                              group_by(mb_region2,ecotype, year) %>%
                              summarize(area=sum(area,na.rm=T)) %>%
                              mutate(Habitat = 'All Focal'))

## add the percent change from 1996
allpatches_reg <- allpatches_reg %>%
  group_by(Habitat,Class,ecotype,mb_region2) %>% 
  arrange(year,by_group=T) %>%
  mutate(delArea = area-lag(area,1),
         delArea96 = area-first(area),
         pct96 = (area/first(area) * 100),
         area=area)
allpatches_reg$year <- as.numeric(allpatches_reg$year)
allpatches_reg$mb_region2 <- factor(allpatches_reg$mb_region2,levels=c('All','Upper North Shore','Lower North Shore',
                                                               'Boston','South Shore','Cape Cod'))
allpatches_reg$Habitat <- factor(allpatches_reg$Habitat,levels=c('All Focal','High Elevation Salt Marsh','Low Elevation Salt Marsh',
                                                         'Unclassified Salt Marsh','Seagrass','Tidal Flats'))
allpatches_reg$ecotype <- factor(allpatches_reg$ecotype,levels=c('All','Low Energy','Low-Med. Energy','Med.-Hig. Energy',
                                                                 'Hig. Energy','Undefined'))

###  visualize the results
g1a<-ggplot(allpatches_reg,
            aes(x=year,y=pct96))+
  facet_grid(Habitat~mb_region2,scales = 'free_y') +
  geom_point(aes(col=ecotype,shape=ecotype))+
  geom_line(aes(col=ecotype))+
  scale_color_manual(values=c('black',"#FFFF21","#ED7A2C","#00AC47","#006cBE","darkgray"))+ #"#006cBE","#00AC47","#ED7A2C","#FFFF21"
  scale_shape_manual(values=c(15,17,18,19,8,6))+
  theme_classic()+
  theme(panel.background = element_rect(fill = 'gray85'),axis.text.x = element_text(angle = 90),legend.position='bottom',
        strip.placement = "outside",panel.border = element_rect(colour = "black", fill=NA, size=1),
        axis.line.x = element_blank(),axis.line.y = element_blank(),
        strip.background = element_blank(),axis.text = element_text(size=10))
g1b<-g1a+
  #new_scale_color()+
  facet_wrap(.~Habitat+mb_region2,scales = 'free_y',ncol=6)+
  scale_y_continuous(labels=function(x) sprintf("%.0f", x))+
  scale_x_continuous(breaks=c(1996,2001,2006,2010,2016))+
  #geom_line(data=allpatches_reg[allpatches_reg$ecotype2=='All',],aes(x=1995,y=pct96, color=pct96),size=2)+
  #scale_color_gradientn(colors=c("red",'chartreuse3','forestgreen',"darkgreen"))+
  ylab('% Change in Habitat Cover From 1996')+
  theme(strip.text.x = element_blank())
  
read_pptx("\\\\ad.uni-hamburg.de\\redir\\redir0010\\bba4705\\Documents\\Template.pptx") %>%
  #add_slide(layout = "Title and Content", master = "Office") %>%
  ph_with(dml(ggobj=g1a),location=ph_location(left=0.5, top=0.5,width=8,height=8)) %>%
  #add_slide() %>%
  # ph_with(dml(ggobj=g1b),location=ph_location(left=3, top=0.5,width=2,height=4)) %>%
  ph_with(dml(ggobj=g1b),location=ph_location(left=0.5, top=0.5,width=8,height=8)) %>%
  print(target = "./Tables and Figures/BCG.pptx")

#############################################################################
##########################   Connectivity    ################################
#############################################################################
##  read in the connectivity data set calculated in the dataset preparation script
Connections <- read.csv("./Data/Output/Connections_b.csv")
ecotypes <- read_sf("./Data/Input/embayment_units_ecotypes/massbays_estuarine_embays_2021_MODIFIED.shp")
ecotypes$ecotype[ecotypes$ecotype2=='Yellow'] <- 'Low Energy'
ecotypes$ecotype[ecotypes$ecotype2=='Orange'] <- 'Low-Med. Energy'
ecotypes$ecotype[ecotypes$ecotype2=='Green'] <- 'Med.-Hig. Energy'
ecotypes$ecotype[ecotypes$ecotype2=='Blue'] <- 'Hig. Energy'
ecotypes$ecotype[ecotypes$ecotype2=='undefined'] <- 'Undefined'
# merge with ecotypes
Connections <- merge(Connections[Connections$Variable=='Index',] ,
                     st_drop_geometry(ecotypes[,c("NAME","ecotype",'mb_region2')]),
                     by.x="scale",by.y="NAME",all.x=T)
# <- Connections[Connections$Class %in% c(18,18.1,18.2,18.3,19,23,24),]
Connections$Habitat[Connections$Class %in% c(18,18.3)] <- "Unclassified Salt Marsh"
Connections$Habitat[Connections$Class==18.1] <- "High Elevation Salt Marsh"
Connections$Habitat[Connections$Class==18.2] <- "Low Elevation Salt Marsh"
Connections$Habitat[Connections$Class==19] <- "Tidal Flats"
Connections$Habitat[Connections$Class==23] <- "Seagrass"
Connections$Habitat[Connections$Class==24] <- "All Focal-Intra"
Connections$mb_region2[Connections$scale=='All'] <- "NEP"
Connections$ecotype[Connections$scale=='All'] <- "All"


###############################################
####  Where did the habitats go?
################################################
###  read in the land use land cover change dataset
##  this was computed in a previous script
lulcc <- read.csv("./Data/Output/lulcc_patches_wide_b.csv")
lulcc <-  lulcc %>% filter_at(vars(Class,Class_o),any_vars(!is.na(.)))
## read in a table that ties the numeric Class values to the descriptive habitat definitions
NOAACCAP <- read.csv("./Data/Input/CCAP_colors.csv")
##  bring in the ES scores
Master_agg <- read.csv('./Data/Output/Master_agg.csv')
## merge, but separate by the RC scores and the literature scores
Master_agg <- bind_rows(Master_agg,
                          Master_agg %>%
                          filter(Numeric==23&Region=="All") %>%
                          mutate(Numeric=22,
                                 source='Lit'),
                        Master_agg %>%
                          filter(Numeric==16&ES2=="Climate change impacts mitigation") %>%
                          mutate(Numeric=13,
                                 source='Lit'),
                        Master_agg %>%
                          filter(!Numeric %in% c(18,18.1,18.2,18.3,18.4,19,23)) %>%
                          mutate(Numeric=28) %>%
                          group_by(ES2,Numeric, source) %>%
                          summarise(Value=NA)) %>%#mean(Value, na.rm=T))) 
  group_by(Region, ES2) %>%
  arrange(-Value) %>%
  mutate(rank=1:n())
##  read in the ecotypes again
ecotypes <- read_sf("./Data/Input/embayment_units_ecotypes/massbays_estuarine_embays_2021_MODIFIED.shp")
##  merge with the lulcc data
lulcc <- merge(lulcc ,
               st_drop_geometry(ecotypes[,c("NAME","ecotype2",'mb_region2')]),by="NAME",all=T)
##  get rid of unecessary columns
lulcc <- lulcc[,c(2,1,17:18,3:6,7:16)]

##  make sure the LC class is a decimal of only one point to account for the different salt marsh classes
lulcc$Class_o <- round(lulcc$Class_o,1)
lulcc$Class <- round(lulcc$Class,1)
##  only want to evaluate patches moving forward in time
##  so null out values from before the patch existed
lulcc$area_1996[which(lulcc$year>1996)] <- NA
lulcc$area_2001[which(lulcc$year>2001)] <- NA
lulcc$area_2006[which(lulcc$year>2006)] <- NA
lulcc$area_2010[which(lulcc$year>2010)] <- NA
lulcc$area_2016[which(lulcc$year>2016)] <- NA
##  convert the phragmites to just salt marsh
lulcc$Class_o[lulcc$Class_o==18.3] <- 18
lulcc$Class[lulcc$Class==18.3] <- 18
##  convert to long format
##  two different forms, the first is without the seagrass from 2006 in Lower North Shore and Cape Cod
##  The second is with, skipping 2006 and including the change from 2001 to 2010 instead
lulcc_l <- rbind(lulcc[lulcc$year==1996&!is.na(lulcc$converted_area_T_1996),c(1:9,14)] %>%
                   mutate(area=area_1996,converted_area=converted_area_T_1996,year_o=1996,year=1996)%>%select(-area_1996,-converted_area_T_1996),
                 lulcc[lulcc$year==1996&!is.na(lulcc$converted_area_T_2001),c(1:8,10,15)] %>%
                   mutate(area=area_2001,converted_area=converted_area_T_2001,year_o=1996,year=2001)%>%select(-area_2001,-converted_area_T_2001),
                 lulcc[lulcc$year==1996,c(1:8,11,16)] %>%
                   mutate(area=area_2006,converted_area=converted_area_T_2006,year_o=1996,year=2006)%>%select(-area_2006,-converted_area_T_2006),
                 lulcc[lulcc$year==2001,c(1:8,11,16)] %>%
                   mutate(area=area_2006,converted_area=converted_area_T_2006,year_o=2001,year=2006)%>%select(-area_2006,-converted_area_T_2006),
                 lulcc[lulcc$year==1996&!is.na(lulcc$converted_area_T_2010),c(1:8,12,17)] %>%
                   mutate(area=area_2010,converted_area=converted_area_T_2010,year_o=1996,year=2010)%>%select(-area_2010,-converted_area_T_2010),
                 lulcc[lulcc$year==2006,c(1:8,12,17)] %>%
                   mutate(area=area_2010,converted_area=converted_area_T_2010,year_o=2006,year=2010)%>%select(-area_2010,-converted_area_T_2010),
                 ###  the special case for Cape Cod and Lower North Shore
                 lulcc[lulcc$year==2001&lulcc$mb_region2 %in% c("Cape Cod","Lower North Shore"),c(1:8,12,17)] %>%
                   mutate(area=area_2010,converted_area=converted_area_T_2010,year_o=2001,year=2010)%>%select(-area_2010,-converted_area_T_2010),
                 lulcc[lulcc$year==1996&!is.na(lulcc$converted_area_T_2016),c(1:8,13,18)] %>%
                   mutate(area=area_2016,converted_area=converted_area_T_2016,year_o=1996,year=2016)%>%select(-area_2016,-converted_area_T_2016),
                 lulcc[lulcc$year==2010,c(1:8,13,18)] %>%
                   mutate(area=area_2016,converted_area=converted_area_T_2016,year_o=2010,year=2016)%>%select(-area_2016,-converted_area_T_2016))
lulcc_l_alt <- lulcc_l %>%
  filter((mb_region2 %in% c("Cape Cod","Lower North Shore")&(year_o==2006|year==2006)))
lulcc_l <- lulcc_l %>%
  filter(!(mb_region2 %in% c("Cape Cod","Lower North Shore")&(year_o==2006|year==2006)))
#lulcc_l_alt$Class[lulcc_l_alt$Class==23] <- NA
#lulcc_l_alt$Class_o[lulcc_l_alt$Class_o==23] <- NA
#lulcc_l <- rbind(lulcc_l,lulcc_l_alt)

###  no need for multiple decimal places in the area
lulcc_l$area <- round(lulcc_l$area,2)
lulcc_l$area_o <- round(lulcc_l$area_o,2)
##  if there was no class in a patch, it is registered as NA, should be 0
##  exceptions are for seagrass in Cape Cod and Lower North Shore for 2006, which were not surveyed
###  we know the seagrass surveys for 2006 were incomplete in Cape Cod and the Lower North Shore
##  so remove these from the data
##  instead use the conversion from 2001 to 2006
lulcc_l$area[is.na(lulcc_l$area)] <- 0
lulcc_l$converted_area[is.na(lulcc_l$converted_area)] <- 0
#lulcc_l <- lulcc_l[!(lulcc_l$mb_region2 %in% c("Lower North Shore","Cape Cod")&(lulcc_l$Class_o==23|lulcc_l$Class==23)&(lulcc_l$year==2006|lulcc_l$year_o==2006)),]



###  we are only interested in conversions
##  but when one patch retains any or all of its original class, it registers the "area" column as the amount that was retained
##  since we are interested in conversions, this should be 0, or the amount retained minus the original amount
lulcc_l$area[which(lulcc_l$Class==lulcc_l$Class_o)] <- NA#lulcc_l$area[which(lulcc_l$Class==lulcc_l$Class_o)]-lulcc_l$area_o[which(lulcc_l$Class==lulcc_l$Class_o)]
##  save the original areas for later
##  also need the original amount of focal habitat area
##  so create a new class that is the combination of all focal habitats
##  this new class is labeled as 26
lulcc_orig <- bind_rows(lulcc_l,
                        lulcc_l %>%
                          filter(Class_o %in% c(18,18.1,18.2,18.3,19,23))%>%
                          mutate(Class_o=26),
                        lulcc_l %>%
                          filter(Class %in% c(18,18.1,18.2,18.3,19,23))%>%
                          mutate(Class=26))
##  need to do the same for the non-focal classes
##  especially when they are not involved in any focal habitat conversions
##  summarise the non-focal habitats so we dont have to track each individual non-focal patch
lulcc_orig <- lulcc_orig %>% ungroup() %>%
  group_by(Class_o,NAME,year_o) %>%
  mutate(dup=duplicated(patchID)&!is.na(patchID),
         l=n(),
         area_o=ifelse(28 %in% Class,max(area_o),
                       ifelse(l==1,area_o,sum(area_o[!dup])))) %>%
  group_by(Class_o,Class,NAME,year,year_o) %>%
  summarise(area_o=first(area_o),
            converted_area=max(converted_area,na.rm=T),
            lost_area=converted_area) %>%
  select(-converted_area)%>%
  ungroup()

##   now summarise the total amount of added or lost area for each class in each embayment
##  this helps determine the relative contribution of each patch change to the embayment change
lulcc_orig <- merge(lulcc_orig,lulcc_orig %>% #filter(Class!=28)%>%
                      mutate(added_area=lost_area) %>%
                      select(-lost_area,-area_o),
                    by.x=c("Class_o","Class","NAME","year","year_o"),
                    by.y=c("Class","Class_o","NAME","year","year_o"),
                    all=T) %>%
  group_by(Class_o,NAME,year_o) %>% 
  mutate(added_area=ifelse(is.na(added_area),0,added_area),
         lost_area=ifelse(is.na(lost_area),0,lost_area),
         area_o=ifelse(all(is.na(area_o)),0,max(area_o,na.rm=T)))

lulcc_orig <- lulcc_orig %>%  ungroup %>%
  mutate(del_area_o=added_area-lost_area,
         converted_area=added_area+lost_area)

lulcc_orig <- lulcc_orig %>% ungroup() %>%
  group_by(NAME,Class_o,year_o,year) %>%
  mutate(converted_area_T=sum(converted_area[Class!=26],na.rm=T))

##  now aggregate the converted areas
lulcc_l <- lulcc_l[!is.na(lulcc_l$Class),] %>%
  group_by(Class,Class_o,NAME,year_o,year) %>%
  summarise(area=sum(area,na.rm=T),
            converted_area=first(converted_area))
lulcc_l$Class_oo <- lulcc_l$Class_o
lulcc_l <- bind_rows(lulcc_l %>% mutate(area=-area,
                                        converted_area=-converted_area) %>% 
                       ungroup() %>% select(-Class_oo),
                 lulcc_l %>% mutate(Class_o=Class,
                                    Class=Class_oo,
                                    converted_area=converted_area)%>%
                   ungroup() %>%
                   select(-Class_oo))
##  add in the all focal class
lulcc_l <- bind_rows(lulcc_l,
                     lulcc_l[which(lulcc_l$Class_o %in% c(18,18.1,18.2,18.3,19,23)),] %>%
                       mutate(Class_o=26))
lulcc_l$converted_area[is.na(lulcc_l$converted_area)] <- lulcc_l$area[is.na(lulcc_l$converted_area)]
## aggregate again
lulcc_l <- lulcc_l %>%
  group_by(Class,Class_o,NAME,year_o,year) %>%
  summarise(area=sum(area,na.rm=T),
            converted_area=sum(converted_area,na.rm=T))
## now make sure the total change in area by converted habitats
## equals the total change in area previously calculated from allpatches_reg
##  this is to ensure everything matches before continuing
##  first find the total converted area
##  the following steps create comparable areas between the two data sets and then compares them by plotting them
allpatches <- read_sf("./Data/Output/allpatches_seagrass_saltmarsh_modified_IDs.shp")
##  remove the seagrass from 2006 for Cape Cod and Lower North Shore
#allpatches <- allpatches[!(allpatches$mb_region2 %in% c("Cape Cod","Lower North Shore")&allpatches$Class==23&allpatches$year==2006),]
allpatches <- allpatches[!(allpatches$mb_region2 %in% c('Lower North Shore','Cape Cod')&allpatches$year==2006&allpatches$Class==23),] 
allpatches <- bind_rows(st_drop_geometry(allpatches),
                          st_drop_geometry(allpatches[allpatches$Class %in% c(18,18.1,18.2,18.3,19,23),]) %>%
                          mutate(Class=26))
allpatches_NAME <- allpatches %>%
  mutate(Class=ifelse(Class==18.3,18,Class)) %>%
  group_by(NAME,Class, year) %>%
  summarize(area=sum(areaLocal,na.rm=T)) %>%
  tidyr::pivot_wider(names_from=year,values_from=area,values_fill=0) %>%
  tidyr::gather(year,area,-c(NAME,Class))%>%
  arrange(year,by_group=T) %>%
  mutate(delArea = area-lag(area,1),
         area_o = lag(area,1),
         delArea96 = area-first(area),
         pct96 = (area/first(area) * 100),
         year_o = lag(year))
allpatches_NAME$year_o[is.na(allpatches_NAME$year_o)] <- 1996
#allpatches_NAME$delArea[allpatches_NAME$delArea<1] <- 0
##  total converted area of the lulcc converted class
colnames(lulcc_l)[colnames(lulcc_l)=='Class_o'] ='Class_o'
colnames(lulcc_l)[colnames(lulcc_l)=='Class'] ='Class_conv'
lulcc_l2 <- merge(lulcc_l,
                 lulcc_l %>%
                   group_by(Class_o,year_o,year,NAME) %>%
                   summarise(delArea_o=sum(converted_area, na.rm=T)),
                 by=c('Class_o','year_o','year','NAME'))
lulcc_l2 <- merge(lulcc_l2 %>% ungroup(),
                 lulcc_l2 %>% filter(Class_o!=26) %>%
                   group_by(Class_conv,year_o,year,NAME) %>%
                   summarise(delArea_conv=sum(-converted_area, na.rm=T)),
                 by=c('Class_conv','year_o','year','NAME'))
##  now get the change in areas and the original from allpatches
lulcc_l2<- merge(lulcc_l2 %>%ungroup(),allpatches_NAME[,c('NAME','Class','year','year_o','delArea')] %>% ungroup(),
                 by.x = c('NAME','Class_o','year_o','year'),by.y=c('NAME','Class','year_o','year'),all =T)
lulcc_l2 <- merge(lulcc_l2,allpatches_NAME[,c('NAME','Class','year','area')] %>% ungroup(),
                 by.x = c('NAME','Class_o','year_o'),by.y=c('NAME','Class','year'),all=T)  ## the Allpatches "area" is for the year and the Allpatches "area_o" is for the year_o
## same for the converted class
lulcc_l2 <- merge(lulcc_l2,allpatches_NAME[,c('NAME','Class','year','year_o','delArea')],
                 by.x = c('NAME','Class_conv','year_o','year'),by.y=c('NAME','Class','year_o','year'),all=T)
lulcc_l2 <- merge(lulcc_l2,allpatches_NAME[,c('NAME','Class','year','area')],
                 by.x = c('NAME','Class_conv','year_o'),by.y=c('NAME','Class','year'),all=T)
## clean up column names
colnames(lulcc_l2)[colnames(lulcc_l2)=='delArea.x'] <- 'delArea_o2'
colnames(lulcc_l2)[colnames(lulcc_l2)=='area.x'] <- 'area_converted'
colnames(lulcc_l2)[colnames(lulcc_l2)=='area.y'] <- 'area_o_o'
colnames(lulcc_l2)[colnames(lulcc_l2)=='delArea.y'] <- 'delArea_conv2'
colnames(lulcc_l2)[colnames(lulcc_l2)=='area'] <- 'area_o_conv'
##  the NAs for area_o_conv and area_o_conv mean there were no areas for that habitat in that yer in that embayment
lulcc_l2$area_o_o[is.na(lulcc_l2$area_o_o)] <- 0
lulcc_l2$area_o_conv[is.na(lulcc_l2$area_o_conv)] <- 0
##  the resulting NAs for class_o mean there were no changes for that Class
##  we  see that the delArea_o, which we have for all classes
## is identical to the delArea from allpatches
plot(lulcc_l2$delArea_o,lulcc_l2$delArea_o2)
abline(a=0,b=1)

## make sure the same is true for delArea_conv
plot(lulcc_l2$delArea_conv,lulcc_l2$delArea_conv2)
abline(a=0,b=1)
##  so we know the change in coverage from lulcc_l2 matches that from allpatches
##  the version from allpatches isnt complete and contains NAs
## we just wanted to make sure the two data sets are in agreement when they do have data
## remove the unused columns, those with NAS
lulcc_l2 <- lulcc_l2[,-which(names(lulcc_l2) %in% c("delArea_o2","delArea_conv2"))]

##  we still need the original areas of the original habitat,add the total conversion for each habitat in each embayment
## so add those
lulcc_l2 <- merge(lulcc_l2[,-which(colnames(lulcc_l2)=="converted_area")],lulcc_orig[lulcc_orig$Class!=26,],
                 by.x=c('Class_o','Class_conv','NAME','year_o','year'),
                 by.y=c('Class_o','Class','NAME','year_o','year'),all=T)
lulcc_l2 <- merge(lulcc_l2,
                 lulcc_orig[lulcc_orig$Class_o!=26,c('Class_o','Class','NAME','year_o','year','converted_area_T')],
                 by.x=c('Class_o','Class_conv','NAME','year_o','year'),
                 by.y=c('Class','Class_o','NAME','year_o','year'),all=T)
colnames(lulcc_l2)[grepl('.x|.y',colnames(lulcc_l2))] <- c('converted_area_T_o','converted_area_T_conv')
###  check the total converted area against the area_converted
plot(lulcc_l2$area_converted,lulcc_l2$added_area-lulcc_l2$lost_area)

##  can also compare the original areas to the values from allpatches, just to make sure
plot(lulcc_l2$area_o_o[lulcc_l2$Class_o!=28],lulcc_l2$area_o[lulcc_l2$Class_o!=28])
##  so we know that the initial area numbers are good when Class_o does not equal 28
##  we aggregated the non-focal classes to 28, so we cant compare that with the original areas
## correct this by using the area_o value
lulcc_l2$area_o_o[which(lulcc_l2$Class_o==28)] <- lulcc_l2$area_o[which(lulcc_l2$Class_o==28)]
plot(lulcc_l2$area_o_o,lulcc_l2$area_o)
## remove the now redundant column
lulcc_l2 <- lulcc_l2[,-which(colnames(lulcc_l2)=="area_o")]

## add the regions
lulcc_l2 <- merge(lulcc_l2, st_drop_geometry(ecotypes[,c('NAME','mb_region2')]))
## create a copy for later
lulccAGG_NAME <- lulcc_l2 
write.csv(lulccAGG_NAME,".\\Data\\Output\\lulccAGG_NAME.csv",row.names=F)
## aggregate the least important or similar classes
## (all the urban classes into one developed class)
##  convert the Phragmites class into a general salt marsh class
lulccAGG <- merge(lulcc_l2 %>% filter(!is.na(Class_o),!is.na(Class_conv)) %>%
                    distinct(Class_o,NAME,mb_region2,year,year_o,.keep_all=T) %>%
                    mutate(is.focal=Class_o %in% c(18,18.1,18.2,18.3,19,23,26),
                           Class_o =ifelse(is.focal,Class_o,27))%>%
                    mutate(is.phrag =Class_o==18.3,
                           Class_o =ifelse(is.phrag,18,Class_o))%>%
                    group_by(mb_region2,Class_o,year_o,year) %>% 
                    summarise(area_o=sum(area_o_o,na.rm=T),
                              delArea_o=sum(delArea_o,na.rm=T)),
                  lulcc_l2 %>%
                    mutate(is.focal=Class_o %in% c(18,18.1,18.2,18.3,19,23,26),
                           Class_o =ifelse(is.focal,Class_o,27))%>%
                    mutate(is.phrag =Class_o==18.3,
                           Class_o =ifelse(is.phrag,18,Class_o))%>%
                    mutate(is.phrag =Class_conv==18.3,
                           Class_conv =ifelse(is.phrag,18,Class_conv))%>%
                    mutate(is.dev =Class_conv %in% c(2:5),
                           Class_conv =ifelse(is.dev,2,Class_conv)) %>%
                    mutate(is.focal=Class_conv %in% c(18,18.1,18.2,18.3,19,21,23,2,20,17,26),
                           Class_conv =ifelse(is.focal,Class_conv,27))%>%
                    group_by(mb_region2,Class_o,Class_conv,year,year_o) %>%
                    summarise(area_converted=sum(area_converted,na.rm=T)),
                  by=c('mb_region2','Class_o','year_o','year'))
## aggregate to "All" regions
lulccAGG <- bind_rows(lulccAGG,
                      lulccAGG %>% 
                        group_by(mb_region2,year_o,year,Class_o) %>%
                        mutate(dup.o=duplicated(Class_o),
                               mb_region2="All") %>%
                        group_by(mb_region2,year_o,year,Class_o) %>%
                        mutate(area_o=sum(area_o[!dup.o]),
                               delArea_o=sum(delArea_o[!dup.o])) %>%
                        group_by(mb_region2,year_o,year,Class_o,Class_conv) %>%
                        summarise(area_o=first(area_o),
                                  delArea_o=first(delArea_o),
                                  area_converted=sum(area_converted)))

##  calculate the cumulative change and the change since 1996
lulccAGG <- lulccAGG %>%
  group_by(mb_region2,Class_o,Class_conv) %>%
  arrange(year,by_group=T) %>%
  mutate(is.dup=duplicated(year),
         pct96_change_conv = 100*area_converted/first(area_o))

lulccAGG <- lulccAGG %>%
  group_by(mb_region2,Class_o,year_o) %>%
  arrange(year,by_group=T) %>%
  mutate(PctChange_o = 100*(delArea_o/area_o),
         PctChange_conv = 100*(area_converted)/first(area_o),
         pct96_change = 100*((area_o+delArea_o)-first(area_o))/first(area_o),
         pctOfConversion = 100*area_converted/delArea_o)

##  add more descriptive habitat classes
lulccAGG <- merge(lulccAGG,NOAACCAP[,c('Numeric','Habitat')],by.x='Class_o',by.y='Numeric')
lulccAGG <- merge(lulccAGG,NOAACCAP[,c('Numeric','Habitat')],by.x='Class_conv',by.y='Numeric',all.x=T)
lulccAGG$Habitat.x <- factor(lulccAGG$Habitat.x,levels=c('All Focal','High Elevation Salt Marsh','Low Elevation Salt Marsh',
                                                     'Unclassified Salt Marsh','Seagrass','Tidal Flats','All Others'))
lulccAGG$Habitat.y <- factor(lulccAGG$Habitat.y,levels=c('High Elevation Salt Marsh','Low Elevation Salt Marsh',
                                                         'Unclassified Salt Marsh','Seagrass','Tidal Flats',
                                                         'Open Water','Development','Beach','Estuarine Scrub/Shrub Wetland','All Others'))
lulccAGG$mb_region2 <- factor(lulccAGG$mb_region2,levels=c('All','Upper North Shore','Lower North Shore','Boston',
                                                           'South Shore','Cape Cod'))
lulccAGG$year =as.numeric(lulccAGG$year)
allpatches_reg2 <- allpatches_reg
names(allpatches_reg2)[names(allpatches_reg2)=="Habitat"] <- "Habitat.x"
allpatches_reg2$pct96[allpatches_reg2$mb_region2 %in% c("All","Cape Cod","Lower North Shore")&
                        allpatches_reg2$year==2006]=NA
allpatches_reg2 <- allpatches_reg2 %>%
  filter(ecotype=="All") %>%
  group_by(mb_region2,Habitat.x) %>%
  mutate(pct96 = zoo::na.approx(pct96))
###  visualize
g4b <- ggplot(data=lulccAGG[lulccAGG$Habitat.x!='All Others',]%>%
                filter(!(year_o==1996&year %in% c(2006,2010,2016)),
                       !(mb_region2 == "All" & year_o==2006)),
              aes(x=year,y=pct96_change_conv))+
  facet_grid(Habitat.x~mb_region2,scales = 'free_y') +
  geom_bar(stat='identity',col='black',aes(fill=Habitat.y))+
  geom_line(data=allpatches_reg2[allpatches_reg2$ecotype=="All",],aes(x=year,y=pct96-100),
            col='Black',size=1.25)+
  #geom_line(data=lulccAGG[!lulccAGG$Class_conv %in% c(NA,27)&lulccAGG$Habitat.x!='All Others'&lulccAGG$year_o==1996,],#%>%filter(!(year_o==1996&year %in% c(2006,2010,2016))),
  #          aes(x=year,y=PctChange_o),col='Black',size=1.25)+
  scale_fill_manual(values=NOAACCAP$cols2[NOAACCAP$Numeric %in% lulccAGG$Class_conv][c(3:5,9,6,8,1,7,2,10)])+
  scale_x_continuous(breaks=c(1996,2001,2006,2010,2016))+
  theme_classic()+
  geom_hline(yintercept=0,lty=3)+
  theme(axis.text.x = element_text(angle = 90),legend.position='bottom',
        strip.placement = "outside",panel.border = element_rect(colour = "black", fill=NA, size=1),
        axis.line.x = element_blank(),axis.line.y = element_blank(),
        strip.background = element_blank(),axis.text = element_text(size=10),strip.text= element_text(size=10))
g4c <- g4b+
  facet_wrap(.~Habitat.x+mb_region2,scales = 'free_y',ncol=6)+
  scale_y_continuous(labels=function(x) sprintf("%.1f", x))+
  ylab('Annual % Change in Habitat Cover')+
  theme(strip.text.x = element_blank())

g4d <- ggplot(data=lulccAGG[lulccAGG$mb_region2=="All"& lulccAGG$Habitat.x!='All Others',]%>%filter(!(year_o==1996&year %in% c(2006,2010,2016))),
              aes(x=year,y=pct96_change_conv))+
  facet_grid(rows="Habitat.x",scales = "free_y") +
  geom_bar(stat='identity',col='black',aes(fill=Habitat.y))+
  geom_line(data=allpatches_reg2[allpatches_reg2$ecotype=="All",],aes(x=year,y=pct96-100),
            col='Black',size=1.25)+
  #geom_line(data=lulccAGG[lulccAGG$mb_region2=="All"&!lulccAGG$Class_conv %in% c(NA,27)&lulccAGG$Habitat.x!='All Others'&lulccAGG$year_o==1996,],#%>%filter(!(year_o==1996&year %in% c(2006,2010,2016))),
  #          aes(x=year,y=pct96_change),col='Black',size=1.25)+
  scale_fill_manual(values=NOAACCAP$cols2[NOAACCAP$Numeric %in% lulccAGG$Class_conv][c(3:5,9,6,8,1,7,2,10)])+
  scale_x_continuous(breaks=c(1996,2001,2006,2010,2016))+
  theme_classic()+
  geom_hline(yintercept=0,lty=3)+
  theme(axis.text.x = element_text(angle = 90),legend.position='bottom',
        strip.placement = "outside",panel.border = element_rect(colour = "black", fill=NA, size=1),
        axis.line.x = element_blank(),axis.line.y = element_blank(),
        strip.background = element_blank(),axis.text = element_text(size=10),strip.text= element_text(size=10))

read_pptx("\\\\ad.uni-hamburg.de\\redir\\redir0010\\bba4705\\Documents\\Template_landscape.pptx") %>%
  #add_slide(layout = "Title and Content", master = "Office") %>%
  ph_with(dml(ggobj=g4b),location=ph_location(left=0.5, top=0.5,width=10,5,height=7.5)) %>%
  #add_slide() %>%
  # ph_with(dml(ggobj=g1b),location=ph_location(left=3, top=0.5,width=2,height=4)) %>%
  ph_with(dml(ggobj=g4c),location=ph_location(left=0.5, top=0.5,width=10.5,height=7.5)) %>%
  #add_slide(layout = "Titel und Inhalt", master = "Office") %>%
  ph_with(dml(ggobj=g4d),location=ph_location(left=0.5, top=0.5,width=3.5,height=9)) %>%
  print(target = "./Tables and Figures/LULCC.pptx")

###############################################################
########   what about services   ##############################
################################################################
##  read in the lulcc data from the previous step
##  and the Es matrices from a separate script
lulccAGG_ES <- read.csv(".\\Data\\Output\\lulccAGG_NAME.csv")
Master_agg <- read.csv(".\\Data\\Output\\Master_agg.csv")
##  set the areas and changes to NA for seagrass in Cape Cod and Lower North Shore
lulccAGG_ES <- lulccAGG_ES[which(!(lulccAGG_ES$mb_region2 %in% c("Cape Cod","Lower North Shore")&(lulccAGG_ES$year_o==2006|lulccAGG_ES$year==2006))),]

##  merge the above with the service scores and connectivity
##  get ES value of original class
lulccAGG_ES <- bind_rows(merge(lulccAGG_ES[lulccAGG_ES$Class_o %in% c(18,18.1,18.2,18.3,19,23,NA),], 
                                      Master_agg[Master_agg$source=='RC',c('ES2','Region','Value','Numeric')],
                                      by.x=c("Class_o",'mb_region2'),by.y=c("Numeric",'Region'),all.x=T),
                         merge(lulccAGG_ES[!lulccAGG_ES$Class_o %in% c(18,18.1,18.2,18.3,19,23,NA),], 
                                      Master_agg[Master_agg$source=='Lit'&!Master_agg$Numeric%in% c(18,18.1,18.2,18.3,19,23,NA),c('ES2','Value','Numeric')],
                                      by.x=c("Class_o"),by.y=c("Numeric"),all.x=T))

###  now add the aggregate values with the standard deviations
###  this is only different for the RC scores, the lit scores are the same
lulccAGG_ES <-bind_rows(merge(lulccAGG_ES[lulccAGG_ES$Class_o %in% c(18,18.1,18.2,18.3,19,23,NA),],
                     Master_agg[Master_agg$source=='RC'&Master_agg$Region=="All",c('ES2','Value','Value.sd','Numeric')],
                     by.x=c("Class_o",'ES2'),by.y=c("Numeric",'ES2'),all.x=T),
                     merge(lulccAGG_ES[!lulccAGG_ES$Class_o %in% c(18,18.1,18.2,18.3,19,23,NA),], 
                           Master_agg[Master_agg$source=='Lit'&!Master_agg$Numeric%in% c(18,18.1,18.2,18.3,19,23,NA),c('ES2','Value','Value.sd','Numeric')],
                           by.x=c("Class_o","ES2"),by.y=c("Numeric","ES2"),all.x=T))
##  clean up the column names
colnames(lulccAGG_ES)[colnames(lulccAGG_ES)=="Value.x"] <- 'Score_o_Regional'
colnames(lulccAGG_ES)[colnames(lulccAGG_ES)=="Value.y"] <- 'Score_o_Aggregate'                    
colnames(lulccAGG_ES)[colnames(lulccAGG_ES)=="Value.sd"] <- 'sd_o'                    

##  now the converted class
lulccAGG_ES <- bind_rows(merge(lulccAGG_ES[lulccAGG_ES$Class_conv %in% c(18,18.1,18.2,18.3,19,23,NA),], 
                            Master_agg[Master_agg$source=='RC',c('ES2','Region','Value','Numeric')],
                            by.x=c("Class_conv",'mb_region2',"ES2"),by.y=c("Numeric",'Region','ES2'),all.x=T),
                      merge(lulccAGG_ES[!lulccAGG_ES$Class_conv %in% c(18,18.1,18.2,18.3,19,23,NA),], 
                            Master_agg[Master_agg$source=='Lit',c('ES2','Value','Numeric')],
                            by.x=c("Class_conv","ES2"),by.y=c("Numeric","ES2"),all.x=T))
##  and the aggregate values for the converted class
lulccAGG_ES <-bind_rows(merge(lulccAGG_ES[lulccAGG_ES$Class_conv %in% c(18,18.1,18.2,18.3,19,23,NA),],
                              Master_agg[Master_agg$source=='RC'&Master_agg$Region=="All",c('ES2','Value','Value.sd','Numeric')],
                              by.x=c("Class_conv",'ES2'),by.y=c("Numeric",'ES2'),all.x=T),
                        merge(lulccAGG_ES[!lulccAGG_ES$Class_conv %in% c(18,18.1,18.2,18.3,19,23,NA),], 
                              Master_agg[Master_agg$source=='Lit'&!Master_agg$Numeric%in% c(18,18.1,18.2,18.3,19,23,NA),c('ES2','Value','Value.sd','Numeric')],
                              by.x=c("Class_conv","ES2"),by.y=c("Numeric","ES2"),all.x=T))

##  clean up the column names
colnames(lulccAGG_ES)[colnames(lulccAGG_ES)=="Value.x"] <- 'Score_new_Regional'
colnames(lulccAGG_ES)[colnames(lulccAGG_ES)=="Value.y"] <- 'Score_new_Aggregate'
colnames(lulccAGG_ES)[colnames(lulccAGG_ES)=="Value.sd"] <- 'sd_new'

##  now do connectivity, first by original classes and original year
lulccAGG_ES <- merge(lulccAGG_ES,
                  Connections[Connections$met=='PC'&Connections$patchID=='all'&Connections$scale!='All',c('Class','scale','year','n','Value')],
                  by.x=c('Class_o','NAME','year_o'),by.y=c('Class','scale','year'), all.x=T)
##  when the connectivity score is NA, there were no patches, so its 0
##  this is confirmed by the n column
summary(lulccAGG_ES$n[is.na(lulccAGG_ES$Value)])
lulccAGG_ES$Value[is.na(lulccAGG_ES$Value)] <- 0
##  then the connectivity of the original class in the new year
lulccAGG_ES <- merge(lulccAGG_ES,
                 Connections[Connections$met=='PC'&Connections$patchID=='all'&Connections$scale!='All',c('Class','scale','year','Value')],
                 by.x=c('Class_o','NAME','year'),by.y=c('Class','scale','year'), all.x=T)
##  when the connectivity score is NA, there were no patches, so its 0
lulccAGG_ES$Value.y[is.na(lulccAGG_ES$Value.y)] <- 0
## then by converted class in the original year
lulccAGG_ES <- merge(lulccAGG_ES,
                 Connections[Connections$met=='PC'&Connections$patchID=='all'&Connections$scale!='All',c('Class','scale','year','Value')],
                 by.x=c('Class_conv','NAME','year_o'),by.y=c('Class','scale','year'), all.x=T)
##  when the connectivity score is NA, there were no patches, so its 0
lulccAGG_ES$Value[is.na(lulccAGG_ES$Value)] <- 0
##  clean up the column names
colnames(lulccAGG_ES)[colnames(lulccAGG_ES)=='Value.x'] <- 'Conn_o_y1'
colnames(lulccAGG_ES)[colnames(lulccAGG_ES)=='Value.y'] <- 'Conn_o_y2'
colnames(lulccAGG_ES)[colnames(lulccAGG_ES)=='Value'] <- 'Conn_conv_y1'
## then by converted class in the new year
lulccAGG_ES <- merge(lulccAGG_ES,
                     Connections[Connections$met=='PC'&Connections$patchID=='all'&Connections$scale!='All',c('Class','scale','year','Value')],
                     by.x=c('Class_conv','NAME','year'),by.y=c('Class','scale','year'), all.x=T)
##  when the connectivity score is NA, there were no patches, so its 0
lulccAGG_ES$Value[is.na(lulccAGG_ES$Value)] <- 0
colnames(lulccAGG_ES)[colnames(lulccAGG_ES)=='Value'] <- 'Conn_conv_y2'
##  28 Class which has no connectivity shoul be NA
lulccAGG_ES$Conn_conv_y1[which(lulccAGG_ES$Class_conv==28)] <- NA
lulccAGG_ES$Conn_conv_y2[which(lulccAGG_ES$Class_conv==28)] <- NA


##  the change in connectivity is
lulccAGG_ES$delConn_o <- lulccAGG_ES$Conn_o_y2-lulccAGG_ES$Conn_o_y1
lulccAGG_ES$delConn_conv <- lulccAGG_ES$Conn_conv_y2-lulccAGG_ES$Conn_conv_y1
##  this is the total connectivity change for that habitat in that embayment
##  we can estimate the proportion of the connectivity change due to each individual class conversion
##  this is based on the class_o to class_conv percent of the total converted area from Class_o
#lulccAGG_ES$Converted_area_T_o <- lulccAGG_ES$added_area_T_o+lulccAGG_ES$lost_area_T_o
#lulccAGG_ES$Converted_area_T_conv <- lulccAGG_ES$added_area_T_conv+lulccAGG_ES$lost_area_T_conv
##  if the total converted area is less than 1 meter, its essentially zero
lulccAGG_ES$converted_area_T_o[lulccAGG_ES$converted_area_T_o<1&lulccAGG_ES$converted_area_T_o>-1] <- 0
lulccAGG_ES$converted_area_T_conv[lulccAGG_ES$converted_area_T_conv<1&lulccAGG_ES$converted_area_T_conv>-1] <- 0
##  now the percent of the converted area for each class
lulccAGG_ES$perc_conversion_o <- sign(lulccAGG_ES$area_converted)*round((lulccAGG_ES$converted_area)/lulccAGG_ES$converted_area_T_o,2)
lulccAGG_ES$perc_conversion_conv <-  sign(-lulccAGG_ES$area_converted)*round((lulccAGG_ES$converted_area)/lulccAGG_ES$converted_area_T_conv,2)
lulccAGG_ES$perc_conversion_o[lulccAGG_ES$area_converted==0|lulccAGG_ES$converted_area_T_o==0] <- 0
lulccAGG_ES$perc_conversion_conv[lulccAGG_ES$area_converted==0|lulccAGG_ES$converted_area_T_conv==0] <- 0
##  now scale to 0-100
lulccAGG_ES2 <- lulccAGG_ES %>%
  #filter(Class_o!=28) %>%
  group_by(NAME,mb_region2,year_o,year,Class_o,ES2) %>%
  mutate(perc_conv_o_sum = ifelse(sum(perc_conversion_o,na.rm=T)<1&sum(perc_conversion_o,na.rm=T)>-1,0,sum(perc_conversion_o,na.rm=T)),
         perc_conversion_o_scaled=ifelse(perc_conversion_o==0,0,
                                         perc_conversion_o/perc_conv_o_sum)) %>%
  group_by(NAME,mb_region2,year_o,year,Class_conv,ES2) %>%
  mutate(perc_conv_conv_sum = ifelse(sum(perc_conversion_conv,na.rm=T)<1&sum(perc_conversion_conv,na.rm=T)>-1,0,sum(perc_conversion_conv,na.rm=T)),
         perc_conversion_conv_scaled=ifelse(perc_conversion_conv==0,0,
                                            perc_conversion_conv/perc_conv_conv_sum))
## in a few cases, the sum(perc_conversion) is 0, which results in infinity, so we need a special case for these
## first the original class
lulccAGG_ES2 <- bind_rows(lulccAGG_ES2 %>% ungroup() %>%
                           filter(is.finite(perc_conversion_o_scaled)),
                         lulccAGG_ES2 %>% ungroup() %>%
  filter(!is.finite(perc_conversion_o_scaled)) %>%
  group_by(NAME,mb_region2,year_o,year,Class_o,ES2) %>%
  mutate(perc_conv_o_sum = ifelse(perc_conversion_o>0,sum(perc_conversion_o[perc_conversion_o>0],na.rm=T),
                                  sum(perc_conversion_o[perc_conversion_o<0],na.rm=T)),
         nsign = sum(c(any(perc_conversion_o<0),any(perc_conversion_o>0)),na.rm=T),
         perc_conversion_o_scaled = ifelse(perc_conv_o_sum==0,0,ifelse(perc_conversion_o>0,
                                                                       ifelse(nsign==2,2*perc_conversion_o/perc_conv_o_sum,
                                                                              perc_conversion_o/perc_conv_o_sum),
                                                                       ifelse(nsign==2,-perc_conversion_o/perc_conv_o_sum,perc_conversion_o/perc_conv_o_sum))))) %>%
  group_by(NAME,mb_region2,year_o,year,Class_o,ES2) %>%
  mutate(perc_conv_o_scaled_sum=sum(perc_conversion_o_scaled,na.rm=T)) 
## then the converted class
lulccAGG_ES2 <- bind_rows(lulccAGG_ES2 %>% ungroup() %>%
                                filter(is.finite(perc_conversion_conv_scaled)),
                              lulccAGG_ES2 %>% ungroup() %>%
                                filter(!is.finite(perc_conversion_conv_scaled)) %>%
                                group_by(NAME,mb_region2,year_o,year,Class_conv,ES2) %>%
                                mutate(perc_conv_conv_sum = ifelse(perc_conversion_conv>0,sum(perc_conversion_conv[perc_conversion_conv>0],na.rm=T),
                                                                sum(perc_conversion_conv[perc_conversion_conv<0],na.rm=T)),
                                       nsign = sum(c(any(perc_conversion_conv<0),any(perc_conversion_conv>0)),na.rm=T),
                                       perc_conversion_conv_scaled = ifelse(perc_conv_conv_sum==0,0,ifelse(perc_conversion_conv>0,
                                                                                                     ifelse(nsign==2,2*perc_conversion_conv/perc_conv_conv_sum,
                                                                                                            perc_conversion_conv/perc_conv_conv_sum),
                                                                                                     ifelse(nsign==2,-perc_conversion_conv/perc_conv_conv_sum,perc_conversion_conv/perc_conv_conv_sum))))) %>%
  group_by(NAME,mb_region2,year_o,year,Class_conv,ES2) %>%
  mutate(perc_conv_conv_scaled_sum=sum(perc_conversion_conv_scaled,na.rm=T)) 



##  calculate the ES score
##  first of the total for the original class in the original year
##  this is using the regional values for the focal systems and no randomization around th sd
lulccAGG_ES2$ES_o_y1 <- (lulccAGG_ES2$area_o_o*lulccAGG_ES2$Score_o_Regional)+(lulccAGG_ES2$area_o_o*lulccAGG_ES2$Score_o_Regional*lulccAGG_ES2$Conn_o_y1)
##  to make sure we are still counting the same areas before all of the data manipulation, calculate only the area conversions as well
lulccAGG_ES2$ES_o_y2 <- ((lulccAGG_ES2$area_o_o+lulccAGG_ES2$delArea_o)*lulccAGG_ES2$Score_o_Regional)+((lulccAGG_ES2$area_o_o+lulccAGG_ES2$delArea_o)*lulccAGG_ES2$Score_o_Regional*lulccAGG_ES2$Conn_o_y2)

##  now the converted class in the original year
lulccAGG_ES2$ES_conv_y1 <- (lulccAGG_ES2$area_o_conv*lulccAGG_ES2$Score_new_Regional)+(lulccAGG_ES2$area_o_conv*lulccAGG_ES2$Score_new_Regional*lulccAGG_ES2$Conn_conv_y1)
lulccAGG_ES2$ES_conv_y2 <- ((lulccAGG_ES2$area_o_conv+lulccAGG_ES2$delArea_conv)*lulccAGG_ES2$Score_new_Regional)+((lulccAGG_ES2$area_o_conv+lulccAGG_ES2$delArea_conv)*lulccAGG_ES2$Score_new_Regional*lulccAGG_ES2$Conn_conv_y2)

####  now randomize the score around the mean and standard deviation
###  every raster cell will be given a different score based on these mean and sd
###  this takes around 6 hours on a decent computer
###  so save the results afterwards to avoid having to do it multiple times
# library(Rcpp)
# source("./Data/Input/rcpp.R")
# pb <- txtProgressBar(min = 1, max = 100,style = 3)
# ES_o1 <- ES_o2 <- ES_conv1 <- ES_conv2 <- list()
# for (i in 1:100){
#   ES_o1[[i]] <- ESDist(lulccAGG_ES2$area_o_o,lulccAGG_ES2$Score_o_Aggregate,lulccAGG_ES2$sd_o,lulccAGG_ES2$Conn_o_y1)
#   ES_o2[[i]] <- ESDist(lulccAGG_ES2$area_o_o+lulccAGG_ES2$delArea_o,lulccAGG_ES2$Score_o_Aggregate,lulccAGG_ES2$sd_o,lulccAGG_ES2$Conn_o_y2)
#   ES_conv1[[i]] <- ESDist(lulccAGG_ES2$area_o_conv,lulccAGG_ES2$Score_new_Aggregate,lulccAGG_ES2$sd_new,lulccAGG_ES2$Conn_conv_y1)
#   ES_conv2[[i]] <- ESDist(lulccAGG_ES2$area_o_conv+lulccAGG_ES2$delArea_conv,lulccAGG_ES2$Score_new_Aggregate,lulccAGG_ES2$sd_new,lulccAGG_ES2$Conn_conv_y2)
#   Sys.sleep(0.1); setTxtProgressBar(pb,i)
# }
# saveRDS(ES_o1,"./Data/Intermediate/ES_o1b.rds")
# saveRDS(ES_conv1,"./Data/Intermediate/ES_conv1b.rds")
# saveRDS(ES_o2,"./Data/Intermediate/ES_o2b.rds")
# saveRDS(ES_conv2,"./Data/Intermediate/ES_conv2b.rds")

###  once the above step is done, we can just read in the results
ES_o1 <- readRDS("./Data/Intermediate/ES_o1b.rds") 
ES_conv1 <- readRDS("./Data/Intermediate/ES_conv1b.rds") 
ES_o2 <- readRDS("./Data/Intermediate/ES_o2b.rds") 
ES_conv2 <- readRDS("./Data/Intermediate/ES_conv2b.rds") 


### get the means and standard deviations of the 100 runs with the random numbers
##  store these in a new data frame to keep more organized
lulccAGG_ES2_rand <- lulccAGG_ES2
lulccAGG_ES2_rand$ES_o_y1_mean <- rowMeans(data.frame(ES_o1))
lulccAGG_ES2_rand$ES_o_y1_sd <- apply(data.frame(ES_o1), 1, sd)
lulccAGG_ES2_rand$ES_o_y1_max <- lulccAGG_ES2_rand$ES_o_y1_mean+lulccAGG_ES2_rand$ES_o_y1_sd
lulccAGG_ES2_rand$ES_o_y1_min <- lulccAGG_ES2_rand$ES_o_y1_mean-lulccAGG_ES2_rand$ES_o_y1_sd
lulccAGG_ES2_rand$ES_o_y2_mean <- rowMeans(data.frame(ES_o2))
lulccAGG_ES2_rand$ES_o_y2_sd <- apply(data.frame(ES_o2), 1, sd)
lulccAGG_ES2_rand$ES_o_y2_max <- lulccAGG_ES2_rand$ES_o_y2_mean+lulccAGG_ES2_rand$ES_o_y2_sd
lulccAGG_ES2_rand$ES_o_y2_min <- lulccAGG_ES2_rand$ES_o_y2_mean-lulccAGG_ES2_rand$ES_o_y2_sd
lulccAGG_ES2_rand$ES_conv_y1_mean <- rowMeans(data.frame(ES_conv1))
lulccAGG_ES2_rand$ES_conv_y1_sd <- apply(data.frame(ES_conv1), 1, sd)
lulccAGG_ES2_rand$ES_conv_y1_max <- lulccAGG_ES2_rand$ES_conv_y1_mean+lulccAGG_ES2_rand$ES_conv_y1_sd
lulccAGG_ES2_rand$ES_conv_y1_min <- lulccAGG_ES2_rand$ES_conv_y1_mean-lulccAGG_ES2_rand$ES_conv_y1_sd
lulccAGG_ES2_rand$ES_conv_y2_mean <- rowMeans(data.frame(ES_conv2))
lulccAGG_ES2_rand$ES_conv_y2_sd <- apply(data.frame(ES_conv2), 1, sd)
lulccAGG_ES2_rand$ES_conv_y2_max <- lulccAGG_ES2_rand$ES_conv_y2_mean+lulccAGG_ES2_rand$ES_conv_y2_sd
lulccAGG_ES2_rand$ES_conv_y2_min <- lulccAGG_ES2_rand$ES_conv_y2_mean-lulccAGG_ES2_rand$ES_conv_y2_sd
lulccAGG_ES2_rand$ES_o_y1_min[lulccAGG_ES2_rand$ES_o_y1_min<0] = 0
lulccAGG_ES2_rand$ES_o_y2_min[lulccAGG_ES2_rand$ES_o_y2_min<0] = 0
lulccAGG_ES2_rand$ES_conv_y1_min[lulccAGG_ES2_rand$ES_conv_y1_min<0] = 0
lulccAGG_ES2_rand$ES_conv_y2_min[lulccAGG_ES2_rand$ES_conv_y2_min<0] = 0
remove(ES_o1,ES_conv1,ES_o2,ES_conv2)

##  the conversion due to the area and connectivity transition is as follows:
lulccAGG_ES2 <- lulccAGG_ES2 %>%
  filter() %>%
  mutate(delES_o_area =area_converted*Score_o_Regional,
         delES_conv_area = -area_converted*Score_new_Regional,
         delES_o_conn = ((area_o_o*Score_o_Regional*(Conn_o_y2-Conn_o_y1))+(delArea_o*Score_o_Regional*Conn_o_y2))*perc_conversion_o_scaled,
         delES_conv_conn = ((area_o_conv*Score_new_Regional*(Conn_conv_y2-Conn_conv_y1))+(delArea_conv*Score_new_Regional*Conn_conv_y2))*perc_conversion_conv_scaled)
lulccAGG_ES2_rand <- lulccAGG_ES2_rand %>%
  mutate(delES_o_area.max =area_converted*(Score_o_Aggregate+sd_o),
         delES_o_area.mean =area_converted*(Score_o_Aggregate),
         delES_o_area.min =area_converted*(Score_o_Aggregate-sd_o),
         delES_conv_area.max = -area_converted*(Score_new_Aggregate+sd_new),
         delES_conv_area.mean = -area_converted*(Score_new_Aggregate),
         delES_conv_area.min = -area_converted*(Score_new_Aggregate-sd_new),
         delES_o_conn.max = ((area_o_o*(Score_o_Aggregate+sd_o)*(Conn_o_y2-Conn_o_y1))+(delArea_o*(Score_o_Aggregate+sd_o)*Conn_o_y2))*perc_conversion_o_scaled,
         delES_o_conn.mean = ((area_o_o*(Score_o_Aggregate)*(Conn_o_y2-Conn_o_y1))+(delArea_o*(Score_o_Aggregate)*Conn_o_y2))*perc_conversion_o_scaled,
         delES_o_conn.min = ((area_o_o*(Score_o_Aggregate-sd_o)*(Conn_o_y2-Conn_o_y1))+(delArea_o*(Score_o_Aggregate-sd_o)*Conn_o_y2))*perc_conversion_o_scaled,
         delES_conv_conn.max = ((area_o_conv*(Score_new_Aggregate+sd_new)*(Conn_conv_y2-Conn_conv_y1))+(delArea_conv*(Score_new_Aggregate+sd_new)*Conn_conv_y2))*perc_conversion_conv_scaled,
         delES_conv_conn.mean = ((area_o_conv*(Score_new_Aggregate)*(Conn_conv_y2-Conn_conv_y1))+(delArea_conv*(Score_new_Aggregate)*Conn_conv_y2))*perc_conversion_conv_scaled,
         delES_conv_conn.min = ((area_o_conv*(Score_new_Aggregate-sd_new)*(Conn_conv_y2-Conn_conv_y1))+(delArea_conv*(Score_new_Aggregate-sd_new)*Conn_conv_y2))*perc_conversion_conv_scaled)

###  set the "Area Only" values as just the change in area, regardless of the score or the connectivity
lulccAGG_ES2$delES_o_area[which(lulccAGG_ES2$ES2=="Area Only")]  <- lulccAGG_ES2$area_converted[which(lulccAGG_ES2$ES2=="Area Only")] 
lulccAGG_ES2$delES_conv_area[which(lulccAGG_ES2$ES2=="Area Only")]  <- -lulccAGG_ES2$area_converted[which(lulccAGG_ES2$ES2=="Area Only")] 
lulccAGG_ES2_rand$delES_o_area.max[which(lulccAGG_ES2_rand$ES2=="Area Only")]  <- lulccAGG_ES2_rand$area_converted[which(lulccAGG_ES2_rand$ES2=="Area Only")]
lulccAGG_ES2_rand$delES_o_area.mean[which(lulccAGG_ES2_rand$ES2=="Area Only")]  <- lulccAGG_ES2_rand$area_converted[which(lulccAGG_ES2_rand$ES2=="Area Only")]
lulccAGG_ES2_rand$delES_o_area.min[which(lulccAGG_ES2_rand$ES2=="Area Only")]  <- lulccAGG_ES2_rand$area_converted[which(lulccAGG_ES2_rand$ES2=="Area Only")]
lulccAGG_ES2_rand$delES_conv_area.max[which(lulccAGG_ES2_rand$ES2=="Area Only")]  <- -lulccAGG_ES2_rand$area_converted[which(lulccAGG_ES2_rand$ES2=="Area Only")] 
lulccAGG_ES2_rand$delES_conv_area.mean[which(lulccAGG_ES2_rand$ES2=="Area Only")]  <- -lulccAGG_ES2_rand$area_converted[which(lulccAGG_ES2_rand$ES2=="Area Only")] 
lulccAGG_ES2_rand$delES_conv_area.min[which(lulccAGG_ES2_rand$ES2=="Area Only")]  <- -lulccAGG_ES2_rand$area_converted[which(lulccAGG_ES2_rand$ES2=="Area Only")] 


###  group to the same region, summing the individual communities
###  creting the total change in services, delES, which is a combination of the area and connecitivity
###  and eliminating the unnecessary variables
##  also, if the ES value is less than 0.1, it should be 0, this eliminates the tiny irrelevant numbers
lulccAGG_ES2 <- lulccAGG_ES2 %>%
  group_by(Class_o,NAME,mb_region2,year,year_o,ES2) %>%
  mutate(delES_o = sum(delES_o_area,delES_o_conn,na.rm=T),
         delES_o = ifelse(delES_o<.1&delES_o>-0.1,0,delES_o)) %>%
  group_by(Class_conv,NAME,mb_region2,year,year_o,ES2) %>%
  mutate(delES_conv = sum(delES_conv_area,delES_conv_conn,na.rm=T),
         delES_conv = ifelse(delES_conv<.1&delES_conv>-0.1,0,delES_conv))%>%
  ungroup()%>%
  mutate(delES = delES_o+delES_conv)

lulccAGG_ES2_rand <- lulccAGG_ES2_rand %>%
  group_by(Class_o,NAME,mb_region2,year,year_o,ES2) %>%
  mutate(delES_o.max = sum(delES_o_area.max,delES_o_conn.max,na.rm=T),
         delES_o.mean = sum(delES_o_area.mean,delES_o_conn.mean,na.rm=T),
         delES_o.min = sum(delES_o_area.min,delES_o_conn.min,na.rm=T),
         delES_o.max = ifelse(delES_o.max<.1&delES_o.max>-0.1,0,delES_o.max),
         delES_o.mean = ifelse(delES_o.mean<.1&delES_o.mean>-0.1,0,delES_o.mean),
         delES_o.min = ifelse(delES_o.min<.1&delES_o.min>-0.1,0,delES_o.min)) %>%
  group_by(Class_conv,NAME,mb_region2,year,year_o,ES2) %>%
  mutate(delES_conv.max = sum(delES_conv_area.max,delES_conv_conn.max,na.rm=T),
         delES_conv.mean = sum(delES_conv_area.mean,delES_conv_conn.mean,na.rm=T),
         delES_conv.min = sum(delES_conv_area.min,delES_conv_conn.min,na.rm=T),
         delES_conv.max = ifelse(delES_conv.max<.1&delES_conv.max>-0.1,0,delES_conv.max),
         delES_conv.mean = ifelse(delES_conv.mean<.1&delES_conv.mean>-0.1,0,delES_conv.mean),
         delES_conv.min = ifelse(delES_conv.min<.1&delES_conv.min>-0.1,0,delES_conv.min))%>%
  ungroup()%>%
  mutate(delES.max = delES_o.max+delES_conv.max,
         delES.mean = delES_o.mean+delES_conv.mean,
         delES.min = delES_o.min+delES_conv.min)


## check the delES
##  make sure the change in ES is the same as the year 2 minus year 1 values, even though it was not calculated in that way
##  should be the same, a straight line with a slope of 1
plot(lulccAGG_ES2$delES_o[lulccAGG_ES2$Class_conv!=lulccAGG_ES2$Class_o],
     lulccAGG_ES2$ES_o_y2[lulccAGG_ES2$Class_conv!=lulccAGG_ES2$Class_o]-
       lulccAGG_ES2$ES_o_y1[lulccAGG_ES2$Class_conv!=lulccAGG_ES2$Class_o])
abline(a=0,b=1)
##  there are some differences, these are due to the seagrass missing from 2006 in the two regions
plot(lulccAGG_ES2$delES_o[lulccAGG_ES2$Class_conv!=lulccAGG_ES2$Class_o&!(lulccAGG_ES2$mb_region2 %in% c("Cape Cod","Lower North Shore")&lulccAGG_ES2$Class_o==23)],
     lulccAGG_ES2$ES_o_y2[lulccAGG_ES2$Class_conv!=lulccAGG_ES2$Class_o&!(lulccAGG_ES2$mb_region2 %in% c("Cape Cod","Lower North Shore")&lulccAGG_ES2$Class_o==23)]-
       lulccAGG_ES2$ES_o_y1[lulccAGG_ES2$Class_conv!=lulccAGG_ES2$Class_o&!(lulccAGG_ES2$mb_region2 %in% c("Cape Cod","Lower North Shore")&lulccAGG_ES2$Class_o==23)])


##  what is the difference between the calculated value and the actual value
lulccAGG_ES2$diff <- lulccAGG_ES2$delES_o/(lulccAGG_ES2$ES_o_y2-lulccAGG_ES2$ES_o_y1)
summary(lulccAGG_ES2$diff[abs(lulccAGG_ES2$delES_o)>1])
##  perfect
## check the converted class
lulccAGG_ES2$diff <- lulccAGG_ES2$delES_conv/(lulccAGG_ES2$ES_conv_y2-lulccAGG_ES2$ES_conv_y1)
summary(lulccAGG_ES2$diff[abs(lulccAGG_ES2$delES_conv)>1])
summary(lulccAGG_ES2$diff[!lulccAGG_ES2$Class_conv %in% c(26,28)&abs(lulccAGG_ES2$delES_conv)>1])
### for the converted class, there are a few that are off
##  this is ok because we consolidated the non-focal classes and we are really only interested 
##  in the focal classes of Class_o
plot(lulccAGG_ES2$delES_conv[lulccAGG_ES2$Class_conv!=lulccAGG_ES2$Class_o&lulccAGG_ES2$Class_conv %in% c(18,18.1,18.2,19,23)],
     lulccAGG_ES2$ES_conv_y2[lulccAGG_ES2$Class_conv!=lulccAGG_ES2$Class_o&lulccAGG_ES2$Class_conv %in% c(18,18.1,18.2,19,23)]-
       lulccAGG_ES2$ES_conv_y1[lulccAGG_ES2$Class_conv!=lulccAGG_ES2$Class_o&lulccAGG_ES2$Class_conv %in% c(18,18.1,18.2,19,23)])
###  looks great
##  check the random numbers
plot(lulccAGG_ES2_rand$delES_o.mean[lulccAGG_ES2_rand$Class_conv!=lulccAGG_ES2_rand$Class_o],
     lulccAGG_ES2_rand$ES_o_y2_mean[lulccAGG_ES2_rand$Class_conv!=lulccAGG_ES2_rand$Class_o]-
       lulccAGG_ES2_rand$ES_o_y1_mean[lulccAGG_ES2_rand$Class_conv!=lulccAGG_ES2_rand$Class_o])
###  looks great!
plot(lulccAGG_ES2_rand$delES_conv.mean[lulccAGG_ES2_rand$Class_conv!=lulccAGG_ES2_rand$Class_o&lulccAGG_ES2_rand$Class_conv %in% c(18,18.1,18.2,19,23)],
     lulccAGG_ES2_rand$ES_conv_y2_mean[lulccAGG_ES2_rand$Class_conv!=lulccAGG_ES2_rand$Class_o&lulccAGG_ES2_rand$Class_conv %in% c(18,18.1,18.2,19,23)]-
       lulccAGG_ES2_rand$ES_conv_y1_mean[lulccAGG_ES2_rand$Class_conv!=lulccAGG_ES2_rand$Class_o&lulccAGG_ES2_rand$Class_conv %in% c(18,18.1,18.2,19,23)])
## looks great!

## add connectivity
lulccAGG_ES2$delES_perc_o <- 100*(lulccAGG_ES2$delES_o_area+lulccAGG_ES2$delES_o_conn)/(lulccAGG_ES2$ES_o_y1)
lulccAGG_ES2$delES_perc <- 100*(lulccAGG_ES2$delES_o_area+lulccAGG_ES2$delES_conv_area+lulccAGG_ES2$delES_o_conn+lulccAGG_ES2$delES_conv_conn)/(lulccAGG_ES2$ES_o_y1+lulccAGG_ES2$ES_conv_y1)
lulccAGG_ES2$area_converted_perc <- 100*lulccAGG_ES2$area_converted/(lulccAGG_ES2$area_o_o+lulccAGG_ES2$area_o_conv)
##  include the variability from the bootstrapped iterations
lulccAGG_ES2_rand$delES_perc_o.max <- 100*(lulccAGG_ES2_rand$delES_o_area.max+lulccAGG_ES2_rand$delES_o_conn.max)/(lulccAGG_ES2_rand$ES_o_y1_mean+lulccAGG_ES2_rand$sd_o)
lulccAGG_ES2_rand$delES_perc_o.mean <- 100*(lulccAGG_ES2_rand$delES_o_area.mean+lulccAGG_ES2_rand$delES_o_conn.mean)/(lulccAGG_ES2_rand$ES_o_y1_mean)
lulccAGG_ES2_rand$delES_perc_o.min <- 100*(lulccAGG_ES2_rand$delES_o_area.min+lulccAGG_ES2_rand$delES_o_conn.min)/(lulccAGG_ES2_rand$ES_o_y1_mean-lulccAGG_ES2_rand$sd_o)
lulccAGG_ES2_rand$delES_perc.max <- 100*(lulccAGG_ES2_rand$delES_o_area.max+lulccAGG_ES2_rand$delES_conv_area.max+lulccAGG_ES2_rand$delES_o_conn.max+lulccAGG_ES2_rand$delES_conv_conn.max)/(lulccAGG_ES2_rand$ES_o_y1_max+lulccAGG_ES2_rand$ES_conv_y1_max)
lulccAGG_ES2_rand$delES_perc.mean <- 100*(lulccAGG_ES2_rand$delES_o_area.mean+lulccAGG_ES2_rand$delES_conv_area.mean+lulccAGG_ES2_rand$delES_o_conn.mean+lulccAGG_ES2_rand$delES_conv_conn.mean)/(lulccAGG_ES2_rand$ES_o_y1_mean+lulccAGG_ES2_rand$ES_conv_y1)
lulccAGG_ES2_rand$delES_perc.min <- 100*(lulccAGG_ES2_rand$delES_o_area.min+lulccAGG_ES2_rand$delES_conv_area.min+lulccAGG_ES2_rand$delES_o_conn.min+lulccAGG_ES2_rand$delES_conv_conn.min)/(lulccAGG_ES2_rand$ES_o_y1_min+lulccAGG_ES2_rand$ES_conv_y1_min)

lulccAGG_ES2_rand$area_converted_perc <- 100*lulccAGG_ES2$area_converted/(lulccAGG_ES2$area_o_o+lulccAGG_ES2$area_o_conv)



###  aggregate the converted classes to simplify a bit
##  different urban classes become one (2) and 
##  everything but water, urban, 20 and 17, and the focal habitats become grouped as 27 or 'all other'
lulccAGG_ES3 <- lulccAGG_ES2 %>%
  mutate(Class_conv2 =ifelse(Class_conv %in% c(2:5),2,Class_conv)) %>%
  mutate(is.focal=Class_conv %in% c(18,18.1,18.2,18.3,19,21,23,2:5,20,17),
         Class_conv2 =ifelse(is.focal,Class_conv2,27)) %>%
  group_by(NAME,mb_region2,Class_conv2,ES2,year,year_o) %>%
  mutate(area_o_conv=sum(area_o_conv[!duplicated(Class_conv)]),
         ES_conv_y1=sum(ES_conv_y1[!duplicated(Class_conv)]),
         Class_conv =ifelse(Class_conv %in% c(2:5),Class_conv2,Class_conv),
         Class_conv =ifelse(is.focal,Class_conv,Class_conv2)) %>%
  ###   regular sum of total service value and annual change
  group_by(NAME,mb_region2,Class_o,Class_conv,ES2,year,year_o) %>%
  summarise(area_o_o = first(area_o_o),
            area_o_conv = first(area_o_conv),
            area_converted = sum(area_converted,na.rm=T),
            ES_conv_y1=first(ES_conv_y1),
            ES_o_y1=first(ES_o_y1),
            delES_o_area = sum(delES_o_area,na.rm=T),
            delES_conv_area = sum(delES_conv_area,na.rm=T),
            delES_o_conn = sum(delES_o_conn, na.rm=T),
            delES_conv_conn = sum(delES_conv_conn, na.rm=T))

###  same for the random bootstrapped values
lulccAGG_ES3_rand <- lulccAGG_ES2_rand %>%
  mutate(Class_conv2 =ifelse(Class_conv %in% c(2:5),2,Class_conv)) %>%
  mutate(is.focal=Class_conv %in% c(18,18.1,18.2,18.3,19,21,23,2:5,20,17),
         Class_conv2 =ifelse(is.focal,Class_conv2,27)) %>%
  group_by(NAME,mb_region2,Class_conv2,ES2,year,year_o) %>%
  mutate(area_o_conv=sum(area_o_conv[!duplicated(Class_conv)]),
         ES_conv_y1_mean=sum(ES_conv_y1_mean[!duplicated(Class_conv)]),
         ES_conv_y1_max=sum(ES_conv_y1_max[!duplicated(Class_conv)]),
         ES_conv_y1_min=sum(ES_conv_y1_min[!duplicated(Class_conv)]),
         Class_conv =ifelse(Class_conv %in% c(2:5),Class_conv2,Class_conv),
         Class_conv =ifelse(is.focal,Class_conv,Class_conv2)) %>%
  ###   regular sum of total service value and annual change
  group_by(NAME,mb_region2,Class_o,Class_conv,ES2,year,year_o) %>%
  summarise(area_o_o = first(area_o_o),
            area_o_conv = first(area_o_conv),
            area_converted = sum(area_converted,na.rm=T),
            ES_conv_y1_mean=first(ES_conv_y1_mean),
            ES_conv_y1_min=first(ES_conv_y1_min),
            ES_conv_y1_max=first(ES_conv_y1_max),
            ES_o_y1_mean=first(ES_o_y1_mean),
            ES_o_y1_min=first(ES_o_y1_min),
            ES_o_y1_max=first(ES_o_y1_max),
            delES_o_area.max = sum(delES_o_area.max,na.rm=T),
            delES_o_area.mean = sum(delES_o_area.mean,na.rm=T),
            delES_o_area.min = sum(delES_o_area.min,na.rm=T),
            delES_conv_area.max = sum(delES_conv_area.max,na.rm=T),
            delES_conv_area.mean = sum(delES_conv_area.mean,na.rm=T),
            delES_conv_area.min = sum(delES_conv_area.min,na.rm=T),
            delES_o_conn.max = sum(delES_o_conn.max, na.rm=T),
            delES_o_conn.mean = sum(delES_o_conn.mean, na.rm=T),
            delES_o_conn.min = sum(delES_o_conn.min, na.rm=T),
            delES_conv_conn.max = sum(delES_conv_conn.max, na.rm=T),
            delES_conv_conn.mean = sum(delES_conv_conn.mean, na.rm=T),
            delES_conv_conn.min = sum(delES_conv_conn.min, na.rm=T))

##  now group all the focals as 'all focal'
lulccAGG_ES3 <- bind_rows(lulccAGG_ES3,
                         lulccAGG_ES3 %>%
                           filter(Class_o %in% c(18,18.1,18.2,18.3,19,23)) %>%
                           ###  sum of total focal value _0
                           group_by(NAME,mb_region2,ES2,year,year_o) %>%
                           mutate(dup=duplicated(Class_o)) %>%
                           mutate(Class_o=26,
                                  ES_o_y1=sum(ES_o_y1[dup==F]),
                                  area_o_o = sum(area_o_o[dup==F],na.rm=T)) %>%
                           ###   sum of annual change
                           group_by(NAME,mb_region2,Class_o,Class_conv,ES2,year,year_o) %>%
                           summarise(area_o_o = first(area_o_o),
                                     area_o_conv = first(area_o_conv),
                                     area_converted = sum(area_converted,na.rm=T),
                                     ES_conv_y1=first(ES_conv_y1),
                                     ES_o_y1=first(ES_o_y1),
                                     delES_o_area = sum(delES_o_area,na.rm=T),
                                     delES_conv_area = sum(delES_conv_area,na.rm=T),
                                     delES_o_conn = sum(delES_o_conn, na.rm=T),
                                     delES_conv_conn = sum(delES_conv_conn, na.rm=T))) %>%
                           mutate(delES_perc = 100*(delES_o_area+delES_conv_area+delES_o_conn+delES_conv_conn)/(ES_o_y1+ES_conv_y1),
                                  area_converted_perc = 100*area_converted/(area_o_o+area_o_conv))
lulccAGG_ES3_rand <- bind_rows(lulccAGG_ES3_rand,
                          lulccAGG_ES3_rand %>%
                            filter(Class_o %in% c(18,18.1,18.2,18.3,19,23)) %>%
                            ###  sum of total focal value _0
                            group_by(NAME,mb_region2,ES2,year,year_o) %>%
                            mutate(dup=duplicated(Class_o)) %>%
                            mutate(Class_o=26,
                                   ES_o_y1_max=sum(ES_o_y1_max[dup==F]),
                                   ES_o_y1_mean=sum(ES_o_y1_mean[dup==F]),
                                   ES_o_y1_min=sum(ES_o_y1_min[dup==F]),
                                   area_o_o = sum(area_o_o[dup==F],na.rm=T)) %>%
                            ###   sum of annual change
                            group_by(NAME,mb_region2,Class_o,Class_conv,ES2,year,year_o) %>%
                            summarise(area_o_o = first(area_o_o),
                                      area_o_conv = first(area_o_conv),
                                      area_converted = sum(area_converted,na.rm=T),
                                      ES_conv_y1_max=first(ES_conv_y1_max),
                                      ES_conv_y1_mean=first(ES_conv_y1_mean),
                                      ES_conv_y1_min=first(ES_conv_y1_min),
                                      ES_o_y1_max=first(ES_o_y1_max),
                                      ES_o_y1_mean=first(ES_o_y1_mean),
                                      ES_o_y1_min=first(ES_o_y1_min),
                                      delES_o_area.max = sum(delES_o_area.max,na.rm=T),
                                      delES_o_area.mean = sum(delES_o_area.mean,na.rm=T),
                                      delES_o_area.min = sum(delES_o_area.min,na.rm=T),
                                      delES_conv_area.max = sum(delES_conv_area.max,na.rm=T),
                                      delES_conv_area.mean = sum(delES_conv_area.mean,na.rm=T),
                                      delES_conv_area.min = sum(delES_conv_area.min,na.rm=T),
                                      delES_o_conn.max = sum(delES_o_conn.max, na.rm=T),
                                      delES_o_conn.mean = sum(delES_o_conn.mean, na.rm=T),
                                      delES_o_conn.min = sum(delES_o_conn.min, na.rm=T),
                                      delES_conv_conn.max = sum(delES_conv_conn.max, na.rm=T),
                                      delES_conv_conn.mean = sum(delES_conv_conn.mean, na.rm=T),
                                      delES_conv_conn.min = sum(delES_conv_conn.min, na.rm=T))) %>%
  mutate(delES_perc.max = 100*(delES_o_area.max+delES_conv_area.max+delES_o_conn.max+delES_conv_conn.max)/(ES_o_y1_max+ES_conv_y1_max),
         delES_perc.mean = 100*(delES_o_area.mean+delES_conv_area.mean+delES_o_conn.mean+delES_conv_conn.mean)/(ES_o_y1_mean+ES_conv_y1_mean),
         delES_perc.min = 100*(delES_o_area.min+delES_conv_area.min+delES_o_conn.min+delES_conv_conn.min)/(ES_o_y1_min+ES_conv_y1_min),
         area_converted_perc = 100*area_converted/(area_o_o+area_o_conv))
### check the finals again
##  does the total ES for a class equal the previous years total plus all the conversions
check <- merge(lulccAGG_ES3 %>%ungroup()%>%
                 filter(!Class_o %in% c(2,3,4,5,26,28)) %>%
                 distinct(NAME,mb_region2,Class_o,year_o,ES2,.keep_all = T) %>%
                 dplyr::select(NAME,mb_region2,Class_o,year_o,ES2,ES_o_y1)%>%
                 mutate(ES_o_y1=ifelse(ES_o_y1<15,0,ES_o_y1)),
               lulccAGG_ES3 %>%ungroup()%>% filter(!Class_o %in% c(26)) %>%
                 group_by(NAME,mb_region2,Class_conv,year_o,year,ES2) %>%
                 summarise(ES_conv_y1=first(ES_conv_y1),
                           delES =sum(delES_conv_area+delES_conv_conn))%>%
                 ungroup()%>%
               mutate(ES_conv_y2=ifelse((ES_conv_y1+delES)<15,0,ES_conv_y1+delES)),
               by.x=c("NAME","mb_region2","Class_o","year_o","ES2"),
               by.y=c("NAME","mb_region2","Class_conv","year","ES2"))%>%
  mutate(ES_ocheck=ES_o_y1/ES_conv_y2)
summary(check$ES_ocheck[check$Class_o %in% c(18,18.1,18.2,18.3,19,23)])
summary(check$ES_ocheck)
##  the non focal habitats will not match up using this check, because we have aggregated them to save space
##  but the check value for the focal systems should be 1


##  make the categories easier to plot and then save
lulccAGG_ES3$mb_region2 <- factor(lulccAGG_ES3$mb_region2,levels=c('Upper North Shore','Lower North Shore','Boston',
                                                                 'South Shore','Cape Cod'))
lulccAGG_ES3_rand$mb_region2 <- factor(lulccAGG_ES3_rand$mb_region2,levels=c('All','Upper North Shore','Lower North Shore','Boston',
                                                                   'South Shore','Cape Cod'))
lulccAGG_ES3$ES2[lulccAGG_ES3$ES2=='Climate Change Impacts Mitigation'] <- 'Climate Change\nImpacts\nMitigation'
lulccAGG_ES3_rand$ES2[lulccAGG_ES3_rand$ES2=='Climate Change Impacts Mitigation'] <- 'Climate Change\nImpacts\nMitigation'
lulccAGG_ES3$ES2[lulccAGG_ES3$ES2=='Protection from Extreme Weather Events and Flooding'] <- 'Protection From\nExtreme Weather\nand Flooding'
lulccAGG_ES3_rand$ES2[lulccAGG_ES3_rand$ES2=='Protection from Extreme Weather Events and Flooding'] <- 'Protection From\nExtreme Weather\nand Flooding'
lulccAGG_ES3$ES2[lulccAGG_ES3$ES2=='Edible or Commercially Important Fauna'] <- 'Edible or\nCommercially\nImportant Fauna'
lulccAGG_ES3_rand$ES2[lulccAGG_ES3_rand$ES2=='Edible or Commercially Important Fauna'] <- 'Edible or\nCommercially\nImportant Fauna'
lulccAGG_ES3$ES2[lulccAGG_ES3$ES2=='Fauna Community'] <- 'Fauna\nCommunity'
lulccAGG_ES3_rand$ES2[lulccAGG_ES3_rand$ES2=='Fauna Community'] <- 'Fauna\nCommunity'
lulccAGG_ES3$ES2[lulccAGG_ES3$ES2=='Water Quality'] <-'Water\nQuality'
lulccAGG_ES3_rand$ES2[lulccAGG_ES3_rand$ES2=='Water Quality'] <-'Water\nQuality'
lulccAGG_ES3$ES2[lulccAGG_ES3$ES2=='Water Quantity'] <- 'Water\nQuantity'
lulccAGG_ES3_rand$ES2[lulccAGG_ES3_rand$ES2=='Water Quantity'] <- 'Water\nQuantity'
lulccAGG_ES3$ES2 <- factor(lulccAGG_ES3$ES2,levels=c('All','Area Only','Climate Change\nImpacts\nMitigation',
                                                   'Protection From\nExtreme Weather\nand Flooding',
                                                   'Edible or\nCommercially\nImportant Fauna','Fauna\nCommunity',
                                                   'Water\nQuality','Water\nQuantity'))
lulccAGG_ES3_rand$ES2 <- factor(lulccAGG_ES3_rand$ES2,levels=c('All','Area Only','Climate Change\nImpacts\nMitigation',
                                                     'Protection From\nExtreme Weather\nand Flooding',
                                                     'Edible or\nCommercially\nImportant Fauna','Fauna\nCommunity',
                                                     'Water\nQuality','Water\nQuantity'))
write.csv(lulccAGG_ES3, "./Data/Output/lulccAGG_ES.csv",row.names=F)
write.csv(lulccAGG_ES3_rand, "./Data/Output/lulccAGG_ES_rand.csv",row.names=F)



###  read in the aggregated dataset again
lulccAGG_ES3 <- read.csv("./Data/Output/lulccAGG_ES.csv")
lulccAGG_ES3_rand <- read.csv("./Data/Output/lulccAGG_ES_rand.csv")
NOAACCAP <- read.csv("./Data/Input/CCAP_colors.csv")

## aggregate to "All" regions
lulccAGG_ES3 <- bind_rows(lulccAGG_ES3,
                      lulccAGG_ES3 %>% 
                        ## first the Class_o
                        group_by(mb_region2,NAME,ES2,year_o,year) %>%
                        mutate(dup.o=duplicated(Class_o)) %>%
                        group_by(ES2,year_o,year,Class_o) %>%
                        mutate(area_o_o=sum(area_o_o[!dup.o]),
                               ES_o_y1=sum(ES_o_y1[!dup.o],na.rm=T)) %>%
                        # then the Class_conv
                        group_by(mb_region2,NAME,ES2,year_o,year) %>%
                        mutate(dup.conv=duplicated(Class_conv)) %>%
                        group_by(ES2,year_o,year,Class_conv) %>%
                        mutate(area_o_conv=sum(area_o_conv[!dup.conv],na.rm=T),
                               ES_conv_y1=sum(ES_conv_y1[!dup.conv],na.rm=T)) %>%
                        group_by(ES2,year_o,year,Class_o,Class_conv) %>%
                        summarise(area_o_o=first(area_o_o),
                                  area_o_conv=first(area_o_conv),
                                  ES_o_y1=first(ES_o_y1),
                                  ES_conv_y1=first(ES_conv_y1),
                                  area_converted=sum(area_converted,na.rm=T),
                                  delES_o_area = sum(delES_o_area,na.rm=T),
                                  delES_conv_area = sum(delES_conv_area,na.rm=T),
                                  delES_o_conn = sum(delES_o_conn,na.rm=T),
                                  delES_conv_conn= sum(delES_conv_conn,na.rm=T)) %>%
                        mutate(mb_region2="All",NAME="All",
                               delES_perc = 100*(delES_o_area+delES_conv_area+delES_o_conn+delES_conv_conn)/(ES_o_y1+ES_conv_y1),
                               area_converted_perc = 100*area_converted/(area_o_o+area_o_conv)))
lulccAGG_ES3_rand <- bind_rows(lulccAGG_ES3_rand,
                          lulccAGG_ES3_rand %>% 
                            ## first the Class_o
                            group_by(mb_region2,NAME,ES2,year_o,year) %>%
                            mutate(dup.o=duplicated(Class_o)) %>%
                            group_by(ES2,year_o,year,Class_o) %>%
                            mutate(area_o_o=sum(area_o_o[!dup.o]),
                                   ES_o_y1_mean=sum(ES_o_y1_mean[!dup.o],na.rm=T),
                                   ES_o_y1_max=sum(ES_o_y1_max[!dup.o],na.rm=T),
                                   ES_o_y1_min=sum(ES_o_y1_min[!dup.o],na.rm=T))%>%
                            # then the Class_conv
                            group_by(mb_region2,NAME,ES2,year_o,year) %>%
                            mutate(dup.conv=duplicated(Class_conv)) %>%
                            group_by(ES2,year_o,year,Class_conv) %>%
                            mutate(area_o_conv=sum(area_o_conv[!dup.conv],na.rm=T),
                                   ES_conv_y1_mean=sum(ES_conv_y1_mean[!dup.conv],na.rm=T),
                                   ES_conv_y1_max=sum(ES_conv_y1_max[!dup.conv],na.rm=T),
                                   ES_conv_y1_min=sum(ES_conv_y1_min[!dup.conv],na.rm=T)) %>%
                            group_by(ES2,year_o,year,Class_o,Class_conv) %>%
                            summarise(area_o_o=first(area_o_o),
                                      area_o_conv=first(area_o_conv),
                                      ES_o_y1_mean=first(ES_o_y1_mean),
                                      ES_o_y1_max=first(ES_o_y1_max),
                                      ES_o_y1_min=first(ES_o_y1_min),
                                      ES_conv_y1_mean=first(ES_conv_y1_mean),
                                      ES_conv_y1_max=first(ES_conv_y1_max),
                                      ES_conv_y1_min=first(ES_conv_y1_min),
                                      area_converted=sum(area_converted,na.rm=T),
                                      delES_o_area.max = sum(delES_o_area.max,na.rm=T),
                                      delES_o_area.mean = sum(delES_o_area.mean,na.rm=T),
                                      delES_o_area.min = sum(delES_o_area.min,na.rm=T),
                                      delES_conv_area.max = sum(delES_conv_area.max,na.rm=T),
                                      delES_conv_area.mean = sum(delES_conv_area.mean,na.rm=T),
                                      delES_conv_area.min = sum(delES_conv_area.min,na.rm=T),
                                      delES_o_conn.max = sum(delES_o_conn.max,na.rm=T),
                                      delES_o_conn.mean = sum(delES_o_conn.mean,na.rm=T),
                                      delES_o_conn.min = sum(delES_o_conn.min,na.rm=T),
                                      delES_conv_conn.max= sum(delES_conv_conn.max,na.rm=T),
                                      delES_conv_conn.mean= sum(delES_conv_conn.mean,na.rm=T),
                                      delES_conv_conn.min= sum(delES_conv_conn.min,na.rm=T)) %>%
                            mutate(mb_region2="All",NAME="All",
                                   delES_perc.max = 100*(delES_o_area.max+delES_conv_area.max+delES_o_conn.max+delES_conv_conn.max)/(ES_o_y1_max+ES_conv_y1_max),
                                   delES_perc.mean= 100*(delES_o_area.mean+delES_conv_area.mean+delES_o_conn.mean+delES_conv_conn.mean)/(ES_o_y1_mean+ES_conv_y1_mean),
                                   delES_perc.min = 100*(delES_o_area.min+delES_conv_area.min+delES_o_conn.min+delES_conv_conn.min)/(ES_o_y1_min+ES_conv_y1_min),
                                   area_converted_perc = 100*area_converted/(area_o_o+area_o_conv)))

###  calculate the conversion factors
##  create a model for each habitat-habitat conversion of the area converted versus the change in services
##  find the slope of the model, this is the conversion factor
##  store the values in a table
plotdata <- lulccAGG_ES3 %>% filter(!is.na(delES_perc)&!is.na(area_converted_perc)&mb_region2!="All"&!is.na(Class_o)&!is.na(Class_conv))
plotdata_rand <- lulccAGG_ES3_rand %>% filter(!is.na(delES_perc.mean)&!is.na(area_converted_perc)&mb_region2!="All"&!is.na(Class_o)&!is.na(Class_conv))
plotdata <- plotdata[!(plotdata$year_o==1996&plotdata$year %in% c(1996,2006,2010,2016)),]
plotdata_rand <- plotdata_rand[!(plotdata_rand$year_o==1996&plotdata_rand$year %in% c(1996,2006,2010,2016)),]
plotdata <- bind_rows(plotdata,
                      plotdata %>% filter(!is.na(ES2),ES2!='Area Only') %>%
                            mutate(ES2='All') %>%
                            group_by(mb_region2,NAME,ES2,year,year_o,Class_o,Class_conv) %>%
                            summarise(
                              area_converted_perc=first(area_converted_perc),
                              delES_perc =mean(delES_perc)))
plotdata_rand <- bind_rows(plotdata_rand,
                      plotdata_rand %>% filter(!is.na(ES2),ES2!='Area Only') %>%
                        mutate(ES2='All') %>%
                        group_by(mb_region2,NAME,ES2,year,year_o,Class_o,Class_conv) %>%
                        summarise(
                          area_converted_perc=first(area_converted_perc),
                          delES_perc.mean =mean(delES_perc.mean),
                          delES_perc.min =mean(delES_perc.min),
                          delES_perc.max =mean(delES_perc.max)))
mods <- matrix(nrow=0,ncol=10)
colnames(mods) <- c('Class_o','Class_conv','ES2','p','r2','slope','slope.mean','slope.max','slope.min', 'area')
for (Co in unique(plotdata$Class_o)){
  for (Cc in unique(plotdata$Class_conv)){
    for (ES in unique(plotdata$ES2)){
      test <- plotdata[plotdata$Class_conv==Cc&plotdata$Class_o==Co&plotdata$ES2==ES,]
      test_rand <- plotdata_rand[plotdata_rand$Class_conv==Cc&plotdata_rand$Class_o==Co&plotdata_rand$ES2==ES,]
      test<- test[is.finite(test$area_converted_perc)&is.finite(test$delES_perc),] 
      test_rand<- test_rand[is.finite(test_rand$area_converted_perc)&is.finite(test_rand$delES_perc.mean),] 
      if (nrow(test)>0){
        mod <- lm(delES_perc~area_converted_perc,data=test)
        mod <- summary(mod)
        if (!is.null(mod$fstatistic)){
          p = pf(mod$fstatistic[1],mod$fstatistic[2],mod$fstatistic[3],lower.tail=F)
        }else{p=NA}
  #    }
      #if (nrow(test_rand)>0){
        mod.mean <- summary(lm(delES_perc.mean~area_converted_perc,data=test_rand))
        mod.max <- summary(lm(delES_perc.max~area_converted_perc,data=test_rand))
        mod.min <- summary(lm(delES_perc.min~area_converted_perc,data=test_rand))
        if (!is.null(mod.mean$fstatistic)){
          p_rand = pf(mod.mean$fstatistic[1],mod.mean$fstatistic[2],mod.mean$fstatistic[3],lower.tail=F)
        }else{p=NA}
        mods <- rbind(mods,c(Co,Cc,ES,p,mod$r.squared,-mod$coefficients[2],-mod.mean$coefficients[2],max(-mod.max$coefficients[2],-mod.min$coefficients[2]),min(-mod.min$coefficients[2],-mod.max$coefficients[2]),sum(abs(test$area_converted))/1000000))
      }
    }
  }
}
mods <- data.frame(mods)
mods2 <- mods[mods$Class_o %in% c(18,18.1,18.2,18.3,19,23,26)|mods$Class_conv %in% c(18,18.1,18.2,18.3,19,23,26),]
mods2 <- mods2[which(as.numeric(mods2$p)<0.05&mods2$p>0),]
mods2$slope <- round(as.numeric(mods2$slope),2)
mods2 <- mods2[mods2$ES2!="Area Only",]

###  format and save the table of values
mods2 <- merge(mods2,NOAACCAP[,c('Numeric','Habitat')],by.x='Class_o',by.y='Numeric',all.x=T)
mods2 <- merge(mods2,NOAACCAP[,c('Numeric','Habitat')],by.x='Class_conv',by.y='Numeric',all.x=T)
mods2$Habitat.x <- factor(mods2$Habitat.x ,levels =c("High Elevation Salt Marsh","Low Elevation Salt Marsh","Unclassified Salt Marsh","Seagrass","Tidal Flats"))
modsTab <- mods2 %>% filter(Class_o %in% c(18,18.1,18.2,18.3,19,23)) %>%
  mutate(p2 = ifelse(p<0.001,'***',ifelse(p>0.01,"**",ifelse(p<0.05,'*',''))),
         value=paste0(round(as.numeric(slope.mean),2)," (",round(as.numeric(slope.min),2),":",round(as.numeric(slope.max),2),')',p2)) %>% #,'; ',formatC(as.numeric(area), format = "e", digits = 2))) %>%
           #paste0(round(as.numeric(slope.min),2),":",round(as.numeric(slope.max),2),' (',round(as.numeric(slope.mean),2),')',p2)) %>% #,'; ',formatC(as.numeric(area), format = "e", digits = 2))) %>%
  select(Habitat.y,Habitat.x,value,ES2) %>%
  arrange(ES2,Habitat.x,Habitat.y) %>%
  tidyr::pivot_wider(names_from=Habitat.y,values_from=value,values_fill='--')
modsTab.means <- mods2 %>% filter(Class_o %in% c(18,18.1,18.2,18.3,19,23)) %>%
  select(Habitat.y,Habitat.x,slope.mean,ES2) %>%
  arrange(ES2,Habitat.x,Habitat.y) %>%
  tidyr::pivot_wider(names_from=Habitat.y,values_from=slope.mean,values_fill='--')

modsTab <- modsTab[,c("Habitat.x","ES2","High Elevation Salt Marsh","Low Elevation Salt Marsh","Unclassified Salt Marsh",
                      "Tidal Flats","Seagrass","Open Water","Beach","Estuarine Scrub/Shrub Wetland","Development")]
modsTab.means <- modsTab.means[,c("Habitat.x","ES2","High Elevation Salt Marsh","Low Elevation Salt Marsh","Unclassified Salt Marsh",
                      "Tidal Flats","Seagrass","Open Water","Beach","Estuarine Scrub/Shrub Wetland","Development")]

write.csv(mods2,"./Data/Output/ExchangeRates_rand.csv",row.names=F)
write.csv(modsTab,"./Data/Output/ExchangeRates_formatted_rand.csv",row.names=F)
write.csv(modsTab.means,"./Data/Output/ExchangeRates_formatted_rand.means.csv",row.names=F)


###  Visualize the change in services
##  start with the original ES distributions
##  add the 'all' services
lulccAGG_ES4 <- bind_rows(lulccAGG_ES3,
                         lulccAGG_ES3[lulccAGG_ES3$ES2!='Area Only',] %>%
                           mutate(ES2='All') %>%
                           group_by(mb_region2,NAME,ES2,year,year_o,Class_o,Class_conv) %>%
                           summarise(#area_o_conv = sum(area_o_conv,na.rm=T),
                                     #area_o_o = sum(area_o_o,na.rm=T),
                                     #area_converted=sum(area_converted,na.rm=T),
                                     ES_o_y1=sum(ES_o_y1,na.rm=T),
                                     ES_conv_y1=sum(ES_conv_y1,na.rm=T),
                                     delES_o_area=sum(delES_o_area,na.rm=T),
                                     delES_o_conn=sum(delES_o_conn,na.rm = T),
                                     delES_conv_area=sum(delES_conv_area,na.rm=T),
                                     delES_conv_conn=sum(delES_conv_conn,na.rm = T))) %>%
  ungroup() %>%
  group_by(mb_region2,NAME,ES2,year,year_o,Class_o) %>%
  mutate(ES_o_y2 = ES_o_y1+sum(delES_o_area+delES_o_conn)) %>%
  group_by(mb_region2,NAME,ES2,year,year_o,Class_conv) %>%
  mutate(ES_conv_y2 = ES_conv_y1+sum(delES_conv_area[Class_o!=26]+delES_conv_conn[Class_o!=26]))
lulccAGG_ES4_rand <- bind_rows(lulccAGG_ES3_rand,
                          lulccAGG_ES3_rand[lulccAGG_ES3_rand$ES2!='Area Only',] %>%
                            mutate(ES2='All') %>%
                            group_by(mb_region2,NAME,ES2,year,year_o,Class_o,Class_conv) %>%
                            summarise(
                              ES_o_y1_max=sum(ES_o_y1_max,na.rm=T),
                              ES_o_y1_mean=sum(ES_o_y1_mean,na.rm=T),
                              ES_o_y1_min=sum(ES_o_y1_min,na.rm=T),
                              ES_conv_y1_max=sum(ES_conv_y1_max,na.rm=T),
                              ES_conv_y1_mean=sum(ES_conv_y1_mean,na.rm=T),
                              ES_conv_y1_min=sum(ES_conv_y1_min,na.rm=T),
                              delES_o_area.max=sum(delES_o_area.max,na.rm=T),
                              delES_o_area.mean=sum(delES_o_area.mean,na.rm=T),
                              delES_o_area.min=sum(delES_o_area.min,na.rm=T),
                              delES_o_conn.max=sum(delES_o_conn.max,na.rm = T),
                              delES_o_conn.mean=sum(delES_o_conn.mean,na.rm = T),
                              delES_o_conn.min=sum(delES_o_conn.min,na.rm = T),
                              delES_conv_area.max=sum(delES_conv_area.max,na.rm=T),
                              delES_conv_area.mean=sum(delES_conv_area.mean,na.rm=T),
                              delES_conv_area.min=sum(delES_conv_area.min,na.rm=T),
                              delES_conv_conn.max=sum(delES_conv_conn.max,na.rm = T),
                              delES_conv_conn.mean=sum(delES_conv_conn.mean,na.rm = T),
                              delES_conv_conn.min=sum(delES_conv_conn.min,na.rm = T))) %>%
  ungroup() %>%
  group_by(mb_region2,NAME,ES2,year,year_o,Class_o) %>%
  mutate(ES_o_y2_mean = ES_o_y1_mean+sum(delES_o_area.mean+delES_o_conn.mean),
         ES_o_y2_max = ES_o_y1_max+sum(delES_o_area.max+delES_o_conn.max),
         ES_o_y2_min = ES_o_y1_min+sum(delES_o_area.min+delES_o_conn.min)) %>%
  group_by(mb_region2,NAME,ES2,year,year_o,Class_conv) %>%
  mutate(ES_conv_y2_mean = ES_conv_y1_mean+sum(delES_conv_area.mean[Class_o!=26]+delES_conv_conn.mean[Class_o!=26]),
         ES_conv_y2_max = ES_conv_y1_max+sum(delES_conv_area.max[Class_o!=26]+delES_conv_conn.max[Class_o!=26]),
         ES_conv_y2_min = ES_conv_y1_min+sum(delES_conv_area.min[Class_o!=26]+delES_conv_conn.min[Class_o!=26]))


plotdata <- lulccAGG_ES4 %>% ungroup() %>%
  filter(year==1996,Class_o %in% c(18,18.1,18.2,18.3,19,23)) %>%
  distinct(NAME,mb_region2,Class_o,ES2,.keep_all = T) %>%
  group_by(mb_region2,ES2)%>%
  mutate(ES_T = sum(ES_o_y1),
         areaT = sum(area_o_o)) %>%
  group_by(mb_region2,ES2,Class_o) %>%
  summarise(ES_o_y1= sum(ES_o_y1),
            ES_T=first(ES_T),
            area_o_o= sum(area_o_o),
            areaT=first(areaT)) %>%
  mutate(ES_perc=ES_o_y1/ES_T,
         area_perc=area_o_o/areaT,
         year=1996) %>%
  group_by(mb_region2,ES2) %>%
  arrange(ES_perc) %>%
  mutate(ESrank = n():1)
plotdata_rand <- lulccAGG_ES4_rand[!is.na(lulccAGG_ES4_rand$ES2)&!is.na(lulccAGG_ES4_rand$mb_region2),] %>% ungroup() %>%
  filter(year==1996,Class_o %in% c(18,18.1,18.2,18.3,19,23)) %>%
  distinct(NAME,mb_region2,Class_o,ES2,.keep_all = T) %>%
  group_by(mb_region2,ES2)%>%
  mutate(ES_T_mean = sum(ES_o_y1_mean),
         ES_T_max = sum(ES_o_y1_max),
         ES_T_min = sum(ES_o_y1_min),
         areaT = sum(area_o_o)) %>%
  group_by(mb_region2,ES2,Class_o) %>%
  summarise(ES_o_y1_mean= sum(ES_o_y1_mean),
            ES_o_y1_max= sum(ES_o_y1_max),
            ES_o_y1_min= sum(ES_o_y1_min),
            ES_T_mean=first(ES_T_mean),
            ES_T_max=first(ES_T_max),
            ES_T_min=first(ES_T_min),
            area_o_o= sum(area_o_o),
            areaT=first(areaT)) %>%
  mutate(ES_perc_max=ES_o_y1_max/ES_T_max,
         ES_perc_mean=ES_o_y1_mean/ES_T_mean,
         ES_perc_min=ES_o_y1_min/ES_T_min,
         area_perc=area_o_o/areaT,
         year=1996) %>%
  group_by(mb_region2,ES2) %>%
  arrange(ES_perc_mean) %>%
  mutate(ESrank = n():1)
plotdata <- merge(plotdata,NOAACCAP[,c('Numeric','Habitat')],by.x='Class_o',by.y='Numeric',all.x=T)
plotdata_rand <- merge(plotdata_rand,NOAACCAP[,c('Numeric','Habitat')],by.x='Class_o',by.y='Numeric',all.x=T)

plotdata$Habitat <- factor(plotdata$Habitat,levels=c('High Elevation Salt Marsh','Low Elevation Salt Marsh',
                                                             'Unclassified Salt Marsh','Seagrass','Tidal Flats'))#,
                                                             #'Open Water','Development','Beach','Estuarine Scrub/Shrub Wetland','All Others'))
plotdata_rand$Habitat <- factor(plotdata_rand$Habitat,levels=c('High Elevation Salt Marsh','Low Elevation Salt Marsh',
                                                     'Unclassified Salt Marsh','Seagrass','Tidal Flats'))#,
                                                     #'Open Water','Development','Beach','Estuarine Scrub/Shrub Wetland','All Others'))

plotdata$mb_region2 <- factor(plotdata$mb_region2,levels=c('All','Upper North Shore','Lower North Shore','Boston','South Shore','Cape Cod'))
plotdata_rand$mb_region2 <- factor(plotdata_rand$mb_region2,levels=c('All','Upper North Shore','Lower North Shore','Boston','South Shore','Cape Cod'))

plotdata_rand <- plotdata_rand %>%
  ungroup() %>%
  arrange(desc(Habitat),.by_group = TRUE) %>%
  group_by(mb_region2,ES2) %>%
  mutate(ES_perc_mean_cum=cumsum(ES_perc_mean),
         ES_perc_min_cum=cumsum(ES_perc_min),
         ES_perc_max_cum=cumsum(ES_perc_max))
  

g6 <- ggplot(plotdata_rand[plotdata_rand$ES2!='Area Only',],aes(x=year,y=round(ES_perc_mean*100),fill=Habitat)) +
  facet_grid(ES2~mb_region2) +
  geom_col(col='Black')+
  #error bars are negligible and not worth showing in the plot
  #geom_errorbar(aes(ymin=100*ES_perc_min_cum,ymax=100*ES_perc_max_cum),width=0.05)+
  scale_fill_manual(values=NOAACCAP$cols2[NOAACCAP$Numeric %in% plotdata_rand$Class_o][c(1:3,5,4)])+
  scale_y_continuous(breaks=c(0,50,100))+
  theme_classic()+
  theme(axis.text.x = element_blank(),legend.position='bottom',
        strip.placement = "outside",panel.border = element_blank(),#element_rect(colour = "black", fill=NA, size=1),
        axis.line.x = element_blank(),axis.line.y = element_blank(),strip.text.x = element_text(angle=90),
        strip.background = element_blank(),axis.text = element_text(size=10),strip.text= element_text(size=10))

read_pptx("\\\\ad.uni-hamburg.de\\redir\\redir0010\\bba4705\\Documents\\Template.pptx") %>%
  ph_with(dml(ggobj=g6),location=ph_location(left=0.5, top=0.5,width=4,height=7.5)) %>%
  print(target = "./Tables and Figures/ESG_1996.pptx")
  


## focal ecosystem totals
lulccAGG_ES_totals <- lulccAGG_ES4 %>% ungroup() %>%
  filter(Class_o %in% c(18,18.1,18.2,18.3,19,23)|Class_conv %in% c(18,18.1,18.2,18.3,19,23)) %>%
  filter(!Class_o %in% c(26,28)) %>%
  group_by(mb_region2,year_o,year,ES2) %>% #group_by(mb_region2,Class_o,year_o,year,ES2) %>%
  mutate(focal.o=Class_o %in% c(18,18.1,18.2,18.3,19,23),
         focal.conv=Class_conv %in% c(18,18.1,18.2,18.3,19,23),
         delES_f = sum(delES_conv_area[focal.conv],delES_conv_conn[focal.conv],na.rm = T),
         delES_nf_f = sum(delES_conv_area[!focal.o&focal.conv],delES_conv_conn[!focal.o&focal.conv],na.rm = T),
         delES_f_f = sum(delES_o_area[focal.o&focal.conv],delES_o_conn[focal.o&focal.conv],na.rm = T)) %>%
  ##  get the converted class sums for y2, this is for a check in the next step
  group_by(mb_region2,NAME,year_o,year,ES2) %>%
  mutate(dup_conv =duplicated(cbind(Class_conv,NAME,mb_region2)),
         ES_conv_y2_f= sum(ES_conv_y2[!dup_conv&focal.conv],na.rm = T)) %>% ungroup() %>%
  distinct(Class_o,NAME,mb_region2,year,year_o,ES2,.keep_all = T) %>%
  group_by(mb_region2,year_o,year,ES2) %>%
  mutate(dup_conv =duplicated(NAME)) %>%
  arrange(year) %>%
  summarise(delES_f = first(delES_f),
            delES_nf_f = first(delES_nf_f),
            delES_f_f = first(delES_f_f),
            ES_o_y1_f=sum(ES_o_y1[focal.o],na.rm = T),
            ES_o_y2_f = sum(ES_conv_y2_f[!dup_conv&focal.conv],na.rm = T),
            ES_o_y2_f_check = ES_o_y1_f+delES_f,
            check = ES_o_y2_f/ES_o_y2_f_check) %>%
  group_by(mb_region2,ES2) %>%# group_by(mb_region2,Class_o,year_o,year,ES2) %>%
  arrange(year) %>%
  mutate(is.dup=duplicated(year),
         is.dup.o=duplicated(year_o),
         delES_f_96 = ifelse(is.dup,lag(delES_f),delES_f),
         delES_ff_96 = ifelse(is.dup,lag(delES_f_f),delES_f_f),
         delES_nff_96 = ifelse(is.dup,lag(delES_nf_f),delES_nf_f)) %>%
  dplyr::select(-is.dup,-is.dup.o)
summary(lulccAGG_ES_totals$check)

lulccAGG_ES_totals_rand <- lulccAGG_ES4_rand %>% ungroup() %>%
  filter(Class_o %in% c(18,18.1,18.2,18.3,19,23)|Class_conv %in% c(18,18.1,18.2,18.3,19,23)) %>%
  filter(!Class_o %in% c(26,28)) %>%
  group_by(mb_region2,year_o,year,ES2) %>% #group_by(mb_region2,Class_o,year_o,year,ES2) %>%
  mutate(focal.o=Class_o %in% c(18,18.1,18.2,18.3,19,23),
         focal.conv=Class_conv %in% c(18,18.1,18.2,18.3,19,23),
         delES_f.max = sum(delES_conv_area.max[focal.conv],delES_conv_conn.max[focal.conv],na.rm = T),
         delES_f.mean = sum(delES_conv_area.mean[focal.conv],delES_conv_conn.mean[focal.conv],na.rm = T),
         delES_f.min = sum(delES_conv_area.min[focal.conv],delES_conv_conn.min[focal.conv],na.rm = T),
         delES_nf_f.max = sum(delES_conv_area.max[!focal.o&focal.conv],delES_conv_conn.max[!focal.o&focal.conv],na.rm = T),
         delES_nf_f.mean = sum(delES_conv_area.mean[!focal.o&focal.conv],delES_conv_conn.mean[!focal.o&focal.conv],na.rm = T),
         delES_nf_f.min = sum(delES_conv_area.min[!focal.o&focal.conv],delES_conv_conn.min[!focal.o&focal.conv],na.rm = T),
         delES_f_f.max = sum(delES_o_area.max[focal.o&focal.conv],delES_o_conn.max[focal.o&focal.conv],na.rm = T),
         delES_f_f.mean = sum(delES_o_area.mean[focal.o&focal.conv],delES_o_conn.mean[focal.o&focal.conv],na.rm = T),
         delES_f_f.min = sum(delES_o_area.min[focal.o&focal.conv],delES_o_conn.min[focal.o&focal.conv],na.rm = T)) %>%
  ##  get the converted class sums for y2, this is for a check in the next step
  group_by(mb_region2,NAME,year_o,year,ES2) %>%
  mutate(dup_conv =duplicated(cbind(Class_conv,NAME,mb_region2)),
         ES_conv_y2_f_max= sum(ES_conv_y2_max[!dup_conv&focal.conv],na.rm = T),
         ES_conv_y2_f_mean= sum(ES_conv_y2_mean[!dup_conv&focal.conv],na.rm = T),
         ES_conv_y2_f_min= sum(ES_conv_y2_min[!dup_conv&focal.conv],na.rm = T)) %>% ungroup() %>%
  distinct(Class_o,NAME,mb_region2,year,year_o,ES2,.keep_all = T) %>%
  group_by(mb_region2,year_o,year,ES2) %>%
  mutate(dup_conv =duplicated(NAME)) %>%
  arrange(year) %>%
  summarise(delES_f.max = first(delES_f.max),delES_f.mean = first(delES_f.mean),delES_f.min = first(delES_f.min),
            delES_nf_f.max = first(delES_nf_f.max),delES_nf_f.mean = first(delES_nf_f.mean),delES_nf_f.min = first(delES_nf_f.min),
            delES_f_f.max = first(delES_f_f.max),delES_f_f.mean = first(delES_f_f.mean),delES_f_f.min = first(delES_f_f.min),
            ES_o_y1_f_max=sum(ES_o_y1_max[focal.o],na.rm = T),ES_o_y1_f_mean=sum(ES_o_y1_mean[focal.o],na.rm = T),ES_o_y1_f_min=sum(ES_o_y1_min[focal.o],na.rm = T),
            ES_o_y2_f_max = sum(ES_conv_y2_f_max[!dup_conv&focal.conv],na.rm = T), ES_o_y2_f_mean = sum(ES_conv_y2_f_mean[!dup_conv&focal.conv],na.rm = T), ES_o_y2_f_min = sum(ES_conv_y2_f_min[!dup_conv&focal.conv],na.rm = T),
            ES_o_y2_f_check = ES_o_y1_f_mean+delES_f.mean,
            check = ES_o_y2_f_mean/ES_o_y2_f_check) %>%
  group_by(mb_region2,ES2) %>%# group_by(mb_region2,Class_o,year_o,year,ES2) %>%
  arrange(year) %>%
  mutate(is.dup=duplicated(year),
         is.dup.o=duplicated(year_o),
         delES_f_96.max = ifelse(is.dup,lag(delES_f.max),delES_f.max), delES_f_96.mean = ifelse(is.dup,lag(delES_f.mean),delES_f.mean), delES_f_96.min = ifelse(is.dup,lag(delES_f.min),delES_f.min),
         delES_ff_96.max = ifelse(is.dup,lag(delES_f_f.max),delES_f_f.max),delES_ff_96.mean = ifelse(is.dup,lag(delES_f_f.mean),delES_f_f.mean),delES_ff_96.min = ifelse(is.dup,lag(delES_f_f.min),delES_f_f.min),
         delES_nff_96.max = ifelse(is.dup,lag(delES_nf_f.max),delES_nf_f.max),delES_nff_96.mean = ifelse(is.dup,lag(delES_nf_f.mean),delES_nf_f.mean),delES_nff_96.min = ifelse(is.dup,lag(delES_nf_f.min),delES_nf_f.min)) %>%
  dplyr::select(-is.dup,-is.dup.o)


###  check a given year to make sure the y1 and y2 services totals line up
plot(lulccAGG_ES_totals$ES_o_y2_f[lulccAGG_ES_totals$year_o==2010&!is.na(lulccAGG_ES_totals$ES2)],
     lulccAGG_ES_totals$ES_o_y1_f[lulccAGG_ES_totals$year_o==2016&!is.na(lulccAGG_ES_totals$ES2)])
abline(a=0,b=1)
plot(lulccAGG_ES_totals$ES_o_y2_f[lulccAGG_ES_totals$year_o==2010&!is.na(lulccAGG_ES_totals$ES2)],
     lulccAGG_ES_totals$ES_o_y1_f[lulccAGG_ES_totals$year_o==2016&!is.na(lulccAGG_ES_totals$ES2)])
abline(a=0,b=1)


###  calculate the changes and the percent contribution of each change to the total change
lulccAGG_ES_changes <- lulccAGG_ES4 %>% ungroup() %>%
  filter(Class_o %in% c(18,18.1,18.2,18.3,19,23)|Class_conv %in% c(18,18.1,18.2,18.3,19,23)) %>%
  filter(!Class_o %in% c(26,28)) %>%
  group_by(mb_region2,Class_conv,year_o,year,ES2) %>% ## for delES variables, the Class grouping shouldnt matter, each value is unique
  summarise(delES_o_area=sum(delES_o_area),
            delES_o_conn =sum(delES_o_conn),
            delES_conv_area=sum(delES_conv_area),
            delES_conv_conn =sum(delES_conv_conn),
            delES_o =delES_o_area+delES_o_conn,
            delES_conv =delES_conv_area+delES_conv_conn,
            delES_area = delES_o_area+delES_conv_area,
            delES_conn = delES_o_conn+delES_conv_conn,
            delES =delES_o_area+delES_o_conn+delES_conv_area+delES_conv_conn) %>%
  group_by(mb_region2,ES2) %>%
  arrange(year)

lulccAGG_ES_changes_rand <- lulccAGG_ES4_rand %>% ungroup() %>%
  filter(Class_o %in% c(18,18.1,18.2,18.3,19,23)|Class_conv %in% c(18,18.1,18.2,18.3,19,23)) %>%
  filter(!Class_o %in% c(26,28)) %>%
  group_by(mb_region2,Class_conv,year_o,year,ES2) %>% ## for delES variables, the Class grouping shouldnt matter, each value is unique
  summarise(delES_o_area.max=sum(delES_o_area.max),delES_o_area.mean=sum(delES_o_area.mean),delES_o_area.min=sum(delES_o_area.min),
            delES_o_conn.max =sum(delES_o_conn.max),delES_o_conn.mean =sum(delES_o_conn.mean),delES_o_conn.min =sum(delES_o_conn.min),
            delES_conv_area.max=sum(delES_conv_area.max),delES_conv_area.mean=sum(delES_conv_area.mean),delES_conv_area.min=sum(delES_conv_area.min),
            delES_conv_conn.max =sum(delES_conv_conn.max),delES_conv_conn.mean =sum(delES_conv_conn.mean),delES_conv_conn.min =sum(delES_conv_conn.min),
            delES_o.max =delES_o_area.max+delES_o_conn.max,delES_o.mean =delES_o_area.mean+delES_o_conn.mean,delES_o.min =delES_o_area.min+delES_o_conn.min,
            delES_conv.max =delES_conv_area.max+delES_conv_conn.max,delES_conv.mean =delES_conv_area.mean+delES_conv_conn.mean,delES_conv.min =delES_conv_area.min+delES_conv_conn.min,
            delES_area.max = delES_o_area.max+delES_conv_area.max, delES_area.mean = delES_o_area.mean+delES_conv_area.mean, delES_area.min = delES_o_area.min+delES_conv_area.min,
            delES_conn.max = delES_o_conn.max+delES_conv_conn.max,delES_conn.mean = delES_o_conn.mean+delES_conv_conn.mean,delES_conn.min = delES_o_conn.min+delES_conv_conn.min,
            delES.max =delES_o_area.max+delES_o_conn.max+delES_conv_area.max+delES_conv_conn.max,
            delES.mean =delES_o_area.mean+delES_o_conn.mean+delES_conv_area.mean+delES_conv_conn.mean,
            delES.min =delES_o_area.min+delES_o_conn.min+delES_conv_area.min+delES_conv_conn.min) %>%
  group_by(mb_region2,ES2) %>%
  arrange(year)

###  do the delES values add up? values here should be 1
summary((lulccAGG_ES_changes_rand$delES_conn.mean+lulccAGG_ES_changes_rand$delES_area.mean)/lulccAGG_ES_changes_rand$delES.mean)
summary((lulccAGG_ES_changes_rand$delES_o_conn.mean+lulccAGG_ES_changes_rand$delES_conv_conn.mean)/lulccAGG_ES_changes_rand$delES_conn.mean)
###  if so, what are the overall contributions of connectivity and area?
summary(lulccAGG_ES_changes_rand$delES_conn.mean/lulccAGG_ES_changes_rand$delES.mean)
summary(lulccAGG_ES_changes_rand$delES_area.mean/lulccAGG_ES_changes_rand$delES.mean)
###  create table of the contribution of connectivity versus area 
lulccAGG_ES_changes_rand %>%
  ungroup() %>%
  group_by(mb_region2) %>%
  summarise(conn=median(delES_conn.mean/delES.mean,na.rm=T),
            area=median(delES_area.mean/delES.mean,na.rm=T))



###  merge the totals with the changes
lulccAGG_ES5 <- merge(lulccAGG_ES_totals[,c('mb_region2','ES2','year_o','year','ES_o_y1_f','delES_f','delES_f_96','delES_f_f','delES_ff_96','delES_nf_f','delES_nff_96')],
                      lulccAGG_ES_changes[,c('mb_region2','ES2','year_o','year','Class_conv','delES_conv','delES_conv_area','delES_conv_conn')],
                      by=c('mb_region2','ES2','year_o','year')) %>%
  ##get the 1996 ES value
  group_by(mb_region2,ES2) %>%
  mutate(ES_o_96_f = first(ES_o_y1_f),
         perc_ES_96_f = ((ES_o_y1_f+delES_f)-ES_o_96_f)/ES_o_96_f)%>%
  ## now get the change from 96
  ungroup() %>%
  mutate(perc96_delES_f = delES_ff_96/ES_o_96_f,
         perc96_delES_nff = delES_nff_96/ES_o_96_f,
         perc_totES_converted = delES_conv/ES_o_96_f,
         perc_area = delES_conv_area/delES_conv, perc_conn = delES_conv_conn/delES_conv,
         percOfFocalConversion = ifelse(Class_conv %in% c(18,18.1,18.2,19,23),delES_conv/delES_f,NA))

lulccAGG_ES5_rand <- merge(lulccAGG_ES_totals_rand[,c('mb_region2','ES2','year_o','year','ES_o_y1_f_max','ES_o_y1_f_mean','ES_o_y1_f_min','delES_f.max','delES_f.mean','delES_f.min',
                                                      'delES_f_96.max','delES_f_96.mean','delES_f_96.min','delES_f_f.max','delES_f_f.mean','delES_f_f.min',
                                                      'delES_ff_96.max','delES_ff_96.mean','delES_ff_96.min','delES_nf_f.max','delES_nf_f.mean','delES_nf_f.min',
                                                      'delES_nff_96.max','delES_nff_96.mean','delES_nff_96.min')],
                      lulccAGG_ES_changes_rand[,c('mb_region2','ES2','year_o','year','Class_conv','delES_conv.max','delES_conv.mean','delES_conv.min',
                                             'delES_conv_area.max','delES_conv_area.mean','delES_conv_area.min','delES_conv_conn.max','delES_conv_conn.mean','delES_conv_conn.min')],
                      by=c('mb_region2','ES2','year_o','year')) %>%
  ##get the 1996 ES value
  group_by(mb_region2,ES2) %>%
  mutate(ES_o_96_f_max = first(ES_o_y1_f_max),ES_o_96_f_mean = first(ES_o_y1_f_mean),ES_o_96_f_min = first(ES_o_y1_f_min),
         perc_ES_96_f.max = ((ES_o_y1_f_max+delES_f.max)-ES_o_96_f_max)/ES_o_96_f_max,
         perc_ES_96_f.mean = ((ES_o_y1_f_mean+delES_f.mean)-ES_o_96_f_mean)/ES_o_96_f_mean,
         perc_ES_96_f.min = ((ES_o_y1_f_min+delES_f.min)-ES_o_96_f_min)/ES_o_96_f_min)%>%
  ## now get the change from 96
  ungroup() %>%
  mutate(perc96_delES_f.max = delES_ff_96.max/ES_o_96_f_max,perc96_delES_f.mean = delES_ff_96.mean/ES_o_96_f_mean,perc96_delES_f.min = delES_ff_96.min/ES_o_96_f_min,
         perc96_delES_nff.max = delES_nff_96.max/ES_o_96_f_max,perc96_delES_nff.mean = delES_nff_96.mean/ES_o_96_f_mean,perc96_delES_nff.min = delES_nff_96.min/ES_o_96_f_min,
         perc_totES_converted.max = delES_conv.max/ES_o_96_f_max,perc_totES_converted.mean = delES_conv.mean/ES_o_96_f_mean,perc_totES_converted.min = delES_conv.min/ES_o_96_f_min,
         perc_area.max = delES_conv_area.max/delES_conv.max,perc_area.mean = delES_conv_area.mean/delES_conv.mean,perc_area.min = delES_conv_area.min/delES_conv.min,
         perc_conn.max = delES_conv_conn.max/delES_conv.max,perc_conn.mean = delES_conv_conn.mean/delES_conv.mean,perc_conn.min = delES_conv_conn.min/delES_conv.min,
         percOfFocalConversion.max = ifelse(Class_conv %in% c(18,18.1,18.2,19,23),delES_conv.max/delES_f.max,NA),
         percOfFocalConversion.mean = ifelse(Class_conv %in% c(18,18.1,18.2,19,23),delES_conv.mean/delES_f.mean,NA),
         percOfFocalConversion.min = ifelse(Class_conv %in% c(18,18.1,18.2,19,23),delES_conv.min/delES_f.min,NA))


lulccAGG_ES5 <- merge(lulccAGG_ES5,NOAACCAP[,c('Numeric','Habitat')],by.x='Class_conv',by.y='Numeric',all.x=T)
lulccAGG_ES5_rand <- merge(lulccAGG_ES5_rand,NOAACCAP[,c('Numeric','Habitat')],by.x='Class_conv',by.y='Numeric',all.x=T)
lulccAGG_ES5$Habitat <- factor(lulccAGG_ES5$Habitat,levels=c('High Elevation Salt Marsh','Low Elevation Salt Marsh',
                                                             'Unclassified Salt Marsh','Seagrass','Tidal Flats',
                                                             'Open Water','Development','Beach','Estuarine Scrub/Shrub Wetland','All Others'))
lulccAGG_ES5_rand$Habitat <- factor(lulccAGG_ES5_rand$Habitat,levels=c('High Elevation Salt Marsh','Low Elevation Salt Marsh',
                                                             'Unclassified Salt Marsh','Seagrass','Tidal Flats',
                                                             'Open Water','Development','Beach','Estuarine Scrub/Shrub Wetland','All Others'))
lulccAGG_ES5$mb_region2 <- factor(lulccAGG_ES5$mb_region2,levels=c('All','Upper North Shore','Lower North Shore','Boston','South Shore','Cape Cod'))
lulccAGG_ES5_rand$mb_region2 <- factor(lulccAGG_ES5_rand$mb_region2,levels=c('All','Upper North Shore','Lower North Shore','Boston','South Shore','Cape Cod'))
lulccAGG_ES5$year <- as.numeric(lulccAGG_ES5$year)
lulccAGG_ES5_rand$year <- as.numeric(lulccAGG_ES5_rand$year)

lulccAGG_ES5_rand <- lulccAGG_ES5_rand %>%
  ungroup() %>%
  arrange(desc(Habitat),.by_group = TRUE) %>%
  group_by(mb_region2,ES2,year,year_o) %>%
  mutate(sign=ifelse(perc_totES_converted.mean<0,-1,1),
         perc_totES_converted.min_cum.neg=ifelse(sign==-1,cumsum(perc_totES_converted.min[sign==-1]),NA),
         perc_totES_converted.min_cum.pos=ifelse(sign==1,cumsum(perc_totES_converted.min[sign==1]),NA),
         perc_totES_converted.max_cum.neg=ifelse(sign==-1,cumsum(perc_totES_converted.max[sign==-1]),NA),
         perc_totES_converted.max_cum.pos=ifelse(sign==1,cumsum(perc_totES_converted.max[sign==1]),NA))

g7 <- ggplot(data=lulccAGG_ES5_rand%>%filter(!(year_o==1996&year %in% c(2006,2010,2016)),
                                             !(mb_region2 =="All"&(year==2006|year_o==2006)),ES2!="Area Only"),
              aes(x=year,y=100*perc_totES_converted.mean))+
  facet_grid(ES2~mb_region2,scales = 'free_y') +
  geom_bar(stat='identity',col='black',lwd=0.1,aes(fill=Habitat))+
  geom_errorbar(data=lulccAGG_ES5_rand%>%filter(!(year_o==1996&year %in% c(2006,2010,2016)), 
                                                !(mb_region2 =="All"&(year==2006|year_o==2006)),ES2!="Area Only"),
                position=position_dodge(width=5),
                aes(ymin=100*perc_totES_converted.min_cum.pos, 
                    ymax=100*perc_totES_converted.max_cum.pos),
                color="gray30",stat='identity',width=0.5)+
  geom_errorbar(data=lulccAGG_ES5_rand%>%filter(!(year_o==1996&year %in% c(2006,2010,2016)), 
                                                !(mb_region2 =="All"&(year==2006|year_o==2006)),ES2!="Area Only"),
                position=position_dodge(width=5),
                aes(ymin=100*perc_totES_converted.min_cum.neg, 
                    ymax=100*perc_totES_converted.max_cum.neg),
                color="gray30",stat='identity',width=0.5)+
 # geom_line(data=lulccAGG_ES5[lulccAGG_ES5$year_o==1996,],
   #         aes(x=year,y=100*perc96_delES_f),col='Black',size=1.2)+
 # geom_line(data=lulccAGG_ES5[lulccAGG_ES5$year_o==1996,],
  #          aes(x=year,y=100*perc96_delES_nff),col='Black',size=1.2,lty=2)+
  geom_line(data=lulccAGG_ES5[lulccAGG_ES5$year_o==1996&!(lulccAGG_ES5$mb_region2=="All"&lulccAGG_ES5$year==2006),]%>% filter(ES2!="Area Only"),
            aes(x=year,y=100*perc_ES_96_f),col='Black',size=1)+
  geom_line(data=lulccAGG_ES5_rand[lulccAGG_ES5_rand$year_o==1996&!(lulccAGG_ES5_rand$mb_region2=="All"&lulccAGG_ES5_rand$year==2006),]%>% filter(ES2!="Area Only"),
            aes(x=year,y=100*perc_ES_96_f.mean),col='Black',size=1,lty=2)+
  geom_ribbon(data=lulccAGG_ES5_rand[lulccAGG_ES5_rand$year_o==1996&!(lulccAGG_ES5_rand$mb_region2=="All"&lulccAGG_ES5_rand$year==2006),]%>% filter(ES2!="Area Only"),
             aes(x=year,ymin=100*perc_ES_96_f.min,ymax=100*perc_ES_96_f.max),col='gray30',alpha=0.2)+
  scale_fill_manual(values=NOAACCAP$cols2[NOAACCAP$Numeric %in% lulccAGG_ES5$Class_conv][c(3,4,5,9,6,8,1,7,2,10)])+
  #scale_alpha_manual(values=NOAACCAP$alpha[NOAACCAP$Numeric %in% lulccAGG_ES5$Class_conv][c(3,4,5,9,6,8,1,7,2,10)])+
  scale_x_continuous(breaks=c(1996,2001,2006,2010,2016))+
  theme_classic()+
  geom_hline(yintercept=0,lty=3)+
  theme(axis.text.x = element_text(angle = 90),legend.position='bottom',
        strip.placement = "outside",panel.border = element_rect(colour = "black", fill=NA, size=1),
        axis.line.x = element_blank(),axis.line.y = element_blank(),
        strip.background = element_blank(),axis.text = element_text(size=10),strip.text= element_text(size=10))


g7b <- g7+
  facet_wrap(.~ES2+mb_region2,scales = 'free_y',ncol=6)+
  scale_y_continuous(labels=function(x) sprintf("%.0f", x))+
  ylab('% Change in 1996 Service Provisioning')+
  theme(strip.text.x = element_blank())

g8 <- ggplot(data=lulccAGG_ES5%>%filter(!(year_o==1996&year %in% c(2006,2010,2016)),ES2!="Area Only"),
             aes(x=year,y=100*perc_totES_converted))+
  facet_grid(ES2~mb_region2,scales = 'free_y') +
  geom_bar(stat='identity',col='black',lwd=0.1,aes(fill=Habitat))+
  geom_line(data=lulccAGG_ES5[lulccAGG_ES5$year_o==1996,],
            aes(x=year,y=100*perc96_delES_nff),col='Black',size=1.2,lty=2)+
  scale_fill_manual(values=NOAACCAP$cols2[NOAACCAP$Numeric %in% lulccAGG_ES5$Class_conv][c(3,4,5,9,6,8,1,7,2,10)])+
  #scale_alpha_manual(values=NOAACCAP$alpha[NOAACCAP$Numeric %in% lulccAGG_ES5$Class_conv][c(3,4,5,9,6,8,1,7,2,10)])+
  scale_x_continuous(breaks=c(1996,2001,2006,2010,2016))+
  theme_classic()+
  geom_hline(yintercept=0,lty=3)+
  theme(axis.text.x = element_text(angle = 90),legend.position='bottom',
        strip.placement = "outside",panel.border = element_rect(colour = "black", fill=NA, size=1),
        axis.line.x = element_blank(),axis.line.y = element_blank(),
        strip.background = element_blank(),axis.text = element_text(size=10),strip.text= element_text(size=10))

g8b <- g8+
  facet_wrap(.~ES2+mb_region2,scales = 'free_y',ncol=6)+
  scale_y_continuous(labels=function(x) sprintf("%.0f", x))+
  ylab('% Change in 1996 Service Provisioning')+
  theme(strip.text.x = element_blank())

read_pptx("\\\\ad.uni-hamburg.de\\redir\\redir0010\\bba4705\\Documents\\Template_landscape.pptx") %>%
  ph_with(dml(ggobj=g7),location=ph_location(left=0.5, top=0.5,width=10.5,height=7.5)) %>%
 # add_slide() %>%
  ph_with(dml(ggobj=g7b),location=ph_location(left=0.5, top=0.5,width=10.5,height=7.5)) %>%
  print(target = "./Tables and Figures/ESG.pptx")

plotdata <- lulccAGG_ES5%>%filter(!(year_o==1996&year %in% c(2006,2010,2016)))%>%filter(ES2=="All")%>%filter(Class_conv %in% c(18,18.1,18.2,18.3,19,23))
plotdata <- bind_rows(plotdata %>%
                      mutate(ChangeType='Area',
                               perc_change=perc_area) %>%
                      select(mb_region2,Habitat,year,ChangeType,perc_change),
                      plotdata %>%
                      mutate(ChangeType='Connectivity',
                            perc_change=perc_conn) %>%
                      dplyr::select(mb_region2,Habitat,year,ChangeType,perc_change))



############################################################################
######   individual patch level for each embayment #########################
############################################################################
###  this script does the above for each individual patch, without the complications of 
##   aggregating to different levels
##  it then maps the results for each embayment

library(raster)
library(dplyr)
library(sf)
library(fasterize)
library(officer)
library(rvg)
library(ggplot2)
library(ggmap)
library(cowplot)
library(scales)
library(flextable)

##  to be able to plot agains the Google Satellite images, must register a free API key
register_google("AIzaSyDLTggzhqmtnP0PneUnY1ISaQCNaSY-H0E")

##  A table the ties the numeric Class values to the descriptive habitat definitions
Master_agg <- read.csv('./Data/Output/Master_agg.csv')
Master_agg <- bind_rows(Master_agg,
                        Master_agg %>%
                          filter(Numeric==23&Region=="All") %>%
                          mutate(Numeric=22,
                                 source='Lit'),
                        Master_agg %>%
                          filter(Numeric==16&ES2=="Climate change impacts mitigation") %>%
                          mutate(Numeric=13,
                                 source='Lit'),
                        Master_agg %>%
                          filter(!Numeric %in% c(18,18.1,18.2,18.3,18.4,19,23)) %>%
                          mutate(Numeric=28) %>%
                          group_by(ES2,Numeric, source) %>%
                          summarise(Value=NA)) %>%#mean(Value, na.rm=T))) 
  group_by(Region, ES2) %>%
  arrange(-Value) %>%
  mutate(rank=1:n())
ecotypes <- read_sf("./Data/Input/embayment_units_ecotypes/massbays_estuarine_embays_2021_MODIFIED.shp")

#####   read in the connectivity data
Connections <- read.csv("./Data/Output/Connections_b.csv")
Connections <- merge(Connections[Connections$Variable=='Index',] ,
                     st_drop_geometry(ecotypes[,c("NAME","ecotype2",'mb_region2')]),
                     by.x="scale",by.y="NAME",all.x=T)
Connections$Habitat[Connections$Class %in% c(18,18.3)] <- "Unclassified Salt Marsh"
Connections$Habitat[Connections$Class==18.1] <- "High Elevation Salt Marsh"
Connections$Habitat[Connections$Class==18.2] <- "Low Elevation Salt Marsh"
Connections$Habitat[Connections$Class==19] <- "Tidal Flats"
Connections$Habitat[Connections$Class==23] <- "Seagrass"
Connections$Habitat[Connections$Class==24] <- "All Focal-Intra"
Connections$mb_region2[Connections$scale=='All'] <- "NEP"
Connections$ecotype[Connections$scale=='All'] <- "All"

allpatches <- read_sf("./Data/Output/allpatches_seagrass_saltmarsh_modified_IDs.shp")
###  the original CCAP rasters
Rasters <- list.files("./Data/Input/NOAA CCAP",pattern=".img|.tif",recursive=T,full.names = T)
Rasters <- Rasters[grep("conus",Rasters)]
###  colors and descriptors for the rasters
NOAACCAP <- read.csv("./Data/Input/CCAP_colors.csv")
NOAACCAP$cols[NOAACCAP$cols2!=""] <- NOAACCAP$cols2[NOAACCAP$cols2!=""]
###  create separate datasets for the first and final years
allpatches_y1 <- allpatches %>% st_drop_geometry()%>% filter(year==1996)
allpatches_y2 <- allpatches %>% st_drop_geometry()%>% filter(year==2016)
##  merge with the ES scores and the connectivity
allpatches_y1 <- bind_rows(merge(allpatches_y1[allpatches_y1$Class %in% c(18,18.1,18.2,18.3,19,23),], 
                                    Master_agg[Master_agg$source=='RC',c('ES2','Region','Value','Numeric')] %>% mutate(Numeric=ifelse(Numeric==18.4,18.3,Numeric)),
                                    by.x=c("Class",'mb_region2'),by.y=c("Numeric",'Region'),all.x=T),
                              merge(allpatches_y1[allpatches_y1$Class %in% Master_agg$Numeric[Master_agg$source=='Lit'],], 
                                    Master_agg[Master_agg$source=='Lit',c('ES2','Value','Numeric')],
                                    by.x=c("Class"),by.y=c("Numeric"),all.x=T))
colnames(allpatches_y1)[colnames(allpatches_y1)=="Value"] <- 'Score'
allpatches_y2 <- bind_rows(merge(allpatches_y2[allpatches_y2$Class %in% c(18,18.1,18.2,18.3,19,23),], 
                                 Master_agg[Master_agg$source=='RC',c('ES2','Region','Value','Numeric')] %>% mutate(Numeric=ifelse(Numeric==18.4,18.3,Numeric)),
                                 by.x=c("Class",'mb_region2'),by.y=c("Numeric",'Region'),all.x=T),
                           merge(allpatches_y2[allpatches_y2$Class %in% Master_agg$Numeric[Master_agg$source=='Lit'],], 
                                 Master_agg[Master_agg$source=='Lit',c('ES2','Value','Numeric')],
                                 by.x=c("Class"),by.y=c("Numeric"),all.x=T))
colnames(allpatches_y2)[colnames(allpatches_y2)=="Value"] <- 'Score'
allpatches_y1 <- merge(allpatches_y1,
                          Connections[Connections$met=='PC'&Connections$patchID=='all'&Connections$scale!='All',c('Class','scale','year','Value')],
                          by.x=c('Class','NAME','year'),by.y=c('Class','scale','year'), all.x=T)
colnames(allpatches_y1)[colnames(allpatches_y1)=="Value"] <- 'Conn'
allpatches_y2 <- merge(allpatches_y2,
                       Connections[Connections$met=='PC'&Connections$patchID=='all'&Connections$scale!='All',c('Class','scale','year','Value')],
                       by.x=c('Class','NAME','year'),by.y=c('Class','scale','year'), all.x=T)
colnames(allpatches_y2)[colnames(allpatches_y2)=="Value"] <- 'Conn'

###  calculate the capacity 
allpatches_y1$ES <- (allpatches_y1$areaLocal*allpatches_y1$Score)+(allpatches_y1$areaLocal*allpatches_y1$Score*allpatches_y1$Conn)
allpatches_y2$ES <- (allpatches_y2$areaLocal*allpatches_y2$Score)+(allpatches_y2$areaLocal*allpatches_y2$Score*allpatches_y2$Conn)
##  now per pixel to map to the raster
allpatches_y1$ES_perPixel <- allpatches_y1$ES/(allpatches_y1$areaLocal/900)
allpatches_y2$ES_perPixel <- allpatches_y2$ES/(allpatches_y2$areaLocal/900)

###  aggregate to ALL services
allpatches_y1 <- bind_rows(allpatches_y1,
                           allpatches_y1 %>%
                             group_by(mb_region2,NAME,Class,patchID) %>%
                             summarise(ES_perPixel=sum(ES_perPixel,na.rm=T)) %>%
                             mutate(ES2="All",year=1996))
allpatches_y2 <- bind_rows(allpatches_y2,
                           allpatches_y2 %>%
                             group_by(mb_region2,NAME,Class,patchID) %>%
                             summarise(ES_perPixel=sum(ES_perPixel,na.rm=T)) %>%
                             mutate(ES2="All",year=2016))
###  merge with the original data
allpatches_y1 <- merge(allpatches[allpatches$patchID %in% allpatches_y1$patchID,
                                       c('patchID','NAME','geometry')],allpatches_y1,by=c('patchID','NAME'),all.y=T)
allpatches_y2 <- merge(allpatches[allpatches$patchID %in% allpatches_y2$patchID,
                                  c('patchID','NAME','geometry')],allpatches_y2,by=c('patchID','NAME'),all.y=T)

##  if not saved, run this, takes ~1 hour
##  calculates the intersections between the patches of 1996 and those of 2016
#int <- st_intersects(allpatches_y1,allpatches_y2%>% filter(Class %in% c(18,18.1,18.2,19,23)))
#int_y1 <- unique(c(which(allpatches_y1$Class %in% c(18,18.1,18.2,19,23)),which(lengths(int) > 0)))
#save(int_y1, file = "./Data/Output/int_y1.rda")
#int <- st_intersects(allpatches_y2,allpatches_y1%>%filter(Class %in% c(18,18.1,18.2,19,23)))
#int_y2 <- unique(c(which(allpatches_y2$Class %in% c(18,18.1,18.2,19,23)),which(lengths(int) > 0)))
#save(int_y2, file = "./Data/Output/int_y2.rda")

## read in the saved intersection indices
load("./Data/Output/int_y1.rda")
load("./Data/Output/int_y2.rda")

###  subset by only the intersected patches from above
allpatches_y1 <- allpatches_y1[int_y1,]
allpatches_y2 <- allpatches_y2[int_y2,]
remove(int, int_y1,int_y2)

lulccAGG_NAME <- read.csv(".\\Data\\Output\\lulccAGG_NAME.csv")
#################################
###  then aggregate the data
##############################
###  read in the aggregated dataset again
lulccAGG_ES3_rand <- read.csv("./Data/Output/lulccAGG_ES_rand.csv")
NOAACCAP <- read.csv("./Data/Input/CCAP_colors.csv")

## aggregate to "All" regions
lulccAGG_ES3_rand <- bind_rows(lulccAGG_ES3_rand,
                               lulccAGG_ES3_rand %>% 
                                 ## first the Class_o
                                 group_by(mb_region2,NAME,ES2,year_o,year) %>%
                                 mutate(dup.o=duplicated(Class_o)) %>%
                                 group_by(ES2,year_o,year,Class_o) %>%
                                 mutate(area_o_o=sum(area_o_o[!dup.o]),
                                        ES_o_y1_mean=sum(ES_o_y1_mean[!dup.o],na.rm=T),
                                        ES_o_y1_max=sum(ES_o_y1_max[!dup.o],na.rm=T),
                                        ES_o_y1_min=sum(ES_o_y1_min[!dup.o],na.rm=T))%>%
                                 # then the Class_conv
                                 group_by(mb_region2,NAME,ES2,year_o,year) %>%
                                 mutate(dup.conv=duplicated(Class_conv)) %>%
                                 group_by(ES2,year_o,year,Class_conv) %>%
                                 mutate(area_o_conv=sum(area_o_conv[!dup.conv],na.rm=T),
                                        ES_conv_y1_mean=sum(ES_conv_y1_mean[!dup.conv],na.rm=T),
                                        ES_conv_y1_max=sum(ES_conv_y1_max[!dup.conv],na.rm=T),
                                        ES_conv_y1_min=sum(ES_conv_y1_min[!dup.conv],na.rm=T)) %>%
                                 group_by(ES2,year_o,year,Class_o,Class_conv) %>%
                                 summarise(area_o_o=first(area_o_o),
                                           area_o_conv=first(area_o_conv),
                                           ES_o_y1_mean=first(ES_o_y1_mean),
                                           ES_o_y1_max=first(ES_o_y1_max),
                                           ES_o_y1_min=first(ES_o_y1_min),
                                           ES_conv_y1_mean=first(ES_conv_y1_mean),
                                           ES_conv_y1_max=first(ES_conv_y1_max),
                                           ES_conv_y1_min=first(ES_conv_y1_min),
                                           area_converted=sum(area_converted,na.rm=T),
                                           delES_o_area.max = sum(delES_o_area.max,na.rm=T),
                                           delES_o_area.mean = sum(delES_o_area.mean,na.rm=T),
                                           delES_o_area.min = sum(delES_o_area.min,na.rm=T),
                                           delES_conv_area.max = sum(delES_conv_area.max,na.rm=T),
                                           delES_conv_area.mean = sum(delES_conv_area.mean,na.rm=T),
                                           delES_conv_area.min = sum(delES_conv_area.min,na.rm=T),
                                           delES_o_conn.max = sum(delES_o_conn.max,na.rm=T),
                                           delES_o_conn.mean = sum(delES_o_conn.mean,na.rm=T),
                                           delES_o_conn.min = sum(delES_o_conn.min,na.rm=T),
                                           delES_conv_conn.max= sum(delES_conv_conn.max,na.rm=T),
                                           delES_conv_conn.mean= sum(delES_conv_conn.mean,na.rm=T),
                                           delES_conv_conn.min= sum(delES_conv_conn.min,na.rm=T)) %>%
                                 mutate(mb_region2="All",NAME="All",
                                        delES_perc.max = 100*(delES_o_area.max+delES_conv_area.max+delES_o_conn.max+delES_conv_conn.max)/(ES_o_y1_max+ES_conv_y1_max),
                                        delES_perc.mean= 100*(delES_o_area.mean+delES_conv_area.mean+delES_o_conn.mean+delES_conv_conn.mean)/(ES_o_y1_mean+ES_conv_y1_mean),
                                        delES_perc.min = 100*(delES_o_area.min+delES_conv_area.min+delES_o_conn.min+delES_conv_conn.min)/(ES_o_y1_min+ES_conv_y1_min),
                                        area_converted_perc = 100*area_converted/(area_o_o+area_o_conv)))
lulccAGG_ES4_rand <- bind_rows(lulccAGG_ES3_rand,
                               lulccAGG_ES3_rand[lulccAGG_ES3_rand$ES2!='Area Only',] %>%
                                 mutate(ES2='All') %>%
                                 group_by(mb_region2,NAME,ES2,year,year_o,Class_o,Class_conv) %>%
                                 summarise(
                                   ES_o_y1_max=sum(ES_o_y1_max,na.rm=T),
                                   ES_o_y1_mean=sum(ES_o_y1_mean,na.rm=T),
                                   ES_o_y1_min=sum(ES_o_y1_min,na.rm=T),
                                   ES_conv_y1_max=sum(ES_conv_y1_max,na.rm=T),
                                   ES_conv_y1_mean=sum(ES_conv_y1_mean,na.rm=T),
                                   ES_conv_y1_min=sum(ES_conv_y1_min,na.rm=T),
                                   delES_o_area.max=sum(delES_o_area.max,na.rm=T),
                                   delES_o_area.mean=sum(delES_o_area.mean,na.rm=T),
                                   delES_o_area.min=sum(delES_o_area.min,na.rm=T),
                                   delES_o_conn.max=sum(delES_o_conn.max,na.rm = T),
                                   delES_o_conn.mean=sum(delES_o_conn.mean,na.rm = T),
                                   delES_o_conn.min=sum(delES_o_conn.min,na.rm = T),
                                   delES_conv_area.max=sum(delES_conv_area.max,na.rm=T),
                                   delES_conv_area.mean=sum(delES_conv_area.mean,na.rm=T),
                                   delES_conv_area.min=sum(delES_conv_area.min,na.rm=T),
                                   delES_conv_conn.max=sum(delES_conv_conn.max,na.rm = T),
                                   delES_conv_conn.mean=sum(delES_conv_conn.mean,na.rm = T),
                                   delES_conv_conn.min=sum(delES_conv_conn.min,na.rm = T))) %>%
  ungroup() %>%
  group_by(mb_region2,NAME,ES2,year,year_o,Class_o) %>%
  mutate(ES_o_y2_mean = ES_o_y1_mean+sum(delES_o_area.mean+delES_o_conn.mean),
         ES_o_y2_max = ES_o_y1_max+sum(delES_o_area.max+delES_o_conn.max),
         ES_o_y2_min = ES_o_y1_min+sum(delES_o_area.min+delES_o_conn.min)) %>%
  group_by(mb_region2,NAME,ES2,year,year_o,Class_conv) %>%
  mutate(ES_conv_y2_mean = ES_conv_y1_mean+sum(delES_conv_area.mean[Class_o!=26]+delES_conv_conn.mean[Class_o!=26]),
         ES_conv_y2_max = ES_conv_y1_max+sum(delES_conv_area.max[Class_o!=26]+delES_conv_conn.max[Class_o!=26]),
         ES_conv_y2_min = ES_conv_y1_min+sum(delES_conv_area.min[Class_o!=26]+delES_conv_conn.min[Class_o!=26]))

## Ecosystem change over time  
ES_AGG <- lulccAGG_ES4_rand
ES_AGG_totals <- ES_AGG %>% ungroup() %>%
  filter(Class_o %in% c(18,18.1,18.2,18.3,19,23)|Class_conv %in% c(18,18.1,18.2,18.3,19,23)) %>%
  filter(!Class_o %in% c(26,28)) %>%
  group_by(NAME,year_o,year,ES2) %>% 
  mutate(focal.o=Class_o %in% c(18,18.1,18.2,18.3,19,23),
         focal.conv=Class_conv %in% c(18,18.1,18.2,18.3,19,23),
         delES_f.max = sum(delES_conv_area.max[focal.conv],delES_conv_conn.max[focal.conv],na.rm = T),
         delES_f.mean = sum(delES_conv_area.mean[focal.conv],delES_conv_conn.mean[focal.conv],na.rm = T),
         delES_f.min = sum(delES_conv_area.min[focal.conv],delES_conv_conn.min[focal.conv],na.rm = T),
         delES_nf_f.max = sum(delES_conv_area.max[!focal.o&focal.conv],delES_conv_conn.max[!focal.o&focal.conv],na.rm = T),
         delES_nf_f.mean = sum(delES_conv_area.mean[!focal.o&focal.conv],delES_conv_conn.mean[!focal.o&focal.conv],na.rm = T),
         delES_nf_f.min = sum(delES_conv_area.min[!focal.o&focal.conv],delES_conv_conn.min[!focal.o&focal.conv],na.rm = T),
         delES_f_f.max = sum(delES_o_area.max[focal.o&focal.conv],delES_o_conn.max[focal.o&focal.conv],na.rm = T),
         delES_f_f.mean = sum(delES_o_area.mean[focal.o&focal.conv],delES_o_conn.mean[focal.o&focal.conv],na.rm = T),
         delES_f_f.min = sum(delES_o_area.min[focal.o&focal.conv],delES_o_conn.min[focal.o&focal.conv],na.rm = T)) %>%
  ##  get the converted class sums for y2, this is for a check in the next step
  # group_by(mb_region2,NAME,year_o,year,ES2) %>%
  mutate(dup_conv =duplicated(cbind(Class_conv,NAME)),
         ES_conv_y2_f_max= sum(ES_conv_y2_max[!dup_conv&focal.conv],na.rm = T),
         ES_conv_y2_f_mean= sum(ES_conv_y2_mean[!dup_conv&focal.conv],na.rm = T),
         ES_conv_y2_f_min= sum(ES_conv_y2_min[!dup_conv&focal.conv],na.rm = T)) %>% ungroup() %>%
  distinct(NAME,Class_o,year,year_o,ES2,.keep_all = T) %>%
  group_by(NAME,year_o,year,ES2) %>%
  mutate(dup_conv =duplicated(NAME)) %>%
  arrange(year) %>%
  summarise(delES_f.max = first(delES_f.max),delES_f.mean = first(delES_f.mean),delES_f.min = first(delES_f.min),
            delES_nf_f.max = first(delES_nf_f.max),delES_nf_f.mean = first(delES_nf_f.mean),delES_nf_f.min = first(delES_nf_f.min),
            delES_f_f.max = first(delES_f_f.max),delES_f_f.mean = first(delES_f_f.mean),delES_f_f.min = first(delES_f_f.min),
            ES_o_y1_f_max=sum(ES_o_y1_max[focal.o],na.rm = T),ES_o_y1_f_mean=sum(ES_o_y1_mean[focal.o],na.rm = T),ES_o_y1_f_min=sum(ES_o_y1_min[focal.o],na.rm = T),
            ES_o_y2_f_max = sum(ES_conv_y2_f_max[!dup_conv&focal.conv],na.rm = T), ES_o_y2_f_mean = sum(ES_conv_y2_f_mean[!dup_conv&focal.conv],na.rm = T), ES_o_y2_f_min = sum(ES_conv_y2_f_min[!dup_conv&focal.conv],na.rm = T),
            ES_o_y2_f_check = ES_o_y1_f_mean+delES_f.mean,
            check = ES_o_y2_f_mean/ES_o_y2_f_check) %>%
  group_by(NAME,ES2) %>%# group_by(mb_region2,Class_o,year_o,year,ES2) %>%
  arrange(year) %>%
  mutate(is.dup=duplicated(year),
         is.dup.o=duplicated(year_o),
         delES_f_96.max = ifelse(is.dup,lag(delES_f.max),delES_f.max), delES_f_96.mean = ifelse(is.dup,lag(delES_f.mean),delES_f.mean), delES_f_96.min = ifelse(is.dup,lag(delES_f.min),delES_f.min),
         delES_ff_96.max = ifelse(is.dup,lag(delES_f_f.max),delES_f_f.max),delES_ff_96.mean = ifelse(is.dup,lag(delES_f_f.mean),delES_f_f.mean),delES_ff_96.min = ifelse(is.dup,lag(delES_f_f.min),delES_f_f.min),
         delES_nff_96.max = ifelse(is.dup,lag(delES_nf_f.max),delES_nf_f.max),delES_nff_96.mean = ifelse(is.dup,lag(delES_nf_f.mean),delES_nf_f.mean),delES_nff_96.min = ifelse(is.dup,lag(delES_nf_f.min),delES_nf_f.min)) %>%
  dplyr::select(-is.dup,-is.dup.o)

ES_AGG_changes <- ES_AGG %>% ungroup() %>%
  filter(Class_o %in% c(18,18.1,18.2,18.3,19,23)|Class_conv %in% c(18,18.1,18.2,18.3,19,23)) %>%
  filter(!Class_o %in% c(26,28)) %>%
  group_by(NAME,Class_conv,year_o,year,ES2) %>% ## for delES variables, the Class grouping shouldnt matter, each value is unique
  summarise(delES_o_area.max=sum(delES_o_area.max),delES_o_area.mean=sum(delES_o_area.mean),delES_o_area.min=sum(delES_o_area.min),
            delES_o_conn.max =sum(delES_o_conn.max),delES_o_conn.mean =sum(delES_o_conn.mean),delES_o_conn.min =sum(delES_o_conn.min),
            delES_conv_area.max=sum(delES_conv_area.max),delES_conv_area.mean=sum(delES_conv_area.mean),delES_conv_area.min=sum(delES_conv_area.min),
            delES_conv_conn.max =sum(delES_conv_conn.max),delES_conv_conn.mean =sum(delES_conv_conn.mean),delES_conv_conn.min =sum(delES_conv_conn.min),
            delES_o.max =delES_o_area.max+delES_o_conn.max,delES_o.mean =delES_o_area.mean+delES_o_conn.mean,delES_o.min =delES_o_area.min+delES_o_conn.min,
            delES_conv.max =delES_conv_area.max+delES_conv_conn.max,delES_conv.mean =delES_conv_area.mean+delES_conv_conn.mean,delES_conv.min =delES_conv_area.min+delES_conv_conn.min,
            delES_area.max = delES_o_area.max+delES_conv_area.max, delES_area.mean = delES_o_area.mean+delES_conv_area.mean, delES_area.min = delES_o_area.min+delES_conv_area.min,
            delES_conn.max = delES_o_conn.max+delES_conv_conn.max,delES_conn.mean = delES_o_conn.mean+delES_conv_conn.mean,delES_conn.min = delES_o_conn.min+delES_conv_conn.min,
            delES.max =delES_o_area.max+delES_o_conn.max+delES_conv_area.max+delES_conv_conn.max,
            delES.mean =delES_o_area.mean+delES_o_conn.mean+delES_conv_area.mean+delES_conv_conn.mean,
            delES.min =delES_o_area.min+delES_o_conn.min+delES_conv_area.min+delES_conv_conn.min) %>%
  group_by(NAME,ES2) %>%
  arrange(year)
###  merge the totals with the changes
ES_AGG2 <- merge(ES_AGG_totals[,c('NAME','ES2','year_o','year','ES_o_y1_f_max','ES_o_y1_f_mean','ES_o_y1_f_min','delES_f.max','delES_f.mean','delES_f.min',
                                  'delES_f_96.max','delES_f_96.mean','delES_f_96.min','delES_f_f.max','delES_f_f.mean','delES_f_f.min',
                                  'delES_ff_96.max','delES_ff_96.mean','delES_ff_96.min','delES_nf_f.max','delES_nf_f.mean','delES_nf_f.min',
                                  'delES_nff_96.max','delES_nff_96.mean','delES_nff_96.min')],
                 ES_AGG_changes[,c('NAME','ES2','year_o','year','Class_conv','delES_conv.max','delES_conv.mean','delES_conv.min','delES_conv_area.max',
                                   'delES_conv_area.mean','delES_conv_area.min','delES_conv_conn.max','delES_conv_conn.mean','delES_conv_conn.min')],
                 by=c('NAME','ES2','year_o','year')) %>%
  ##get the 1996 ES value
  group_by(NAME,ES2) %>%
  mutate(ES_o_96_f_max = first(ES_o_y1_f_max),ES_o_96_f_mean = first(ES_o_y1_f_mean),ES_o_96_f_min = first(ES_o_y1_f_min),
         perc_ES_96_f.max = ((ES_o_y1_f_max+delES_f.max)-ES_o_96_f_max)/ES_o_96_f_max,
         perc_ES_96_f.mean = ((ES_o_y1_f_mean+delES_f.mean)-ES_o_96_f_mean)/ES_o_96_f_mean,
         perc_ES_96_f.min = ((ES_o_y1_f_min+delES_f.min)-ES_o_96_f_min)/ES_o_96_f_min)%>%
  ## now get the change from 96
  ungroup() %>%
  mutate(perc96_delES_f.max = delES_ff_96.max/ES_o_96_f_max,perc96_delES_f.mean = delES_ff_96.mean/ES_o_96_f_mean,perc96_delES_f.min = delES_ff_96.min/ES_o_96_f_min,
         perc96_delES_nff.max = delES_nff_96.max/ES_o_96_f_max,perc96_delES_nff.mean = delES_nff_96.mean/ES_o_96_f_mean,perc96_delES_nff.min = delES_nff_96.min/ES_o_96_f_min,
         perc_totES_converted.max = delES_conv.max/ES_o_96_f_max,perc_totES_converted.mean = delES_conv.mean/ES_o_96_f_mean,perc_totES_converted.min = delES_conv.min/ES_o_96_f_min,
         perc_area.max = delES_conv_area.max/delES_conv.max,perc_area.mean = delES_conv_area.mean/delES_conv.mean,perc_area.min = delES_conv_area.min/delES_conv.min,
         perc_conn.max = delES_conv_conn.max/delES_conv.max,perc_conn.mean = delES_conv_conn.mean/delES_conv.mean,perc_conn.min = delES_conv_conn.min/delES_conv.min,
         percOfFocalConversion.max = ifelse(Class_conv %in% c(18,18.1,18.2,19,23),delES_conv.max/delES_f.max,NA),
         percOfFocalConversion.mean = ifelse(Class_conv %in% c(18,18.1,18.2,19,23),delES_conv.mean/delES_f.mean,NA),
         percOfFocalConversion.min = ifelse(Class_conv %in% c(18,18.1,18.2,19,23),delES_conv.min/delES_f.min,NA))
ES_AGG2 <- merge(ES_AGG2,NOAACCAP[,c('Numeric','Habitat')],by.x='Class_conv',by.y='Numeric',all.x=T)
ES_AGG2$Habitat <- factor(ES_AGG2$Habitat,levels=c('High Elevation Salt Marsh','Low Elevation Salt Marsh',
                                                   'Unclassified Salt Marsh','Seagrass','Tidal Flats',
                                                   'Open Water','Development','Beach','Estuarine Scrub/Shrub Wetland','All Others'))
ES_AGG2$year <- as.numeric(ES_AGG2$year)

ES_AGG2 <- ES_AGG2 %>%
  ungroup() %>%
  arrange(desc(Habitat),.by_group = TRUE) %>%
  group_by(NAME,ES2,year,year_o) %>%
  mutate(sign=ifelse(perc_totES_converted.mean<0,-1,1),
         perc_totES_converted.min_cum.neg=ifelse(sign==-1,cumsum(perc_totES_converted.min[sign==-1]),NA),
         perc_totES_converted.min_cum.pos=ifelse(sign==1,cumsum(perc_totES_converted.min[sign==1]),NA),
         perc_totES_converted.max_cum.neg=ifelse(sign==-1,cumsum(perc_totES_converted.max[sign==-1]),NA),
         perc_totES_converted.max_cum.pos=ifelse(sign==1,cumsum(perc_totES_converted.max[sign==1]),NA))

###  percent of focal services provided by each habitat
ES_AGG_LC <- ES_AGG[!is.na(ES_AGG$ES2),] %>% ungroup() %>%
  filter(year %in% c(1996,2016),Class_o %in% c(18,18.1,18.2,18.3,19,23)) %>%
  distinct(NAME,Class_o,ES2,year,.keep_all = T) %>%
  group_by(NAME,ES2,year)%>%
  mutate(ES_T_mean = sum(ES_o_y2_mean),
         ES_T_max = sum(ES_o_y2_max),
         ES_T_min = sum(ES_o_y2_min),
         areaT = sum(area_o_o)) %>%
  group_by(NAME,ES2,Class_o,year) %>%
  summarise(ES_o_y2_mean= sum(ES_o_y2_mean),
            ES_o_y2_max= sum(ES_o_y2_max),
            ES_o_y2_min= sum(ES_o_y2_min),
            ES_T_mean=first(ES_T_mean),
            ES_T_max=first(ES_T_max),
            ES_T_min=first(ES_T_min),
            area_o_o= sum(area_o_o),
            areaT=first(areaT)) %>%
  mutate(ES_perc_max=ES_o_y2_max/ES_T_max,
         ES_perc_mean=ES_o_y2_mean/ES_T_mean,
         ES_perc_min=ES_o_y2_min/ES_T_min,
         area_perc=area_o_o/areaT) %>%
  group_by(NAME,ES2,year) %>%
  arrange(ES_perc_mean) %>%
  mutate(ESrank = n():1)

ES_AGG_LC <- merge(ES_AGG_LC,NOAACCAP[,c('Numeric','Habitat')],by.x='Class_o',by.y='Numeric',all.x=T)
ES_AGG_LC$Habitat[ES_AGG_LC$Habitat=='High Elevation Salt Marsh'] <- 'High Elevation\nSalt Marsh'
ES_AGG_LC$Habitat[ES_AGG_LC$Habitat=='Low Elevation Salt Marsh'] <- 'Low Elevation\nSalt Marsh'
ES_AGG_LC$Habitat[ES_AGG_LC$Habitat=='Unclassified Salt Marsh'] <- 'Unclassified\nSalt Marsh'
ES_AGG_LC$Habitat <- factor(ES_AGG_LC$Habitat,levels=c('High Elevation\nSalt Marsh','Low Elevation\nSalt Marsh',
                                                       'Unclassified\nSalt Marsh','Seagrass','Tidal Flats'))
ES_AGG_LC <- ES_AGG_LC %>%
  ungroup() %>%
  arrange(desc(Habitat),.by_group = TRUE) %>%
  group_by(NAME,ES2) %>%
  mutate(ES_perc_mean_cum=cumsum(ES_perc_mean),
         ES_perc_min_cum=cumsum(ES_perc_min),
         ES_perc_max_cum=cumsum(ES_perc_max))

LC_AGG <- lulccAGG_NAME[(lulccAGG_NAME$year_o==1996|lulccAGG_NAME$year_o==2016),] %>%
  distinct(NAME,year_o,Class_o,.keep_all = T) %>%
  group_by(NAME,year_o) %>%
  mutate(TotalF=sum(area_o_o[Class_o %in% c(18,18.1,18.2,19,23)]),
         PercentF = round(100*area_o_o/TotalF,1))
LC_AGG <- merge(LC_AGG ,NOAACCAP[,c('Numeric','Habitat')],by.x='Class_o',by.y='Numeric',all.x=T)
LC_AGG$Habitat <- factor(LC_AGG$Habitat,levels=c('High Elevation Salt Marsh','Low Elevation Salt Marsh',
                                                 'Unclassified Salt Marsh','Seagrass','Tidal Flats'))
remove(lulccAGG_ES3,lulccAGG_ES3_rand,lulccAGG_ES4_rand)
gc()
###   prepare the rasters
r1 <- raster(Rasters[1])
#r2 <- raster(Rasters[1])
ecotypes <- st_transform(ecotypes,3857)
ggmap_bbox <- function(map) {
  if (!inherits(map, "ggmap")) stop("map must be a ggmap object")
  # Extract the bounding box (in lat/lon) from the ggmap to a numeric vector, 
  # and set the names to what sf::st_bbox expects:
  map_bbox <- setNames(unlist(attr(map, "bb")), 
                       c("ymin", "xmin", "ymax", "xmax"))
  
  # Coonvert the bbox to an sf polygon, transform it to 3857, 
  # and convert back to a bbox (convoluted, but it works)
  bbox_3857 <- st_bbox(st_transform(st_as_sfc(st_bbox(map_bbox, crs = 4326)), 3857))
  
  # Overwrite the bbox of the ggmap object with the transformed coordinates 
  attr(map, "bb")$ll.lat <- bbox_3857["ymin"]
  attr(map, "bb")$ll.lon <- bbox_3857["xmin"]
  attr(map, "bb")$ur.lat <- bbox_3857["ymax"]
  attr(map, "bb")$ur.lon <- bbox_3857["xmax"]
  map
}
CapStr <- function(y) {
  y <- tolower(y)
  c <- strsplit(y, " ")[[1]]
  paste(toupper(substring(c, 1,1)), substring(c, 2),
        sep="", collapse=" ")
}
tmp<-tmpDir()
scale <- c(0,0)
#for (R in unique(allpatches_y1$mb_region2)){
  pb <- txtProgressBar(min = 1, max = length(unique(allpatches_y1$NAME)),style = 3)
  #read_pptx("\\\\ad.uni-hamburg.de\\redir\\redir0010\\bba4705\\Documents\\Template.pptx") %>%
  #  print(target = paste0("./Tables and Figures/Maps_",R,".pptx"))
#  x=0;y=0
E="All"
names=unique(allpatches_y1$NAME)
names <- names[order(names)]
P = read_pptx("\\\\ad.uni-hamburg.de\\redir\\redir0010\\bba4705\\Documents\\Template.pptx")
  for (N in names){
      patches <- allpatches[allpatches$NAME==N,]
      if (max(patches$areaLocal[patches$Class %in% c(18,18.1,18.2,18.3,19,23)])>90){
        ####################################
        ###  First the maps
        ################################
        cent = patches %>% summarise()
        if (as.numeric(st_area(ecotypes[ecotypes$NAME==N,]))<15000000){
          bbox = st_as_sfc(st_bbox(st_buffer(cent,1000)))
        }else{
          bbox = st_as_sfc(st_bbox(cent))
        }
        cent = st_centroid(cent)
        cent = st_coordinates(st_transform(cent,4326))
        bbox = st_coordinates(st_transform(bbox,3857))
        r <- crop(r1,patches)
        r_y1C <- fasterize(allpatches_y1[allpatches_y1$ES2==E&allpatches_y1$NAME==N,],r,field='Class')
        r_y2C <- fasterize(allpatches_y2[allpatches_y2$ES2==E&allpatches_y2$NAME==N,],r,field='Class')
        r_y1 <- fasterize(allpatches_y1[allpatches_y1$ES2==E&allpatches_y1$NAME==N,],r,field='ES_perPixel')
        r_y2 <- fasterize(allpatches_y2[allpatches_y2$ES2==E&allpatches_y2$NAME==N,],r,field='ES_perPixel',background=0)
        r_C <- overlay(r_y1C,r_y2C,fun=function(a,b){
          ifelse(a %in% c(18,18.1,18.2,19,23)|b %in% c(18,18.1,18.2,19,23),1,NA)
        })
        r_diff <- r_y2-r_y1
        r_diff <- r_diff*r_C
        r_perc <- 100*r_diff/r_y1
        r_diff <- projectRaster(r_diff,crs=CRS("+proj=merc +a=6378137 +b=6378137 +lat_ts=0 +lon_0=0 +x_0=0 +y_0=0 +k=1 +units=m +nadgrids=@null +wktext +no_defs +type=crs")) ## EPSG3857
        r_perc <- projectRaster(r_perc,crs=CRS("+proj=merc +a=6378137 +b=6378137 +lat_ts=0 +lon_0=0 +x_0=0 +y_0=0 +k=1 +units=m +nadgrids=@null +wktext +no_defs +type=crs"))
        MIN=min(getValues(r_perc),na.rm=T)
        MAX=max(getValues(r_perc),na.rm=T)
        if (MIN<scale[1]){scale[1]<-MIN}
        if (MAX<scale[2]){scale[2]<-MAX}
        diff_df <- as(r_diff, "SpatialPixelsDataFrame")
        perc_df <- as(r_perc, "SpatialPixelsDataFrame")
        diff_df <- as.data.frame(diff_df)
        perc_df <- as.data.frame(perc_df)
        colnames(diff_df) <- c("value", "x", "y")
        colnames(perc_df) <- c("value", "x", "y")
        perc_df$value[!is.finite(perc_df$value)] <- NA
        perc_df$value <- round(perc_df$value,2)
        m=get_googlemap(center=c(lon=cent[1],lat=cent[2]),zoom=10,maptype = "satellite",color='bw')
        m <- ggmap_bbox(m)
        m_transparent <- matrix(adjustcolor(m,alpha.f = 0.75),nrow = nrow(m))
        attributes(m_transparent) <- attributes(m)
        
        g_loc <- ggplot(ecotypes %>% st_buffer(.5) %>% st_union() %>% st_sf())+
          geom_sf(col="black",fill="transparent")+
          geom_sf(data=st_as_sfc(st_bbox(ecotypes[ecotypes$NAME==N,])),col="red",size=1)+
         # geom_sf(data=ecotypes[ecotypes$NAME==N,],fill="red")+
          theme_void()+
          theme(panel.background = element_rect(fill = 'white', color = 'black'))
            
        
        g96 <-ggmap(m) + 
          coord_sf(crs = st_crs(3857))+
          geom_sf(data=ecotypes[ecotypes$NAME==N,],color="white",fill=NA,inherit.aes=F)+
          scale_x_continuous(limits = c(min(bbox[,1]), max(bbox[,1]))) +
          scale_y_continuous(limits = c(min(bbox[,2]), max(bbox[,2])))+
          geom_sf(data=st_transform(patches[patches$year==1996,],3857),aes(fill=factor(Class)),color=NA, inherit.aes=F)+
          #geom_tile(data=y1_df, aes(x=x, y=y, fill=value), alpha=0.8)+
          scale_fill_manual(values=NOAACCAP$cols[NOAACCAP$Numeric %in% unique(patches$Class[patches$year==1996])])+ 
          ylab("")+xlab("")+
          theme_bw()+
          theme(legend.position = "none",axis.text.x = element_text(size=10),axis.text.y = element_text(size=10),
                axis.title = element_blank(),plot.margin=grid::unit(c(0,0,0,0), "mm"))

        
        g16 <-ggmap(m) + 
          geom_sf(data=ecotypes[ecotypes$NAME==N,],color="white",fill=NA,inherit.aes=F)+
          coord_sf(crs = st_crs(3857))+
          scale_x_continuous(limits = c(min(bbox[,1]), max(bbox[,1]))) +
          scale_y_continuous(limits = c(min(bbox[,2]), max(bbox[,2])))+
          geom_sf(data=st_transform(patches[patches$year==2016,],3857),aes(fill=factor(Class)),color=NA, inherit.aes=F)+
          #geom_tile(data=y1_df, aes(x=x, y=y, fill=value), alpha=0.8)+
          scale_fill_manual(values=NOAACCAP$cols[NOAACCAP$Numeric %in% unique(patches$Class[patches$year==2016])])+ 
          ylab("")+xlab("")+
          theme_bw()+
          theme(legend.position = "none",axis.text.x = element_text(size=10),axis.text.y = element_text(size=10),
                axis.title = element_blank(),plot.margin=grid::unit(c(0,0,0,0), "mm"))
          #theme(legend.position = "none",axis.text.x = element_text(size=14,angle=90),axis.text.y = element_blank(),
          #      plot.background = element_blank(),panel.background = element_blank(),
          #      axis.title = element_blank(),plot.margin=grid::unit(c(0,0,0,0), "mm"))
        #getPalette <- colorRampPalette(RColorBrewer::brewer.pal(11,"PiYG"))
        if (round(MIN)!=0|round(MAX)!=0){
          if (MIN<0 & MAX>0){
            cuts <- unique(c(seq(floor(MIN), 0, length.out = 6),seq(0,ceiling(MAX), length.out = 6)))
            cols = RColorBrewer::brewer.pal(11,"PiYG")
            cols <- c(cols[1:5],"white",cols[7:11])
          }else if(MAX<=0) {
            cuts <- unique(c(seq(floor(MIN),ceiling(MAX), length.out = 11),0))
            cols = c(colorRampPalette(RColorBrewer::brewer.pal(11,"PiYG")[1:5])(11),"white")
          }else{
            cuts <- unique(c(0,seq(floor(MIN),ceiling(MAX), length.out = 11)))
            cols = c("white",colorRampPalette(RColorBrewer::brewer.pal(11,"PiYG")[7:11])(11))
          }
          perc_df$bins <- as.character(cut(perc_df$value,cuts))
          if(sum(perc_df$value==0)>0){
            perc_df$bins[perc_df$value==0] <- "0"
          }else{
            cols<- cols[-which(cols=="white")]
          }
          perc_df <- perc_df[order(perc_df$value),]
          perc_df$bins <- factor(perc_df$bins,levels=unique(perc_df$bins))
          labs <- levels(perc_df$bins)
          labs <- gsub("[(]|[]]","",labs)
          labs <- gsub(",","  -  ",labs)
          if (length(labs)!=length(cols)){
            n0 <- which(labs==0)
            if (n0<6&n0>1){
              cols <- cols[-(n0:5)]
              if (length(labs)!=length(cols)){
                n0 <- which(labs==0)
                cols <- cols[-((n0+1):(n0+length(cols)-length(labs)))]
              }
            } else if (n0==6){
              cols <- cols[-(7:(7+(11-length(cols))))]
            }else if (n0>6){
              cols <- cols[-(10:(n0-1))]
            }else{
              cols <- cols[-(2:(11-length(labs)+1))]
            }
            
          }
        }
        
        gdel <- ggmap(m_transparent) + 
          coord_sf(crs = st_crs(3857))+
          geom_sf(data=ecotypes[ecotypes$NAME==N,],color="white",fill=NA,inherit.aes=F)+
          scale_x_continuous(limits = c(min(bbox[,1]), max(bbox[,1]))) +
          scale_y_continuous(limits = c(min(bbox[,2]), max(bbox[,2])))
          if (length(unique(perc_df$value))>2){
            gdel=gdel+geom_tile(data=perc_df, aes(x=x, y=y, fill=bins))+
            scale_fill_manual(values=cols,drop=FALSE,labels=labs)+
            #scale_fill_gradient2(low="red",mid="black",high="cyan")+
            guides(fill=guide_legend(title="% Change",reverse=T))}
        gdel=gdel+
          #scale_fill_gradient(low="red",high="green",limits=c(min(perc_df$value),max(perc_df$value)),midpoint=0)+
          #scale_fill_gradientn(colors=c('red','green',"darkgreen"),values=c(min(perc_df$value),0,max(perc_df$value)))+ 
          ylab("")+xlab("")+
          theme_bw()+
          theme(legend.position = "right",legend.text=element_text(size=14),
                legend.title = element_text(size=14),
                axis.title = element_blank(),plot.margin=grid::unit(c(0,0,0,0), "mm"))
        
        LC_tab <- merge(data.frame(Habitat=c('High Elevation Salt Marsh','Low Elevation Salt Marsh','Unclassified Salt Marsh','Seagrass','Tidal Flats')),
                         LC_AGG[LC_AGG$NAME==N,c("year_o","Habitat","area_o_o")],by="Habitat",all.x=T)
        LC_tab$year_o[is.na(LC_tab$year_o)] <- 1996
        LC_tab <- rbind(LC_tab,LC_tab[is.na(LC_tab$area_o_o),] %>%mutate(year_o=2016))
        LC_tab$area_o_o[is.na(LC_tab$area_o_o)] <- 0 
        LC_tab$area_o_o <- round(LC_tab$area_o_o/10000,2)
        LC_tab <- bind_rows(LC_tab, LC_tab %>% group_by(year_o) %>%
                              summarise(area_o_o=sum(area_o_o)) %>%
                              mutate(Habitat="All Focal"),
                            data.frame(Habitat='Total',year_o=c(1996,2016),area_o_o=as.numeric(st_area(ecotypes[ecotypes$NAME==N,]))/10000))
        LC_tab <- LC_tab %>%
          group_by(year_o) %>%
          mutate(PercentFocal = round(100*area_o_o/area_o_o[Habitat=='All Focal'],1),
                 PercentAll = round(100*area_o_o/area_o_o[Habitat=='Total'],1))
         LC_tab <- bind_rows(LC_tab %>% tidyr::pivot_wider(names_from = Habitat, values_from = area_o_o,id_cols=year_o) %>% mutate(Metric = 'Area (ha)'),
                             LC_tab %>% tidyr::pivot_wider(names_from = Habitat, values_from = PercentFocal,id_cols=year_o) %>% mutate(Metric = 'Percent Focal'),
                             LC_tab %>% tidyr::pivot_wider(names_from = Habitat, values_from = PercentAll,id_cols=year_o) %>% mutate(Metric = 'Percent All'))
         LC_tab <- LC_tab[,c("year_o","Metric","Total","All Focal","High Elevation Salt Marsh","Low Elevation Salt Marsh","Unclassified Salt Marsh","Seagrass","Tidal Flats")]
         colnames( LC_tab) <- c("year_o","Metric","Total","All Focal","High Elevation\nSalt Marsh","Low Elevation\nSalt Marsh","Unclassified\nSalt Marsh","Seagrass","Tidal Flats")
         LC_tab[,3:9] <- sapply(LC_tab[,3:9], function(x) round(x,1))
         ##  table of values
        table <- lulccAGG_NAME[lulccAGG_NAME ==N&lulccAGG_NAME$Class_o %in% c(18,18.1,18.2,19,23)&
                                 lulccAGG_NAME$year_o==1996&lulccAGG_NAME$year==2016,]
        table <- table[table$area_converted!=0,]
        if (nrow(table)==0){
          table <- data.frame(label="No Focal Habitat Cover Change from 1996-2016")
        }else{
          table <- merge(table,NOAACCAP[,c('Numeric','Habitat')],by.x='Class_o',by.y='Numeric',all.x=T)
          table <- table %>%
            mutate(Class_conv=ifelse(Class_conv %in% c(2,3,4,5),2,ifelse(!Class_conv %in% c(18,18.1,18.2,18.3,19,21,23,2:5,20,17),27,Class_conv))) %>%
            group_by(Class_o,Class_conv,Habitat) %>%
            summarise(area_converted=sum(area_converted),
                      area_o_o = unique(area_o_o))
          table <- rbind(merge(table,NOAACCAP[NOAACCAP$Numeric %in% c(2,18,18.1,18.2,18.3),c('Numeric','Habitat','Class')],by.x='Class_conv',by.y='Numeric',all.x=F),
                         merge(table,NOAACCAP[!NOAACCAP$Numeric %in% c(2,18,18.1,18.2,18.3),c('Numeric','Habitat','Class')],by.x='Class_conv',by.y='Numeric',all.x=F))
          table$Habitat.y[is.na(table$Habitat.y)] <- table$Class[is.na(table$Habitat.y)]
          
          
          table <- merge(table,ES_AGG[ES_AGG$NAME==N&ES_AGG$year_o==1996&ES_AGG$year==2016&ES_AGG$ES2=="All",
                                      c("Class_o","Class_conv","delES_o_area.mean","delES_o_conn.mean","delES_conv_area.mean","delES_conv_conn.mean")],
                         by=c("Class_o","Class_conv"))
          #table <- merge(table,ES_AGG[ES_AGG$NAME==N&ES_AGG$year_o==1996&ES_AGG$year==2016&ES_AGG$ES2=="All",c("Class_o","Class_conv","delES_conv_area.mean","delES_conv_conn.mean")],
          #               by=c("Class_o","Class_conv"))
          table$Perc_converted = round(100*table$area_converted/table$area_o_o,2)
          table$area_converted <- round(table$area_converted/10000,1)
          table$ES_o_96_f_mean <- as.numeric(unique(ES_AGG2[ES_AGG2$year_o==1996&ES_AGG2$year==2016&ES_AGG2$ES2=="All"&ES_AGG2$NAME==N,c("ES_o_96_f_mean")]))
          table$perc_f_o_96 <- (table$delES_o_area.mean+table$delES_o_conn.mean)/table$ES_o_96_f_mean
          table$perc_f_conv_96 <- (table$delES_conv_area.mean+table$delES_conv_conn.mean)/table$ES_o_96_f_mean
          table$perc_f_conv_96 <- round(table$perc_f_conv_96*100,1)
          table$perc_f_o_96 <- round(table$perc_f_o_96*100,1)
          table <- table %>%
            mutate(Net_contribution_to_focal_ES =if_else(Class_o %in% c(18,18.1,18.2,19,23),
                                                         if_else(Class_conv %in% c(18,18.1,18.2,19,23),perc_f_o_96+perc_f_conv_96,perc_f_o_96),perc_f_conv_96))
          table <- table[,c("Habitat.x","Habitat.y","area_converted","perc_f_o_96","perc_f_conv_96","Net_contribution_to_focal_ES")]
          #table$area_o_o <- round(table$area_o_o/10000,1)
          colnames(table) <- c("Focal\nHabitat","Conversion\nHabitat","Area\nConverted\n1996-2016\n(ha)",
                               "% Original Habitat\nService Change\nRelative to all 1996\nFocal Services","% Conversion Habitat\nService Change\nRelative to all 1996\nFocal Services",
                               "Net % Change in\n Focal Services\nRelative to 1996")
          table <- table[order(table$`Area\nConverted\n1996-2016\n(ha`),]
          table <- table[table$`Area\nConverted\n1996-2016\n(ha)`!=0,]
          table <- table [!duplicated(t(apply(table[,1:2] , 1, sort))), ]
          table$`Focal\nHabitat` <- gsub("Elevation","Elev.",table$`Focal\nHabitat`)
          table$`Conversion\nHabitat` <- gsub("Elevation","Elev.",table$`Conversion\nHabitat`)
          table$`Conversion\nHabitat` <- gsub("Estuarine","Estu.",table$`Conversion\nHabitat`)
        }
       
        
        cols <- NOAACCAP[NOAACCAP$Habitat %in% unique(ES_AGG2$Habitat[ES_AGG2$NAME==N]),]
        cols <- cols$cols2[match(levels(droplevels(ES_AGG2$Habitat[ES_AGG2$NAME==N])),cols$Habitat)]
        g1 <- ggplot(data=ES_AGG2 %>%filter(NAME==N,!(year_o==1996&year %in% c(2006,2010,2016)),ES2!="Area Only"),
                     aes(x=year,y=100*perc_totES_converted.mean))+
          #facet_grid(~ES2,scales = 'free_y') +
          facet_wrap(.~ES2,scales = 'free_y',ncol=7)+
          geom_bar(stat='identity',col='black',lwd=0.1,aes(fill=Habitat))+
          geom_errorbar(data=ES_AGG2%>%filter(NAME==N,!(year_o==1996&year %in% c(2006,2010,2016)),ES2!="Area Only"),
                        position=position_dodge(width=5),
                        aes(ymin=100*perc_totES_converted.min_cum.pos, 
                            ymax=100*perc_totES_converted.max_cum.pos),
                        color="gray30",stat='identity',width=0.5)+
          geom_errorbar(data=ES_AGG2%>%filter(NAME==N,!(year_o==1996&year %in% c(2006,2010,2016)),ES2!="Area Only"),
                        position=position_dodge(width=5),
                        aes(ymin=100*perc_totES_converted.min_cum.neg, 
                            ymax=100*perc_totES_converted.max_cum.neg),
                        color="gray30",stat='identity',width=0.5)+
          geom_line(data=ES_AGG2[ES_AGG2$year_o==1996&ES_AGG2$NAME==N,]%>% filter(ES2!="Area Only"),
                    aes(x=year,y=round(perc_ES_96_f.mean*100,1)),col='Black',size=1,lty=2)+
          geom_ribbon(data=ES_AGG2[ES_AGG2$year_o==1996&ES_AGG2$NAME==N,]%>% filter(ES2!="Area Only"),
                      aes(x=year,ymin=round(perc_ES_96_f.min*100,1),ymax=round(perc_ES_96_f.max*100,1)),col='gray30',alpha=0.2)+
          scale_fill_manual(values=cols)+          
          scale_y_continuous(labels=function(x) sprintf("%.0f", x))+
          ylab('% Change in 1996 Focal Habitat \nService Provisioning')+
          scale_x_continuous(breaks=c(1996,2001,2006,2010,2016))+
          guides(fill=guide_legend(title=NULL,override.aes = list(size = 1)))+
          theme_classic()+
          geom_hline(yintercept=0,lty=3)+
          theme(axis.text.x = element_text(angle = 90),legend.position='bottom',legend.box.margin = margin(0,0,0,0),legend.margin = margin(0,0,0,0),
                strip.placement = "outside",panel.border = element_rect(colour = "black", fill=NA, size=1),plot.margin = margin(0,0,0,0),
                axis.line.x = element_blank(),axis.line.y = element_blank(),legend.text = element_text(size=9),
                axis.title.y=element_text(size=10),strip.background = element_blank(),axis.title.x=element_blank(),
                axis.text = element_text(size=10),strip.text= element_text(size=9))
        ggsave(paste0(tmp,"g.tiff"),g1,width=10,height=3.25)
  
        
        cols <- NOAACCAP[NOAACCAP$Habitat %in% gsub("\n"," ",unique(ES_AGG_LC$Habitat[ES_AGG_LC$year==1996&ES_AGG_LC$NAME==N])),]
        cols <- cols$cols2[match(gsub("\n"," ",levels(droplevels(ES_AGG_LC$Habitat[ES_AGG_LC$year==1996&ES_AGG_LC$NAME==N]))),cols$Habitat)]
        gES96 <- ggplot(ES_AGG_LC[ES_AGG_LC$year==1996&ES_AGG_LC$NAME==N,],aes(x=year,y=round(ES_perc_mean*100),fill=Habitat)) +
          facet_grid(~ES2) +
          geom_col(col='Black')+
          #error bars are negligible and not worth showing in the plot
          #geom_errorbar(aes(ymin=100*ES_perc_min_cum,ymax=100*ES_perc_max_cum),width=0.05)+
          scale_fill_manual(values=cols)+
          scale_y_continuous(breaks=c(0,50,100),expand = c(0, 0))+
          ylab("% of Focal Habitat\nService Provisioning")+
          theme_classic()+
          guides(fill=guide_legend(title=NULL,override.aes = list(size = 8)))+
          theme(axis.text.x = element_blank(),legend.position='bottom',axis.title.x=element_blank(),axis.title.y=element_text(size=14),
                axis.ticks = element_blank(),strip.placement = "outside",panel.border = element_blank(),#element_rect(colour = "black", fill=NA, size=1),
                axis.line.x = element_blank(),axis.line.y = element_blank(),strip.text.x = element_text(size=13),
                strip.background = element_blank(),axis.text = element_text(size=14),legend.text = element_text(size=14))

        gES96b <-ggplot(LC_AGG[LC_AGG$year_o==1996&!is.na(LC_AGG$Habitat)&LC_AGG$NAME==N,],aes(x=year_o,y=PercentF,fill=Habitat))+
          geom_col(col='Black')+
          #error bars are negligible and not worth showing in the plot
          #geom_errorbar(aes(ymin=100*ES_perc_min_cum,ymax=100*ES_perc_max_cum),width=0.05)+
          scale_fill_manual(values=cols)+
          scale_y_continuous(breaks=c(0,50,100))+
          ylab("% of Focal Habitat Area")+
          theme_classic()+
          guides(fill="none")+
          theme(axis.text.x = element_blank(),legend.position='bottom',axis.title.x=element_blank(),axis.title.y=element_text(size=14),
                axis.ticks = element_blank(),strip.placement = "outside",panel.border = element_blank(),#element_rect(colour = "black", fill=NA, size=1),
                axis.line.x = element_blank(),axis.line.y = element_blank(),
                strip.background = element_blank(),axis.text = element_text(size=14))
        ggsave(paste0(tmp,"gES96.tiff"),gES96,width=11)
        ggsave(paste0(tmp,"gES96b.tiff"),gES96b,width=1.75)
        
        
        cols <- NOAACCAP[NOAACCAP$Habitat %in% gsub("\n"," ",unique(ES_AGG_LC$Habitat[ES_AGG_LC$year==2016&ES_AGG_LC$NAME==N])),]
        cols <- cols$cols2[match(gsub("\n"," ",levels(droplevels(ES_AGG_LC$Habitat[ES_AGG_LC$year==2016&ES_AGG_LC$NAME==N]))),cols$Habitat)]
        gES16 <- ggplot(ES_AGG_LC[ES_AGG_LC$year==2016&ES_AGG_LC$NAME==N,],aes(x=year,y=round(ES_perc_mean*100),fill=Habitat)) +
          facet_grid(~ES2) +
          geom_col(col='Black')+
          #error bars are negligible and not worth showing in the plot
          #geom_errorbar(aes(ymin=100*ES_perc_min_cum,ymax=100*ES_perc_max_cum),width=0.05)+
          scale_fill_manual(values=cols)+
          scale_y_continuous(breaks=c(0,50,100),expand = c(0, 0))+
          ylab("% of Focal Habitat\nService Provisioning")+
          theme_classic()+
          guides(fill=guide_legend(title=NULL,override.aes = list(size = 8)))+
          theme(axis.text.x = element_blank(),legend.position='bottom',axis.title.x=element_blank(),axis.title.y=element_text(size=14),
                axis.ticks = element_blank(),strip.placement = "outside",panel.border = element_blank(),#element_rect(colour = "black", fill=NA, size=1),
                axis.line.x = element_blank(),axis.line.y = element_blank(),strip.text.x = element_text(size=13),
                strip.background = element_blank(),axis.text = element_text(size=14),legend.text = element_text(size=14))
        gES16b <-ggplot(LC_AGG[LC_AGG$year_o==2016&!is.na(LC_AGG$Habitat)&LC_AGG$NAME==N,],aes(x=year_o,y=PercentF,fill=Habitat))+
          geom_col(col='Black')+
          #error bars are negligible and not worth showing in the plot
          #geom_errorbar(aes(ymin=100*ES_perc_min_cum,ymax=100*ES_perc_max_cum),width=0.05)+
          scale_fill_manual(values=cols)+
          scale_y_continuous(breaks=c(0,50,100))+
          ylab("% of Focal Habitat Area")+
          theme_classic()+
          guides(fill="none")+
          theme(axis.text.x = element_blank(),legend.position='bottom',axis.title.x=element_blank(),axis.title.y=element_text(size=14),
                axis.ticks = element_blank(),strip.placement = "outside",panel.border = element_blank(),#element_rect(colour = "black", fill=NA, size=1),
                axis.line.x = element_blank(),axis.line.y = element_blank(),
                strip.background = element_blank(),axis.text = element_text(size=14))
        ggsave(paste0(tmp,"gES16.tiff"),gES16,width=11)
        ggsave(paste0(tmp,"gES16b.tiff"),gES16b,width=1.75)
        
        pf <- flextable(table)
        pfLC96 <- flextable(LC_tab[LC_tab$year_o==1996,-1])
        pfLC16 <- flextable(LC_tab[LC_tab$year_o==2016,-1])
        pf <- fontsize(pf, size = 10, part = "all")
        pfLC96 <- fontsize(pfLC96, size = 10, part = "all")
        pfLC16 <- fontsize(pfLC16, size = 10, part = "all")
        pf <- padding(pf, padding.left =0,padding.right =0,padding.top=0,padding.bottom=0, part = "all")
        pfLC96 <- padding(pfLC96, padding.left =2,padding.top=0,padding.bottom=0, part = "all")
        pfLC16 <- padding(pfLC16, padding.left =2,padding.top=0,padding.bottom=0, part = "all")
        pf <- valign(pf, valign = "bottom", part = "all")
        #pf <- align(pf,j=c(2,4:8),align="center",part="all")
        pf <- autofit(pf)
        pfLC96 <- autofit(pfLC96)
        pfLC16 <- autofit(pfLC16)
        pf <- height(pf, height = .05)
        pfLC96 <- height(pfLC96, height = .05)
        pfLC16 <- height(pfLC16, height = .05)
        
        ggsave(paste0(tmp,"gloc.tiff"),g_loc,dpi=200)#,width=6.25)
        knitr::plot_crop(paste0(tmp,"gloc.tiff"))
        dimloc <- dim(tiff::readTIFF(paste0(tmp,"gloc.tiff")))
        ggsave(paste0(tmp,"g96.tiff"),g96,dpi=200)
        knitr::plot_crop(paste0(tmp,"g96.tiff"))
        dim96 <- dim(tiff::readTIFF(paste0(tmp,"g96.tiff")))
        ##resize the plots to make sure they fit in the required space
        ggsave(paste0(tmp,"g16.tiff"),g16,dpi=200)#,width=6.25)
        knitr::plot_crop(paste0(tmp,"g16.tiff"))
        maxW=8.25
        maxH=6.0
        Exp = maxW/(dim96[2]/200)
        if ((dim96[1]/200)*Exp<maxH){
          W96 <- W16 <- maxW
          H96 <- H16 <- dim96[1]*Exp/200 
        }else {
          Exp = maxH/(dim96[1]/200)
          W96 <- W16 <- dim96[2]*Exp/200 
          H96 <- H16 <- maxH
        }
        
        ggsave(paste0(tmp,"gdel.tiff"),gdel,dpi=200)#,width=7.25)
        knitr::plot_crop(paste0(tmp,"gdel.tiff"))
        dimdel <- dim(tiff::readTIFF(paste0(tmp,"gdel.tiff")))
        Exp = maxW/(dimdel[2]/200)
        if ((dimdel[1]/200)*Exp<maxH){
          Wdel <- maxW
          Hdel <- dimdel[1]*Exp/200 
        }else {
          Exp = maxH/(dimdel[1]/200)
          Wdel <- dimdel[2]*Exp/200 
          Hdel <- maxH
        }

     
        P %>%
          add_slide(layout="Titel und Inhalt",master="Office") %>%
          ph_with(fpar(ftext(paste0(CapStr(N),"  -   1996"), fp_text(font.size = 14, bold = TRUE))),location=ph_location(left=0.55, top=0.25,height=0.25,width=8)) %>%
          ph_with(external_img(paste0(tmp,"g96.tiff")),location=ph_location(left=0.15, top=0.5, width=W96,height=H96)) %>%
          ph_with(external_img(paste0(tmp,"gloc.tiff")),location=ph_location(left=0.15, top=0.5, width=1.5,height=1.5*dimloc[1]/dimloc[2])) %>%
          ph_with(external_img("\\\\ad.uni-hamburg.de\\redir\\redir0010\\bba4705\\Documents\\MassBays\\Tables and Figures\\Maps\\legend.tif"),
                  location=ph_location(left=0.15, top=6.5, width=7.5,height=1)) %>%
          ph_with(external_img(paste0(tmp,"gES96.tiff")),location=ph_location(left=1.65, top=7.5, width=6.65,height=2.3)) %>%
          ph_with(external_img(paste0(tmp,"gES96b.tiff")),location=ph_location(left=0.2, top=7.75, width=1.25,height=1.8)) %>%
          ph_with(pfLC96, location=ph_location(left=.75, top=9.8,width=3,height=3)) %>%
          
          add_slide(layout="Titel und Inhalt",master="Office") %>%
          ph_with(fpar(ftext(paste0(CapStr(N),"  -   2016"), fp_text(font.size = 14, bold = TRUE))),location=ph_location(left=0.55, top=0.25,height=0.25,width=8)) %>%
          ph_with(external_img(paste0(tmp,"g16.tiff")),location=ph_location(left=0.15, top=0.5,width=W16,height=H16)) %>%
          ph_with(external_img(paste0(tmp,"gloc.tiff")),location=ph_location(left=0.15, top=0.5, width=1.5,height=1.5*dimloc[1]/dimloc[2])) %>%
          ph_with(external_img("\\\\ad.uni-hamburg.de\\redir\\redir0010\\bba4705\\Documents\\MassBays\\Tables and Figures\\Maps\\legend.tif"),
                  location=ph_location(left=0.15, top=6.5, width=7.5,height=1)) %>%
          ph_with(external_img(paste0(tmp,"gES16.tiff")),location=ph_location(left=1.65, top=7.5, width=6.65,height=2.3)) %>%
          ph_with(external_img(paste0(tmp,"gES16b.tiff")),location=ph_location(left=0.2, top=7.75, width=1.25,height=1.8)) %>%
          ph_with(pfLC16, location=ph_location(left=.75, top=9.8,width=3,height=3)) %>%
          
          add_slide(layout="Titel und Inhalt",master="Office") %>%
          ph_with(fpar(ftext(paste0(CapStr(N),"  -   Change 1996-2016"), fp_text(font.size = 14, bold = TRUE))),location=ph_location(left=0.55, top=0.25,height=0.25,width=8)) %>%
          ph_with(external_img(paste0(tmp,"gdel.tiff")),location=ph_location(left=0.15, top=0.5,width=Wdel,height=Hdel)) %>%
          ph_with(external_img(paste0(tmp,"gloc.tiff")),location=ph_location(left=0.15, top=0.5, width=1.5,height=1.5*dimloc[1]/dimloc[2])) %>%
          ph_with(external_img(paste0(tmp,"g.tiff")),location=ph_location(left=0.2, top=6, width=8,height=2.75)) %>%
          ph_with(pf, location=ph_location(left=0.25, top=8.8,width=9.5,height=2)) %>%
          print(target = paste0("\\\\ad.uni-hamburg.de\\redir\\redir0010\\bba4705\\Documents/MassBays/Tables and Figures/Maps/Maps_Embayments/All Embayments.pptx"))
        
          #print(target =  "\\\\ad.uni-hamburg.de\\redir\\redir0010\\bba4705\\Documents/MassBays/Tables and Figures/Maps/Maps_Embayments/AllEmbayments.pptx")
         
        #read_docx("\\\\ad.uni-hamburg.de\\redir\\redir0010\\bba4705\\Documents\\MassBays\\Tables and Figures\\Maps\\Maps_Embayments\\template.docx") %>%
         # body_add_img(external_img(paste0(tmp,"gall.tiff")), style = NULL, height=4,width=7.5, pos = "after") %>%
         # body_add_par("\t\t\t\t\t\t") %>%
         # body_add_img(external_img(paste0(tmp,"g.tiff")), style = NULL, width=7.5, height=3.5, pos = "after") %>%
          #print(target = paste0("\\\\ad.uni-hamburg.de\\redir\\redir0010\\bba4705\\Documents/MassBays/Tables and Figures/Maps/Maps_Embayments/Maps_",gsub("/","-",N),".docx"))
        #y=y+2.5
      }
      Sys.sleep(0.1); setTxtProgressBar(pb,which(names==N))
  }


ACCAP$Class <- factor(NOAACCAP$Class,levels=NOAACCAP$Class[order(NOAACCAP$Numeric)])
glegend <- ggplot(NOAACCAP[!NOAACCAP$Numeric %in% c(0,1,18.3,18.4,24:28),],aes(Numeric,fill=Class))+
  geom_bar() +
  scale_fill_manual(values=NOAACCAP$cols[!NOAACCAP$Numeric %in% c(0,1,18.3,18.4,24:28)])+
  guides(fill=guide_legend(ncol=4))+
  theme(legend.position="bottom",legend.key.size = unit(0.8,"line"))

read_pptx("\\\\ad.uni-hamburg.de\\redir\\redir0010\\bba4705\\Documents\\Template.pptx") %>%
  ph_with(dml(ggobj=glegend),location=ph_location(left=0.5, top=0.5,width=9,height=8)) %>%
  print(target = "./Tables and Figures/Maps/legend.pptx")

